import dataclasses
import dis
import types
from .bytecode_analysis import get_indexof as get_indexof, propagate_line_nums as propagate_line_nums, remove_extra_line_nums as remove_extra_line_nums, stacksize_analysis as stacksize_analysis
from .utils import is_safe_constant as is_safe_constant
from _typeshed import Incomplete
from typing import Any, Callable, Iterator

@dataclasses.dataclass
class InstructionExnTabEntry:
    start: Instruction
    end: Instruction
    target: Instruction
    depth: int
    lasti: bool
    def __eq__(self, o) -> bool: ...

@dataclasses.dataclass
class Instruction:
    opcode: int
    opname: str
    arg: int | None
    argval: Any
    offset: int | None = ...
    starts_line: int | None = ...
    is_jump_target: bool = ...
    positions: dis.Positions | None = ...
    target: Instruction | None = ...
    exn_tab_entry: InstructionExnTabEntry | None = ...
    def __hash__(self) -> int: ...
    def __eq__(self, other) -> bool: ...
    def short_inst_repr(self) -> str: ...

def convert_instruction(i: dis.Instruction) -> Instruction: ...

class _NotProvided: ...

def inst_has_op_bits(name): ...
def create_instruction(name, *, arg: Incomplete | None = None, argval=..., target: Incomplete | None = None) -> Instruction: ...
def create_jump_absolute(target) -> Instruction: ...
def create_load_const(val, checked: bool = True) -> Instruction: ...
def create_dup_top() -> Instruction: ...
def create_rot_n(n) -> list[Instruction]: ...
def add_push_null(inst_or_insts: Instruction | list[Instruction]) -> list[Instruction]: ...
def add_push_null_call_function_ex(inst_or_insts: Instruction | list[Instruction]) -> list[Instruction]: ...
def create_call_function(nargs, push_null) -> list[Instruction]: ...
def create_call_method(nargs) -> list[Instruction]: ...
def create_load_method(name) -> Instruction: ...
def create_setup_with(target) -> Instruction: ...
def create_swap(n) -> list[Instruction]: ...
def lnotab_writer(lineno: int, byteno: int = 0) -> tuple[list[int], Callable[[int, int], None]]: ...
def linetable_310_writer(first_lineno): ...
def encode_varint(n: int) -> list[int]: ...
def linetable_311_writer(first_lineno: int): ...

@dataclasses.dataclass
class ExceptionTableEntry:
    start: int
    end: int
    target: int
    depth: int
    lasti: bool

def encode_exception_table_varint(n: int) -> list[int]: ...
def decode_exception_table_varint(bytes_iter: Iterator[int]) -> int: ...
def check_exception_table(tab: list[ExceptionTableEntry]) -> None: ...
def parse_exception_table(exntab: bytes) -> list[ExceptionTableEntry]: ...
def assemble_exception_table(tab: list[ExceptionTableEntry]) -> bytes: ...
def assemble(instructions: list[Instruction], firstlineno: int) -> tuple[bytes, bytes]: ...
def virtualize_jumps(instructions) -> None: ...
def flip_jump_direction(instruction: Instruction) -> None: ...
def devirtualize_jumps(instructions) -> None: ...
def virtualize_exception_table(exn_tab_bytes: bytes, instructions: list[Instruction]): ...
def compute_exception_table(instructions: list[Instruction]) -> list[ExceptionTableEntry]: ...
def check_inst_exn_tab_entries_nested(tab: list[InstructionExnTabEntry], indexof) -> None: ...
def propagate_inst_exn_table_entries(instructions: list[Instruction]) -> None: ...
def check_inst_exn_tab_entries_valid(instructions: list[Instruction]): ...
def strip_extended_args(instructions: list[Instruction]) -> None: ...
def overwrite_instruction(old_inst, new_insts): ...
def remove_load_call_method(instructions: list[Instruction]) -> list[Instruction]: ...
def remove_jump_if_none(instructions: list[Instruction]) -> None: ...
def remove_binary_store_slice(instructions: list[Instruction]) -> None: ...

FUSED_INSTS: Incomplete

def remove_fused_load_store(instructions: list[Instruction]) -> None: ...
def explicit_super(code: types.CodeType, instructions: list[Instruction]) -> None: ...
def fix_extended_args(instructions: list[Instruction]) -> int: ...
def instruction_size(inst) -> int: ...
def check_offsets(instructions) -> None: ...
def update_offsets(instructions) -> None: ...
def debug_bytes(*args) -> str: ...
def debug_checks(code) -> None: ...

HAS_LOCAL: Incomplete
HAS_NAME: Incomplete
HAS_FREE: Incomplete
HAS_CONST: Incomplete

def get_const_index(code_options, val) -> int: ...
def fix_vars(instructions: list[Instruction], code_options, varname_from_oparg: Incomplete | None = None): ...
def clear_instruction_args(instructions) -> None: ...
def get_code_keys() -> list[str]: ...
def transform_code_object(code, transformations, safe: bool = False) -> types.CodeType: ...
def clean_and_assemble_instructions(instructions: list[Instruction], keys: list[str], code_options: dict[str, Any]) -> tuple[list[Instruction], types.CodeType]: ...
def populate_kw_names_argval(instructions, consts) -> None: ...
def cleaned_instructions(code, safe: bool = False) -> list[Instruction]: ...
def unique_id(name) -> str: ...
def is_generator(code: types.CodeType) -> bool: ...
def bytecode_from_template(fn, varname_map: Incomplete | None = None, noreturn: bool = True, noprefix: bool = True): ...
