import dataclasses
import torch.nn
import types
from . import utils as utils
from .bytecode_transformation import Instruction as Instruction, add_push_null as add_push_null, add_push_null_call_function_ex as add_push_null_call_function_ex, create_call_function as create_call_function, create_call_method as create_call_method, create_dup_top as create_dup_top, create_instruction as create_instruction, create_load_const as create_load_const, create_load_method as create_load_method, create_rot_n as create_rot_n
from .exc import unimplemented as unimplemented
from .source import AttrSource as AttrSource, Source as Source
from .utils import is_safe_constant as is_safe_constant, rot_n_helper as rot_n_helper
from .variables.base import ValueMutationExisting as ValueMutationExisting, VariableTracker as VariableTracker
from .variables.nn_module import NNModuleVariable as NNModuleVariable
from .variables.tensor import NumpyNdarrayVariable as NumpyNdarrayVariable, SymNodeVariable as SymNodeVariable, TensorVariable as TensorVariable, UnspecializedPythonVariable as UnspecializedPythonVariable
from .variables.torch_function import TensorWithTFOverrideVariable as TensorWithTFOverrideVariable
from _typeshed import Incomplete
from typing import Counter

@dataclasses.dataclass
class GraphOutputEntry:
    index: int
    variable: VariableTracker

class PyCodegen:
    root: Incomplete
    top_of_stack: VariableTracker | None
    uses: Counter[VariableTracker]
    graph_outputs: dict[int, GraphOutputEntry]
    tempvars: Incomplete
    tx: Incomplete
    graph_output_var: Incomplete
    code_options: Incomplete
    cell_and_freevars: Incomplete
    new_var: Incomplete
    value_from_source: bool
    overridden_sources: dict[Source, Source]
    def __init__(self, tx: Incomplete | None = None, root: torch.nn.Module | None = None, graph_output_var: str | None = None, tempvars: Incomplete | None = None, overridden_sources: Incomplete | None = None) -> None: ...
    def restore_stack(self, stack_values, *, value_from_source: bool = True) -> None: ...
    def graph_output_vars(self): ...
    def call_reconstruct(self, value) -> None: ...
    def add_push_null(self, gen_fn, call_function_ex: bool = False) -> None: ...
    def __call__(self, value, allow_cache: bool = True): ...
    def add_graph_output(self, value): ...
    def load_graph_output(self, index) -> None: ...
    def add_cache(self, value) -> None: ...
    def foreach(self, items) -> None: ...
    def setup_globally_cached(self, name, value): ...
    def clear_tos(self) -> None: ...
    def append_output(self, inst) -> None: ...
    def extend_output(self, insts) -> None: ...
    def get_instructions(self) -> list[Instruction]: ...
    def create_load(self, name) -> Instruction: ...
    def create_load_closure(self, name) -> Instruction: ...
    def create_load_deref(self, name) -> Instruction: ...
    def create_store(self, name) -> Instruction: ...
    def create_store_deref(self, name) -> Instruction: ...
    def create_load_global(self, name, add: bool = False) -> Instruction: ...
    def create_load_const(self, value) -> Instruction: ...
    def create_load_const_unchecked(self, value) -> Instruction: ...
    def load_method(self, name) -> None: ...
    def call_method(self, nargs) -> None: ...
    def create_load_attr(self, name) -> Instruction: ...
    def load_attr(self, name) -> None: ...
    def create_load_attrs(self, names): ...
    def create_store_attr(self, name) -> Instruction: ...
    def store_attr(self, name) -> None: ...
    def load_function_name(self, fn_name, push_null, num_on_stack: int = 0): ...
    def rot_n(self, n): ...
    def pop_null(self): ...
    def pop_top(self) -> None: ...
    def call_function(self, nargs: int, push_null: bool): ...
    def dup_top(self) -> None: ...
    def store(self, varname) -> None: ...
    def load_deref(self, varname) -> None: ...
    def make_function_with_closure(self, fn_name: str, code: types.CodeType, push_null: bool, num_on_stack: int = 0): ...
    def create_load_python_module(self, mod) -> Instruction: ...
    def make_call_generated_code(self, fn_name: str) -> None: ...
    def load_import_from(self, module_name, object_name) -> None: ...
    def create_call_function_kw(self, nargs, kw_names, push_null) -> list[Instruction]: ...
    def create_delete(self, value) -> Instruction: ...
