import torch
import typing
from . import config as config, exc as exc, trace_rules as trace_rules
from .backends.registry import CompilerFn as CompilerFn
from .bytecode_analysis import remove_dead_code as remove_dead_code, remove_pointless_jumps as remove_pointless_jumps
from .bytecode_transformation import Instruction as Instruction, check_inst_exn_tab_entries_valid as check_inst_exn_tab_entries_valid, is_generator as is_generator, propagate_inst_exn_table_entries as propagate_inst_exn_table_entries, transform_code_object as transform_code_object
from .cache_size import CacheSizeRelevantForFrame as CacheSizeRelevantForFrame, compute_cache_size as compute_cache_size, exceeds_cache_size_limit as exceeds_cache_size_limit, is_recompilation as is_recompilation
from .eval_frame import TorchPatcher as TorchPatcher, always_optimize_code_objects as always_optimize_code_objects, dynamo_tls as dynamo_tls, skip_code as skip_code
from .exc import BackendCompilerFailed as BackendCompilerFailed, FailOnRecompileLimitHit as FailOnRecompileLimitHit, InternalTorchDynamoError as InternalTorchDynamoError, RecompileLimitExceeded as RecompileLimitExceeded, SkipCodeRecursiveException as SkipCodeRecursiveException, TorchRuntimeError as TorchRuntimeError, UncapturedHigherOrderOpError as UncapturedHigherOrderOpError, Unsupported as Unsupported, augment_exc_message as augment_exc_message, format_error_msg as format_error_msg, unimplemented as unimplemented
from .guards import CheckFunctionManager as CheckFunctionManager, GuardedCode as GuardedCode, get_and_maybe_log_recompilation_reason as get_and_maybe_log_recompilation_reason
from .hooks import Hooks as Hooks
from .output_graph import OutputGraph as OutputGraph
from .pgo import put_code_state as put_code_state
from .replay_record import ExecutionRecord as ExecutionRecord
from .repro.after_dynamo import WrapBackendDebug as WrapBackendDebug
from .resume_execution import TORCH_DYNAMO_RESUME_IN_PREFIX as TORCH_DYNAMO_RESUME_IN_PREFIX
from .symbolic_convert import DistributedState as DistributedState, InstructionTranslator as InstructionTranslator, LocalState as LocalState, SpeculationLog as SpeculationLog
from .trace_rules import is_numpy as is_numpy
from .types import BytecodeHook as BytecodeHook, CacheEntry as CacheEntry, DynamoFrameType as DynamoFrameType
from .utils import CleanupManager as CleanupManager, CompileTimeInstructionCounter as CompileTimeInstructionCounter, LazyString as LazyString, chromium_event_timed as chromium_event_timed, counters as counters, dynamo_timed as dynamo_timed, format_bytecode as format_bytecode, gen_record_file_name as gen_record_file_name, get_metrics_context as get_metrics_context, increment_frame as increment_frame, is_namedtuple as is_namedtuple, istype as istype, orig_code_map as orig_code_map, reset_graph_break_dup_checker as reset_graph_break_dup_checker, setup_compile_debug as setup_compile_debug, to_int_us as to_int_us, troubleshooting_url as troubleshooting_url, write_record_to_file as write_record_to_file
from .variables.builder import FrameStateSizeEntry as FrameStateSizeEntry
from .variables.torch_function import torch_function_mode_stack_state_mgr as torch_function_mode_stack_state_mgr
from _typeshed import Incomplete
from torch._C._dynamo.guards import GlobalStateGuard as GlobalStateGuard
from torch._dynamo.distributed import get_compile_pg as get_compile_pg
from torch._dynamo.symbolic_convert import TensorifyState as TensorifyState
from torch._guards import CompileContext as CompileContext, CompileId as CompileId, compile_context as compile_context, tracing as tracing
from torch._logging import structured as structured
from torch._utils_internal import compile_time_strobelight_meta as compile_time_strobelight_meta, justknobs_check as justknobs_check, maybe_upload_prof_stats_to_manifold as maybe_upload_prof_stats_to_manifold, signpost_event as signpost_event
from torch.fx.experimental.symbolic_shapes import ConstraintViolationError as ConstraintViolationError, GuardOnDataDependentSymNode as GuardOnDataDependentSymNode
from torch.nn.parallel.distributed import DistributedDataParallel as DistributedDataParallel
from torch.utils._python_dispatch import is_in_torch_dispatch_mode as is_in_torch_dispatch_mode
from torch.utils._traceback import CapturedTraceback as CapturedTraceback, format_traceback_short as format_traceback_short
from torch.utils.hooks import RemovableHandle as RemovableHandle
from types import CodeType, FunctionType, ModuleType
from typing import Any, Callable
from weakref import ReferenceType

np: ModuleType | None
log: Incomplete
bytecode_log: Incomplete
graph_break_log: Incomplete
compile_lock: Incomplete

class TODO_UNKNOWN: ...

class Tracker:
    seen: list[ReferenceType[CodeType]]
    seen_ids: set[int]
    def __init__(self) -> None: ...
    def add(self, strong_obj: CodeType) -> None: ...
    def __contains__(self, item: CodeType) -> bool: ...
    def clear(self) -> None: ...

input_codes: Incomplete
output_codes: Incomplete
initial_global_state: GlobalStateGuard | None

def fx_forward_from_src_skip_result(src: str, globals: dict[str, Any], co_fields: dict[str, str] | None = None) -> FunctionType: ...
def preserve_global_state(fn: Callable[_P, _T]) -> Callable[_P, _T]: ...
def has_tensor_in_frame(frame: DynamoFrameType) -> bool: ...
def exception_handler(e: Exception, code: CodeType, frame: DynamoFrameType | None = None, export: bool = False) -> None: ...

FRAME_COUNTER: int
FRAME_COMPILE_COUNTER: typing.Counter[int | FrameStateSizeEntry]

def maybe_cprofile(func: Callable[_P, _T]) -> Callable[_P, _T]: ...
def cprofile_wrapper(func: Callable[_P, _T]) -> Callable[_P, _T]: ...

class ConvertFrameAssert:
    def __init__(self, compiler_fn: CompilerFn, one_graph: bool = True, export: bool = False, export_constraints: typing.Never | None = None) -> None: ...
    def __call__(self, frame: DynamoFrameType, cache_entry: CacheEntry | None, hooks: Hooks, frame_state: dict[str, int | FrameStateSizeEntry], *, skip: int = 0) -> GuardedCode | None: ...

def convert_frame_assert(compiler_fn: CompilerFn, one_graph: bool = True, export: bool = False, export_constraints: typing.Never | None = None) -> ConvertFrameAssert: ...
def register_bytecode_hook(hook: BytecodeHook) -> RemovableHandle: ...

class ConvertFrame:
    def __init__(self, compiler_fn: CompilerFn, hooks: Hooks) -> None: ...
    def __call__(self, frame: DynamoFrameType, cache_entry: CacheEntry | None, hooks: Hooks, frame_state: dict[str, int | FrameStateSizeEntry], skip: int = 0) -> GuardedCode | torch._C._dynamo.eval_frame.SkipCodeRecursiveFlag | torch._C._dynamo.eval_frame.CacheLimitHitFlag | None: ...

def convert_frame(compiler_fn: CompilerFn, hooks: Hooks) -> ConvertFrame: ...
def replay(filename: str) -> None: ...
def first_real_inst_idx(code: CodeType) -> int: ...

class ConvertFrameProtocol(typing.Protocol):
    def __call__(self, frame: DynamoFrameType, cache_entry: CacheEntry | None, hooks: Hooks, frame_state: dict[str, int | FrameStateSizeEntry], *, skip: int = 0) -> GuardedCode | None: ...

class CatchErrorsWrapper:
    hooks: Incomplete
    def __init__(self, callback: ConvertFrameProtocol, hooks: Hooks) -> None: ...
    def __call__(self, frame: DynamoFrameType, cache_entry: CacheEntry | None, frame_state: dict[str, int | FrameStateSizeEntry]) -> GuardedCode | None: ...

def catch_errors_wrapper(callback: ConvertFrameProtocol, hooks: Hooks) -> CatchErrorsWrapper: ...
