import torch
from _typeshed import Incomplete
from dataclasses import dataclass
from typing import Any, Callable, Iterable

get_cuda_stream: Callable[[int], int] | None
caching_worker_device_properties: dict[str, Any]
caching_worker_current_devices: dict[str, int]

class DeviceInterface:
    class device:
        def __new__(cls, device: _device_t): ...
    class Event:
        def __new__(cls, *args, **kwargs) -> None: ...
    class Stream:
        def __new__(cls, *args, **kwargs) -> None: ...
    class Worker:
        @staticmethod
        def set_device(device: int): ...
        @staticmethod
        def current_device() -> int: ...
        @staticmethod
        def get_device_properties(device: _device_t = None): ...
    @staticmethod
    def current_device() -> None: ...
    @staticmethod
    def set_device(device: _device_t): ...
    @staticmethod
    def maybe_exchange_device(device: int) -> int: ...
    @staticmethod
    def exchange_device(device: int) -> int: ...
    @staticmethod
    def device_count() -> None: ...
    @staticmethod
    def is_available() -> bool: ...
    @staticmethod
    def stream(stream: torch.Stream): ...
    @staticmethod
    def current_stream() -> None: ...
    @staticmethod
    def set_stream(stream: torch.Stream): ...
    @staticmethod
    def get_raw_stream(device_idx: int) -> int: ...
    @staticmethod
    def synchronize(device: _device_t = None): ...
    @classmethod
    def get_device_properties(cls, device: _device_t = None): ...
    @staticmethod
    def get_compute_capability(device: _device_t = None): ...
    @staticmethod
    def is_bf16_supported(including_emulation: bool = False): ...
    @staticmethod
    def memory_allocated(device: _device_t = None) -> int: ...

class DeviceGuard:
    device_interface: Incomplete
    idx: Incomplete
    prev_idx: int
    def __init__(self, device_interface: type[DeviceInterface], index: int | None) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, type: Any, value: Any, traceback: Any): ...

class CudaInterface(DeviceInterface):
    device = torch.cuda.device
    Event = torch.cuda.Event
    Stream = torch.cuda.Stream
    class Worker:
        @staticmethod
        def set_device(device: int): ...
        @staticmethod
        def current_device() -> int: ...
        @staticmethod
        def get_device_properties(device: _device_t = None): ...
    current_device: Incomplete
    set_device: Incomplete
    device_count: Incomplete
    stream: Incomplete
    current_stream: Incomplete
    set_stream: Incomplete
    synchronize: Incomplete
    get_device_properties: Incomplete
    get_raw_stream: Incomplete
    exchange_device: Incomplete
    maybe_exchange_device: Incomplete
    memory_allocated: Incomplete
    is_bf16_supported: Incomplete
    @staticmethod
    def is_available() -> bool: ...
    @staticmethod
    def get_compute_capability(device: _device_t = None): ...

get_xpu_stream: Callable[[int], int] | None

class XpuInterface(DeviceInterface):
    device = torch.xpu.device
    Event = torch.xpu.Event
    Stream = torch.xpu.Stream
    class Worker:
        @staticmethod
        def set_device(device: int): ...
        @staticmethod
        def current_device() -> int: ...
        @staticmethod
        def get_device_properties(device: _device_t = None): ...
    current_device: Incomplete
    set_device: Incomplete
    device_count: Incomplete
    stream: Incomplete
    current_stream: Incomplete
    set_stream: Incomplete
    synchronize: Incomplete
    get_device_properties: Incomplete
    get_raw_stream: Incomplete
    exchange_device: Incomplete
    maybe_exchange_device: Incomplete
    memory_allocated: Incomplete
    @staticmethod
    def is_available() -> bool: ...
    @staticmethod
    def get_compute_capability(device: _device_t = None): ...
    @staticmethod
    def is_bf16_supported(including_emulation: bool = False) -> bool: ...

@dataclass
class CpuDeviceProperties:
    multi_processor_count: int

class CpuInterface(DeviceInterface):
    class Event(torch.Event):
        time: float
        def __init__(self, enable_timing: bool = True) -> None: ...
        def elapsed_time(self, end_event) -> float: ...
        def record(self, stream: Incomplete | None = None) -> None: ...
    @staticmethod
    def is_available() -> bool: ...
    @staticmethod
    def get_compute_capability(device: _device_t = None) -> str: ...
    @staticmethod
    def get_raw_stream(device_idx) -> int: ...
    @staticmethod
    def current_device(): ...
    @staticmethod
    def synchronize(device: _device_t = None): ...
    class Worker:
        @staticmethod
        def get_device_properties(device: _device_t = None): ...

device_interfaces: dict[str, type[DeviceInterface]]

def register_interface_for_device(device: str | torch.device, device_interface: type[DeviceInterface]): ...
def get_interface_for_device(device: str | torch.device) -> type[DeviceInterface]: ...
def get_registered_device_interfaces() -> Iterable[tuple[str, type[DeviceInterface]]]: ...
def init_device_reg() -> None: ...
