import contextlib
import threading
import torch
import torch.fx
from . import config as config, convert_frame as convert_frame, external_utils as external_utils, trace_rules as trace_rules, utils as utils
from .backends.registry import CompilerFn as CompilerFn, lookup_backend as lookup_backend
from .code_context import code_context as code_context
from .exc import CondOpArgsMismatchError as CondOpArgsMismatchError, UserError as UserError, UserErrorType as UserErrorType
from .hooks import Hooks as Hooks
from .mutation_guard import install_generation_tagging_init as install_generation_tagging_init
from .types import CacheEntry as CacheEntry, DynamoCallback as DynamoCallback
from .utils import common_constant_types as common_constant_types, compile_times as compile_times
from _typeshed import Incomplete
from dataclasses import dataclass
from enum import Enum
from torch import _guards
from torch._C._dynamo.eval_frame import reset_code as reset_code, set_guard_error_hook as set_guard_error_hook, set_skip_guard_eval_unsafe as set_skip_guard_eval_unsafe, skip_code as skip_code, unsupported as unsupported
from torch._dispatch.python import enable_python_dispatcher as enable_python_dispatcher
from torch._subclasses import fake_tensor as fake_tensor
from torch._subclasses.fake_tensor import unset_fake_temporarily as unset_fake_temporarily
from torch._utils_internal import justknobs_check as justknobs_check, log_export_usage as log_export_usage
from torch.fx import GraphModule as GraphModule
from torch.fx.experimental.proxy_tensor import make_fx as make_fx
from torch.fx.experimental.symbolic_shapes import ConstraintViolationError as ConstraintViolationError, DimDynamic as DimDynamic, ShapeEnv as ShapeEnv, StatelessSymbolicContext as StatelessSymbolicContext
from typing import Any, Callable, NamedTuple

log: Incomplete
always_optimize_code_objects: Incomplete
null_context = contextlib.nullcontext

class Unset(Enum):
    token = 0

cached_backends: dict[int, CompilerFn]
unset: Incomplete

@dataclass
class DynamoStance:
    stance: str = ...
    skip_guard_eval_unsafe: bool = ...
    backend: str | Callable[..., Any] | None = ...

DONT_WRAP_FILES: Incomplete

class OptimizedModule(torch.nn.Module):
    get_compiler_config: Callable[[], Any]
    dynamo_ctx: Incomplete
    def __init__(self, mod: torch.nn.Module, dynamo_ctx) -> None: ...
    def __reduce__(self): ...
    @property
    def training(self): ...
    @training.setter
    def training(self, value) -> None: ...
    def __getattr__(self, name): ...
    def __setattr__(self, name, val) -> None: ...
    def __dir__(self): ...

def remove_from_cache(f) -> None: ...
def nothing() -> None: ...
def always_false(): ...
def innermost_fn(fn): ...
def make_set_enable_dynamic(enable: bool): ...

class DynamoTLS(threading.local):
    traced_frame_infos: list[str]

dynamo_tls: Incomplete

class _TorchDynamoContext:
    callback: DynamoCallback
    prior: Unset | DynamoCallback
    first_ctx: Incomplete
    export: Incomplete
    compiler_config: Incomplete
    cleanup_fns: list[Callable[[], Any]]
    enter_exit_hooks: Incomplete
    def __init__(self, callback: DynamoCallback, on_enter=..., backend_ctx_ctor=..., patch_fn=..., first_ctx: bool = False, *, export: bool = False, dynamic: Incomplete | None = None, compiler_config: Incomplete | None = None) -> None: ...
    prior_skip_guard_eval_unsafe: Incomplete
    def __enter__(self) -> None: ...
    def __exit__(self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: types.TracebackType | None) -> None: ...
    def __call__(self, fn): ...

class OptimizeContext(_TorchDynamoContext):
    def __init__(self, callback, backend_ctx_ctor, first_ctx: bool = False, *, export: bool = False, dynamic: Incomplete | None = None, compiler_config: Incomplete | None = None, rebuild_ctx: Callable[[], OptimizeContext | _NullDecorator] | None = None) -> None: ...
    def __reduce__(self): ...

class RunOnlyContext(_TorchDynamoContext):
    def __init__(self) -> None: ...
    def __reduce__(self): ...

class DisableContext(_TorchDynamoContext):
    def __init__(self) -> None: ...
    def __call__(self, fn): ...
    def __reduce__(self): ...

def get_compiler_fn(compiler_fn): ...

class _NullDecorator(contextlib.nullcontext):
    def __call__(self, fn): ...

def check_if_dynamo_supported() -> None: ...
def is_dynamo_supported(): ...
def check_if_inductor_supported() -> None: ...
def is_inductor_supported(): ...
def optimize(*args, **kwargs): ...
def explain(f, *extra_args, **extra_kwargs): ...

class FlattenInputOutputSignature(torch.fx.Transformer):
    new_args: Incomplete
    old_args_gen: Incomplete
    matched_output_elements_positions: Incomplete
    flat_results: Incomplete
    def __init__(self, m: torch.fx.GraphModule, flat_args: tuple[Any], matched_input_elements_positions: list[int], flat_results: list[Any], matched_output_elements_positions: list[int], example_fake_inputs: list[torch.Tensor], flat_args_dynamic_dims: list[set[int]], fake_mode: fake_tensor.FakeTensorMode | None = None) -> None: ...
    def placeholder(self, target, args, kwargs): ...
    def output(self, target, args, kwargs): ...
    current_node: Incomplete
    def run_node(self, n): ...
    def transform(self): ...

class ExportResult(NamedTuple):
    graph_module: torch.fx.GraphModule
    guards: _guards.GuardsSet

def check_signature_rewritable(graph) -> None: ...
def rewrite_signature(f_sig, graph, fake_mode, flat_args, in_spec, example_fake_inputs, graph_captured_input, graph_captured_output, dynamo_traced_result, flat_args_dynamic_dims): ...
def export(f: Callable[..., Any], *extra_args, aten_graph: bool = False, pre_dispatch: bool = False, decomposition_table: dict[torch._ops.OpOverload, Callable[..., Any]] | None = None, tracing_mode: str = 'symbolic', dynamic_shapes: dict[str, Any] | tuple[Any] | list[Any] | None = None, specialize_float: bool = True, assume_static_by_default: bool = False, same_signature: bool = True, disable_constraint_solver: bool = False, prefer_deferred_runtime_asserts_over_guards: bool = False, allow_complex_guards_as_runtime_asserts: bool = False, _log_export_usage: bool = True, **extra_kwargs) -> Callable[..., ExportResult]: ...
def optimize_assert(backend, *, hooks=..., export: bool = False, export_constraints: Incomplete | None = None, dynamic: Incomplete | None = None, rebuild_ctx: Incomplete | None = None): ...

class TorchPatcher:
    @staticmethod
    def patch() -> None: ...
    @staticmethod
    def suppress_torch_distributed_warnings(fn): ...
