import sympy
import torch
import torch._guards
import torch.nn
import traceback
from . import config as config, exc as exc, variables as variables
from .backends.registry import CompiledFn as CompiledFn, CompilerFn as CompilerFn
from .bytecode_transformation import Instruction as Instruction, create_call_function as create_call_function, create_instruction as create_instruction, create_load_const as create_load_const, unique_id as unique_id
from .code_context import code_context as code_context
from .codegen import PyCodegen as PyCodegen
from .current_scope_id import enter_new_scope as enter_new_scope
from .exc import BackendCompilerFailed as BackendCompilerFailed, SkipFrame as SkipFrame, exceptions_allowed_to_be_fallback as exceptions_allowed_to_be_fallback, unimplemented as unimplemented, unimplemented_with_warning as unimplemented_with_warning
from .graph_deduplication import apply_graph_deduplication as apply_graph_deduplication
from .graph_region_tracker import GraphRegionTracker as GraphRegionTracker
from .guards import GuardBuilder as GuardBuilder, install_guard as install_guard
from .mutation_guard import is_dynamic_nn_module as is_dynamic_nn_module
from .side_effects import AttributeMutationExisting as AttributeMutationExisting, SideEffects as SideEffects
from .source import AttrSource as AttrSource, BackwardStateSource as BackwardStateSource, ConstantSource as ConstantSource, GetItemSource as GetItemSource, GlobalStateSource as GlobalStateSource, LocalSource as LocalSource, ParamBufferSource as ParamBufferSource, ShapeEnvSource as ShapeEnvSource, SyntheticLocalSource as SyntheticLocalSource, TensorProperty as TensorProperty, TensorPropertySource as TensorPropertySource, is_constant_source as is_constant_source, is_from_local_source as is_from_local_source
from .utils import CleanupHook as CleanupHook, LazyString as LazyString, checkpoint_params as checkpoint_params, clone_inputs as clone_inputs, count_calls as count_calls, counters as counters, dynamo_timed as dynamo_timed, get_instruction_source_311 as get_instruction_source_311, get_locals_to_steal as get_locals_to_steal, get_static_address_type as get_static_address_type, graph_break_reasons as graph_break_reasons, increment_op_count as increment_op_count, lazy_format_graph_code as lazy_format_graph_code, nn_module_proxy as nn_module_proxy, same as same, set_example_value as set_example_value
from .variables.base import VariableTracker as VariableTracker
from .variables.builder import BackwardStateGraphArg as BackwardStateGraphArg, GraphArg as GraphArg, TrackedFake as TrackedFake, wrap_fx_proxy as wrap_fx_proxy
from .variables.lists import BaseListVariable as BaseListVariable
from .variables.misc import CellVariable as CellVariable, NullVariable as NullVariable
from .variables.nn_module import NNModuleVariable as NNModuleVariable
from .variables.tensor import NumpyNdarrayVariable as NumpyNdarrayVariable, SymNodeVariable as SymNodeVariable, TensorVariable as TensorVariable, UnspecializedPythonVariable as UnspecializedPythonVariable
from .variables.torch_function import TensorWithTFOverrideVariable as TensorWithTFOverrideVariable
from _typeshed import Incomplete
from collections.abc import Generator
from dataclasses import dataclass
from torch import fx as fx
from torch._dynamo.exc import TensorifyScalarRestartAnalysis as TensorifyScalarRestartAnalysis
from torch._dynamo.symbolic_convert import InstructionTranslatorBase as InstructionTranslatorBase
from torch._guards import CompileContext as CompileContext, CompileId as CompileId, GlobalContextCheckpointState as GlobalContextCheckpointState, Source as Source, TracingContext as TracingContext
from torch._utils_internal import signpost_event as signpost_event
from torch.fx.experimental._backward_state import BackwardState as BackwardState
from torch.fx.experimental.symbolic_shapes import ShapeEnv as ShapeEnv, free_symbols as free_symbols, is_symbolic as is_symbolic
from torch.fx.passes.runtime_assert import insert_deferred_runtime_asserts as insert_deferred_runtime_asserts
from torch.utils._python_dispatch import is_traceable_wrapper_subclass as is_traceable_wrapper_subclass
from typing import Any, Callable

log: Incomplete
graph_tabular_log: Incomplete
graph_code_log: Incomplete
graph_sizes_log: Incomplete
trace_call_log: Incomplete

@dataclass(frozen=True)
class VariableTrackerCacheKey:
    vt_id: int
    source: Source

class VariableTrackerCache:
    cache: Incomplete
    def __init__(self) -> None: ...
    def lookup(self, value, source): ...
    def add(self, value, source, vt) -> None: ...
    def clone(self): ...
    def clear(self) -> None: ...

@dataclass
class GraphCompileReason:
    reason: str
    user_stack: list[traceback.FrameSummary]
    graph_break: bool = ...
    def __post_init__(self) -> None: ...

class FakeRootModule(torch.nn.Module):
    def __init__(self, nn_modules: dict[str, torch.nn.Module]) -> None: ...

class WrapperBackend:
    backend: CompilerFn
    def __init__(self, backend: CompilerFn) -> None: ...
    restore: Incomplete
    gm: Incomplete
    candidate: Incomplete
    def __call__(self, gm: torch.fx.GraphModule, example_inputs: list[torch.Tensor]): ...
Scope = dict[str, object]

class OutputGraph:
    tracers: Incomplete
    input_source_to_var: dict[Source, VariableTracker]
    export: Incomplete
    export_constraints: Incomplete
    frame_state: Incomplete
    input_source_to_sizes_strides: dict[Source, dict[str, Any]]
    cleanup_hooks: list[Callable[[], Any]]
    compile_id: int
    installed_globals: set[str]
    co_fields: Incomplete
    region_tracker: Incomplete
    tracked_fakes: list[TrackedFake]
    tracing_context: TracingContext
    dynamo_compile_id: CompileId | None
    tracked_fakes_id_to_source: dict[int, list[Source]]
    param_name_to_source: dict[str, Source] | None
    side_effects: Incomplete
    variable_tracker_cache: Incomplete
    unique_var_id: Incomplete
    code_options: Incomplete
    output_instructions: list[Instruction]
    timestamp: int
    register_finalizer_fns: list[Callable[[fx.GraphModule], None]]
    compiler_fn: CompilerFn | None
    global_scope: Incomplete
    local_scope: Incomplete
    root_tx: Incomplete
    source_to_user_stacks: dict[Source, list[traceback.StackSummary]]
    cleanups: list[CleanupHook]
    should_exit: bool
    unspec_variable_map: dict[str, UnspecializedPythonVariable]
    torch_function_enabled: Incomplete
    torch_function_mode_enabled: Incomplete
    torch_function_mode_stack: Incomplete
    has_user_defined_allowed_in_graph: bool
    non_compliant_ops: set[torch._ops.OpOverload]
    compliant_custom_ops: set[torch._ops.OpOverload]
    dynamo_flat_name_to_original_fqn: dict[str, str]
    random_calls: list[tuple[Callable[..., object], tuple[object, ...], dict[str, object]]]
    random_values_var: Incomplete
    pregraph_bytecode: list[Instruction]
    backward_state: dict[str, VariableTracker]
    backward_state_proxy: torch.fx.Proxy | None
    backward_state_var: str | None
    name_of_builtins_dict_key_in_fglobals: str
    guard_on_key_order: set[str]
    def __init__(self, code_options: dict[str, Any], compiler_fn: CompilerFn | None, root_tx, export: bool, export_constraints, frame_state, local_scope: Scope, global_scope: Scope, f_code, torch_function_mode_stack) -> None: ...
    def install_builtins_dict_in_fglobals(self): ...
    def add_backward_state_hook(self, hook: VariableTracker, prefix: str = 'hook'): ...
    def get_backward_state_proxy(self): ...
    def init_ambient_guards(self) -> None: ...
    def synthetic_graph_input(self, fn, args): ...
    def add_cleanup_hook(self, fn: Callable[[], Any]): ...
    def call_cleanup_hooks(self) -> None: ...
    @property
    def root_tracer(self): ...
    @property
    def current_tracer(self): ...
    def is_root_tracer(self): ...
    @property
    def graph(self): ...
    @graph.setter
    def graph(self, value) -> None: ...
    @property
    def input_name_to_proxy(self): ...
    @property
    def real_value_cache(self): ...
    @property
    def bound_symbols(self): ...
    def create_proxy(self, *args, **kwargs): ...
    def create_node(self, *args, **kwargs): ...
    def remove_node(self, *args, **kwargs): ...
    def subtracer(self, source_target, prior_tracer) -> Generator[Incomplete]: ...
    @property
    def output(self): ...
    @property
    def fake_mode(self): ...
    @property
    def shape_env(self): ...
    @property
    def guards(self) -> torch._guards.GuardsSet: ...
    @property
    def nn_modules(self) -> dict[str, Any]: ...
    def save_global_state(self, out: Incomplete | None = None) -> None: ...
    def push_tx(self, tx) -> None: ...
    def pop_tx(self): ...
    @property
    def current_tx(self): ...
    def count_calls(self): ...
    def is_empty_graph(self): ...
    def get_submodule(self, keys): ...
    def new_var(self, name: str = 'tmp'): ...
    def update_co_names(self, name) -> None: ...
    @staticmethod
    def module_key_name(*names): ...
    def register_attr_or_module(self, target: torch.nn.Module | torch.Tensor | Any, *names, **options): ...
    def handle_aliases_for_stolen_lists(self, tx): ...
    partial_convert: Incomplete
    compile_subgraph_reason: Incomplete
    def compile_subgraph(self, tx, partial_convert: bool = False, reason: GraphCompileReason | None = None): ...
    def codegen_suffix(self, tx, stack_values, cg): ...
    def cleanup_graph(self) -> None: ...
    def get_graph_sizes_structured(self): ...
    def get_graph_sizes(self, name: str): ...
    def restore_global_state(self) -> Generator[None]: ...
    def run_compiler_collective(self, tx): ...
    def compile_and_call_fx_graph(self, tx, rv, root, replaced_outputs): ...
    @property
    def placeholders(self) -> list[fx.Node]: ...
    @property
    def graphargs(self) -> list[GraphArg]: ...
    def call_user_compiler(self, gm: fx.GraphModule) -> CompiledFn: ...
    def dedup_pass(self): ...
    def install_subgraph(self, name, sub_gm): ...
    def example_inputs(self) -> list[torch.Tensor]: ...
    def remove_unused_graphargs(self) -> None: ...
    def add_output_instructions(self, prefix: list[Instruction]) -> None: ...
    def install_global_unsafe(self, name, value) -> None: ...
    def install_global_by_id(self, prefix, value) -> str: ...
    def install_global(self, prefix, value) -> str: ...
    def cleanup(self) -> None: ...
    def set_torch_function_state(self, enabled: bool) -> None: ...
    def add_graph_finalizer(self, register_finalizer: Callable[[fx.GraphModule], None]) -> None: ...
    def example_value_from_input_node(self, node: torch.fx.Node): ...

err_epilogue: str

def check_pt2_compliant_op(output_graph, kind, target, args, kwargs) -> None: ...

class LazyProxy:
    tracer: Incomplete
    fn: Incomplete
    args: Incomplete
    kwargs: Incomplete
    def __init__(self, tracer, fn, *args, **kwargs) -> None: ...
    def __call__(self): ...

class SubgraphTracer(fx.Tracer):
    output_graph: Incomplete
    graph: Incomplete
    is_export: Incomplete
    input_name_to_proxy: dict[str, fx.Proxy]
    real_value_cache: dict[fx.Node, torch.Tensor]
    parent: Incomplete
    lifted_freevars: Incomplete
    bound_symbols: dict[sympy.Symbol, torch.fx.Proxy | LazyProxy]
    prev_inst: Incomplete
    under_activation_checkpoint: bool
    allow_side_effects_under_checkpoint: bool
    debug_level: int
    source_fn_stack: Incomplete
    def __init__(self, output_graph, parent: Incomplete | None = None, is_export: bool = False, source_target: Incomplete | None = None) -> None: ...
    def create_proxy(self, kind, target, args, kwargs, name: Incomplete | None = None, type_expr: Incomplete | None = None, proxy_factory_fn: Incomplete | None = None): ...
    def create_node(self, op, target, args: Incomplete | None = None, kwargs: Incomplete | None = None, name: Incomplete | None = None, type_expr: Incomplete | None = None): ...
    def remove_node(self, node) -> None: ...
    def create_graph_input(self, name, type_expr, example_value, before: bool = False, source: Incomplete | None = None): ...
    def lift_tracked_freevar_to_input(self, proxy): ...
    def maybe_lift_tracked_freevar_to_input(self, arg): ...
    def track_unbacked_symbols(self, example_value, e_proxy: LazyProxy | torch.fx.Proxy): ...
    def lookup_unbound_symbols(self, s: torch.SymInt) -> list[sympy.Symbol]: ...
