from . import utils as utils, variables as variables
from .bytecode_transformation import bytecode_from_template as bytecode_from_template, create_call_function as create_call_function, create_call_method as create_call_method, create_instruction as create_instruction
from .codegen import PyCodegen as PyCodegen
from .exc import unimplemented as unimplemented
from .source import GlobalSource as GlobalSource, LocalCellSource as LocalCellSource, LocalSource as LocalSource, Source as Source
from .utils import is_frozen_dataclass as is_frozen_dataclass, nn_module_new as nn_module_new, object_new as object_new
from .variables.base import AttributeMutation as AttributeMutation, AttributeMutationExisting as AttributeMutationExisting, AttributeMutationNew as AttributeMutationNew, ValueMutationExisting as ValueMutationExisting, VariableTracker as VariableTracker, is_side_effect_safe as is_side_effect_safe
from .variables.user_defined import FrozenDataClassVariable as FrozenDataClassVariable
from _typeshed import Incomplete
from types import CellType
from typing import Any

class SideEffects:
    id_to_variable: dict[int, VariableTracker]
    store_attr_mutations: dict[VariableTracker, dict[str, VariableTracker]]
    keepalive: list[Any]
    output_graph_weakref: Incomplete
    save_for_backward: Incomplete
    tensor_hooks: Incomplete
    ca_final_callbacks_var: Incomplete
    def __init__(self, output_graph, id_to_variable: Incomplete | None = None, store_attr_mutations: Incomplete | None = None, keepalive: Incomplete | None = None, save_for_backward: Incomplete | None = None, tensor_hooks: Incomplete | None = None) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def diff(self, other: SideEffects) -> str | None: ...
    def clone(self): ...
    def __contains__(self, item) -> bool: ...
    def __getitem__(self, item): ...
    def should_allow_side_effects_under_checkpoint(self): ...
    def check_allowed_side_effect(self, item): ...
    def store_attr(self, item: VariableTracker, name: str, value: VariableTracker): ...
    def load_attr(self, item, name, deleted_ok: bool = False, check: bool = False): ...
    def store_cell(self, cellvar, value) -> None: ...
    def load_cell(self, cellvar): ...
    def load_global(self, gvar: VariableTracker, name: str): ...
    def store_global(self, gvar: VariableTracker, name: str, value: VariableTracker): ...
    @staticmethod
    def cls_supports_mutation_side_effects(cls): ...
    def is_attribute_mutation(self, item): ...
    def has_pending_mutation(self, item): ...
    def has_pending_mutation_of_attr(self, item, name): ...
    def is_modified(self, item): ...
    track_mutable: Incomplete
    def track_object_existing(self, item: Any, variable: VariableTracker): ...
    def track_object_new(self, cls_source: Source, user_cls: Any, variable_cls: Any, options): ...
    def track_object_new_from_user_defined_class(self, cls_variable: variables.UserDefinedClassVariable): ...
    def track_cell_new(self): ...
    def track_cell_existing(self, source: Source | None, cell: CellType, contents: VariableTracker): ...
    def track_global_existing(self, source: Source, item: Any): ...
    def track_save_for_backward(self, ctx, args) -> None: ...
    def track_tensor_variables_from_runahead_side_effects(self, other) -> None: ...
    def prune_dead_object_new(self, tx): ...
    def mutation(self, var) -> None: ...
    def codegen_save_tempvars(self, cg: PyCodegen): ...
    def register_hook(self, tensor, hook, handle, name) -> None: ...
    def remove_hook(self, idx) -> None: ...
    def codegen_hooks(self, cg) -> None: ...
    def get_ca_final_callbacks_var(self): ...
    def codegen_update_mutated(self, cg: PyCodegen): ...
    def is_empty(self): ...
    def clear(self) -> None: ...

def allow_side_effects_under_checkpoint(tx: InstructionTranslator): ...
