import dataclasses
import dis
import enum
import torch
import torch.fx
import types
import weakref
from . import config as config
from _typeshed import Incomplete
from collections.abc import Generator
from torch import fx as fx
from torch._dispatch.python import enable_python_dispatcher as enable_python_dispatcher
from torch._dynamo.metrics_context import MetricsContext as MetricsContext
from torch._guards import Source as Source, TracingContext as TracingContext, detect_fake_mode as detect_fake_mode
from torch._logging import LazyString as LazyString
from torch._subclasses import UnsupportedFakeTensorException as UnsupportedFakeTensorException
from torch._subclasses.fake_tensor import FakeTensor as FakeTensor, is_fake as is_fake, maybe_get_fake_mode as maybe_get_fake_mode
from torch._subclasses.meta_utils import is_sparse_compressed as is_sparse_compressed
from torch._utils_internal import log_chromium_event_internal as log_chromium_event_internal, log_compilation_event as log_compilation_event, record_chromium_event_internal as record_chromium_event_internal, signpost_event as signpost_event
from torch.fx._utils import lazy_format_graph_code as lazy_format_graph_code
from torch.nn.modules.lazy import LazyModuleMixin as LazyModuleMixin
from torch.utils._triton import has_triton as has_triton, has_triton_package as has_triton_package
from torch.utils.hooks import RemovableHandle as RemovableHandle
from typing import Any, Callable, ClassVar, Counter, DefaultDict, Iterable, Iterator, KeysView, TypeVar, ValuesView, overload
from typing_extensions import Literal, TypeIs

NP_SUPPORTED_MODULES: tuple[types.ModuleType, ...]
NP_TO_TNP_MODULE: Incomplete
T = TypeVar('T')
unpatched_nn_module_getattr: Incomplete
unpatched_nn_module_call: Incomplete
unpatched_nn_module_call_impl: Incomplete
counters: DefaultDict[str, Counter[str]]
optimus_scuba_log: dict[str, Any]
troubleshooting_url: str
nnmodule_doc_url: str
nnmodule_doc_url_msg: Incomplete
log: Incomplete
compilation_time_metrics: dict[str, list[float]]
cumulative_time_spent_ns: dict[str, float]
timer_counter: Incomplete

class ReInplaceTrigger(enum.Enum):
    AUTO_FUNC_V1 = 1
    AUTO_FUNC_V2 = 2
    TRITON_OPS = 3

class ReinplaceCounters:
    @classmethod
    def add_missed_bytes(cls, trigger: ReInplaceTrigger, bytes: int): ...
    @classmethod
    def add_missed_opportunities(cls, trigger: ReInplaceTrigger, count: int): ...
    @classmethod
    def clear(cls) -> None: ...
    @classmethod
    def get_total_missed(cls): ...
    @classmethod
    def get_total_missed_bytes(cls): ...
    @classmethod
    def log(cls) -> None: ...

def tabulate(rows: list[tuple[str, object]] | list[list[object]], headers: tuple[str, ...] | list[str]) -> str: ...

curr_frame: int

def increment_frame() -> None: ...
def reset_frame_count() -> None: ...

op_count: int

def increment_op_count(cnt: int) -> None: ...
def calculate_time_spent() -> dict[str, float]: ...
def print_time_report() -> None: ...
def get_metrics_context() -> MetricsContext: ...
def dynamo_timed(key: str, phase_name: str | None = None, log_pt2_compile_event: bool = False, metadata: dict[str, object] | None = None, dynamo_compile_column_us: str | None = None, log_waitcounter: bool = False) -> Generator[Any, None, None]: ...
@overload
def compile_times(repr: Literal['str'], aggregate: bool = False) -> str: ...
@overload
def compile_times(repr: Literal['csv'], aggregate: bool = False) -> tuple[list[str], list[object]]: ...
def dump_compile_times() -> None: ...

tensortype_to_dtype: Incomplete

class DuplicateWarningChecker:
    maxsize: Incomplete
    def __init__(self, maxsize: int = 4096) -> None: ...
    set: Incomplete
    def reset(self) -> None: ...
    def add(self, key: str | tuple[object, object]) -> bool: ...

graph_break_dup_warning_checker: Incomplete

def setup_compile_debug(): ...
def reset_graph_break_dup_checker() -> None: ...
def add_file_handler(): ...
def setup_log_file(): ...
def gen_record_file_name(exc, code) -> str: ...
def write_record_to_file(filename: str, exec_record) -> None: ...
def count_calls(g: fx.Graph) -> int: ...
def identity(x: T) -> T: ...
def hashable(x): ...
def nothing(*args, **kwargs) -> None: ...

class ExactWeakKeyDictionary:
    values: Incomplete
    refs: Incomplete
    def __init__(self) -> None: ...
    def __getitem__(self, key): ...
    def get(self, key, default: Incomplete | None = None): ...
    def __contains__(self, key) -> bool: ...
    def __setitem__(self, key, value) -> None: ...
    def clear(self) -> None: ...

@overload
def istype(obj: object, allowed_types: type[T]) -> TypeIs[T]: ...
@overload
def istype(obj: object, allowed_types: tuple[type[list[T]], type[tuple[T, ...]]]) -> TypeIs[T]: ...
@overload
def istype(obj: object, allowed_types: Iterable[type]) -> bool: ...
def is_typing(value): ...
def is_numpy_int_type(value): ...
def is_numpy_float_type(value): ...
def is_lru_cache_wrapped_function(value): ...
def is_function_or_wrapper(value): ...
def is_function(value): ...
def is_wrapper_or_member_descriptor(value): ...
def unwrap_if_wrapper(fn): ...
def unwrap_with_attr_name_if_wrapper(fn): ...
def is_numpy_ndarray(value): ...
def istensor(obj): ...
def is_lazy_module(mod): ...
def print_once(*args) -> None: ...
def make_cell(val: Incomplete | None = None): ...
def proxy_args_kwargs(args, kwargs): ...
def to_int_ms(v: float | None) -> int | None: ...
def to_int_us(v: float | None) -> int | None: ...

LOG_FORMAT_VERSION: int

@dataclasses.dataclass
class CompilationMetrics:
    compile_id: str | None = ...
    frame_key: str | None = ...
    co_name: str | None = ...
    co_filename: str | None = ...
    co_firstlineno: int | None = ...
    cache_size: int | None = ...
    accumulated_cache_size: int | None = ...
    guard_count: int | None = ...
    shape_env_guard_count: int | None = ...
    graph_op_count: int | None = ...
    graph_node_count: int | None = ...
    graph_input_count: int | None = ...
    start_time: float | None = ...
    entire_frame_compile_time_s: float | None = ...
    backend_compile_time_s: float | None = ...
    inductor_compile_time_s: float | None = ...
    code_gen_time_s: float | None = ...
    fail_type: str | None = ...
    fail_reason: str | None = ...
    fail_user_frame_filename: str | None = ...
    fail_user_frame_lineno: int | None = ...
    non_compliant_ops: set[str] | None = ...
    compliant_custom_ops: set[str] | None = ...
    restart_reasons: set[str] | None = ...
    dynamo_time_before_restart_s: float | None = ...
    has_guarded_code: bool | None = ...
    remote_cache_time_saved_s: float | None = ...
    structured_logging_overhead_s: float | None = ...
    config_suppress_errors: bool | None = ...
    config_inline_inbuilt_nn_modules: bool | None = ...
    specialize_float: bool | None = ...
    dynamo_config: str | None = ...
    is_forward: bool | None = ...
    num_triton_bundles: int | None = ...
    remote_fx_graph_cache_get_time_ms: int | None = ...
    remote_fx_graph_cache_put_time_ms: int | None = ...
    start_time_us: int | None = ...
    duration_us: int | None = ...
    dynamo_cumulative_compile_time_us: int | None = ...
    aot_autograd_cumulative_compile_time_us: int | None = ...
    inductor_cumulative_compile_time_us: int | None = ...
    inductor_code_gen_cumulative_compile_time_us: int | None = ...
    triton_compile_time_us: int | None = ...
    runtime_cudagraphify_time_us: int | None = ...
    runtime_triton_autotune_time_us: int | None = ...
    dynamo_compile_time_before_restart_us: int | None = ...
    cuda_synchronize_time_us: int | None = ...
    distributed_ephemeral_timeout_us: int | None = ...
    structured_logging_overhead_us: int | None = ...
    remote_fx_graph_cache_get_time_us: int | None = ...
    remote_fx_graph_cache_put_time_us: int | None = ...
    backward_cumulative_compile_time_us: int | None = ...
    end_time_us: int | None = ...
    pre_grad_pass_time_us: int | None = ...
    post_grad_pass_time_us: int | None = ...
    joint_graph_pass_time_us: int | None = ...
    log_format_version: int = ...
    inductor_config: str | None = ...
    remote_cache_version: int | None = ...
    inductor_fx_remote_cache_hit_count: int | None = ...
    inductor_fx_remote_cache_miss_count: int | None = ...
    inductor_fx_remote_cache_backend_type: str | None = ...
    inductor_fx_remote_cache_hit_keys: str | None = ...
    inductor_fx_remote_cache_miss_keys: str | None = ...
    cuda_version: str | None = ...
    triton_version: str | None = ...
    feature_usage: dict[str, bool] | None = ...

DEFAULT_COMPILATION_METRICS_LIMIT: int

def add_compilation_metrics_to_chromium(c: CompilationMetrics) -> None: ...
def record_compilation_metrics(start_time_ns: int, end_time_ns: int, metrics: dict[str, Any], exc_type: type[BaseException] | None, exc_value: BaseException | None): ...
def set_compilation_metrics_limit(new_size: int) -> None: ...
def clear_compilation_metrics() -> None: ...
def get_compilation_metrics() -> list[CompilationMetrics]: ...

class ChromiumEventLogger:
    def get_stack(self) -> list[str]: ...
    def get_top(self) -> str | None: ...
    def get_pt2_compile_substack(self): ...
    def get_event_data(self) -> dict[str, Any]: ...
    tls: Incomplete
    id_: Incomplete
    def __init__(self) -> None: ...
    def try_add_event_data(self, event_name: str, **kwargs) -> None: ...
    def add_event_data(self, event_name: str, **kwargs) -> None: ...
    def log_event_start(self, event_name: str, time_ns: int, metadata: dict[str, Any], log_pt2_compile_event: bool = False) -> None: ...
    def reset(self) -> None: ...
    def log_event_end(self, event_name: str, time_ns: int, metadata: dict[str, Any], start_time_ns: int, log_pt2_compile_event: bool) -> None: ...
    def log_instant_event(self, event_name: str, time_ns: int, metadata: dict[str, Any] | None = None, log_pt2_compile_event: bool = False) -> None: ...

CHROMIUM_EVENT_LOG: ChromiumEventLogger | None

def get_chromium_event_logger() -> ChromiumEventLogger: ...
def chromium_event_timed(event_name: str, reset_event_log: bool = False, log_pt2_compile_event: bool = False) -> Generator[Any, None, None]: ...

@dataclasses.dataclass
class CleanupHook:
    scope: dict[str, Any]
    name: str
    def __call__(self, *args) -> None: ...
    @staticmethod
    def create(scope, name, val): ...

class CleanupManager(ExactWeakKeyDictionary):
    count: int
    instance: ClassVar[CleanupManager]

def clone_tensor(x): ...
def clone_input(x, *, dtype: Incomplete | None = None): ...
def clone_inputs(example_inputs): ...
def skip_frame_if_in_functorch_mode(val: torch.Tensor): ...
def preserve_rng_state() -> Generator[None]: ...
def is_jit_model(model0): ...
def torchscript(model, example_inputs, verbose: bool = False): ...
def getfile(obj): ...
def is_namedtuple(obj): ...
def is_namedtuple_cls(cls): ...
def namedtuple_fields(cls) -> tuple[str, ...]: ...
def checkpoint_params(gm): ...
def timed(model, example_inputs, times: int = 1): ...
def check_is_cuda(gm, example_inputs): ...
def rot_n_helper(n): ...

common_constant_types: set[type]

def is_safe_constant(v): ...
def specialize_symnode(arg): ...
def guard_if_dyn(arg): ...
def check_constant_args(args, kwargs): ...
def check_unspec_python_args(args, kwargs): ...
def check_unspec_or_constant_args(args, kwargs): ...
def check_numpy_ndarray_args(args, kwargs): ...

dict_keys: type[KeysView[Any]]
dict_values: type[ValuesView[Any]]
odict_values: type[ValuesView[Any]]
tuple_iterator: type[Iterator[Any]]
range_iterator: type[Iterator[Any]]
tuple_iterator_len: Incomplete
object_new: Incomplete

def nn_module_new(cls): ...
def product(it): ...
def tuple_iterator_getitem(it, index): ...
iter_next = next

def normalize_range_iter(range_iter) -> tuple[int, int, int]: ...
def to_subclass(t, cls): ...
def dict_keys_getitem(d, n): ...
def enum_repr(value, local): ...
def set_example_value(node, example_value) -> None: ...
def iter_contains(items, search, tx, check_tensor_identity: bool = False): ...
def key_is_id(k): ...
def key_to_id(value): ...
def const_repr(x, *, local) -> str: ...
def dict_keys_repr(const_keys, *, local) -> str: ...

GLOBAL_KEY_PREFIX: str

def get_safe_global_name(tx, root, obj): ...
def wrap_fake_exception(fn): ...
def deepcopy_to_fake_tensor(obj, fake_mode): ...
def rmse(ref, res): ...
def same(ref, res, fp64_ref: Incomplete | None = None, cos_similarity: bool = False, tol: float = 0.0001, equal_nan: bool = False, exact_dtype: bool = True, relax_numpy_equality: bool = False, ignore_non_fp: bool = False, log_error=..., use_larger_multiplier_for_smaller_tensor: bool = False): ...
def format_func_info(code): ...
def disable_cache_limit() -> Generator[None]: ...

orig_code_map: Incomplete
guard_failures: DefaultDict[Any, list[Any]]
graph_break_reasons: list[torch._dynamo.output_graph.GraphCompileReason]
seen_code_map: Incomplete

def get_debug_dir(): ...
def extract_fake_example_value(node, required: bool = True): ...
def ensure_graph_fake(e, tx): ...
def get_fake_values_from_nodes(tx, nodes, allow_non_graph_fake): ...
def get_fake_value(node, tx, allow_non_graph_fake: bool = False): ...
def get_current_node(): ...
def set_current_node(node) -> Generator[None]: ...
def run_node(tracer, node, args, kwargs, nnmodule): ...
def get_real_value(node, tracer): ...
def assert_no_fake_params_or_buffers(gm): ...
def fqn(obj: Any): ...
def ifdynstaticdefault(count1, count2): ...
def import_submodule(mod: types.ModuleType): ...
def object_has_getattribute(value: Any): ...
def class_has_getattribute(cls): ...
def get_custom_getattr(value: Any, ignore_nn_module_getattr: bool = False): ...

class TensorStaticReason(enum.Enum):
    PARAMETER = 2
    NOT_TENSOR = 4
    NN_MODULE_PROPERTY = 5

def tensor_static_reason_to_message(reason: TensorStaticReason): ...
def tensor_always_has_static_shape(tensor: torch.Tensor | Any, is_tensor: bool, tensor_source: Source) -> tuple[bool, TensorStaticReason | None]: ...
def lazy_format_graph_tabular(fn_name, gm): ...
def format_bytecode(prefix, name, filename, line_no, code): ...

forward_hook_names: Incomplete
backward_hook_names: Incomplete
state_dict_hook_names: Incomplete
all_hook_names: Incomplete

def nn_module_has_global_hooks(): ...
def nn_module_get_all_hooks(mod, check_forward_hooks: bool = False, check_backward_hooks: bool = False, check_state_dict_hooks: bool = False): ...
def nnmodule_has_hooks(mod, check_forward_hooks: bool = False, check_backward_hooks: bool = False, check_state_dict_hooks: bool = False): ...
def to_numpy_helper(value): ...
def numpy_to_tensor(value): ...

class numpy_to_tensor_wrapper:
    f: Incomplete
    def __init__(self, f) -> None: ...
    def __call__(self, *args, **kwargs): ...

def numpy_attr_wrapper(obj, name): ...

class numpy_method_wrapper:
    method: Incomplete
    def __init__(self, method: str) -> None: ...
    def __call__(self, *args, **kwargs): ...

class numpy_operator_wrapper:
    op: Incomplete
    def __init__(self, op: Callable[..., Any]) -> None: ...
    def __call__(self, *args, **kwargs): ...

def defake(x): ...
def is_utils_checkpoint(obj): ...
def is_invoke_subgraph(obj): ...
def build_invoke_subgraph_variable(**options): ...
def build_checkpoint_variable(**options): ...
def is_compile_supported(device_type): ...

@dataclasses.dataclass
class _Anchors:
    left_end_lineno: int
    left_end_offset: int
    right_start_lineno: int
    right_start_offset: int

def get_instruction_source_311(code: types.CodeType, inst: dis.Instruction) -> str: ...
def get_static_address_type(t): ...
def is_rng_state_getter_or_setter(value): ...
def is_tensor_base_attr_getter(value): ...
def is_torch_function_object(value): ...
def has_torch_function(vt: torch._dynamo.variables.base.VariableTracker) -> bool: ...
def to_fake_tensor(t, fake_mode): ...
def is_frozen_dataclass(value): ...
def get_first_attr(obj, *attrs): ...
def maybe_enable_compiled_autograd(should_enable, fullgraph: bool = True, dynamic: bool = True) -> Generator[Incomplete, None, Incomplete]: ...
def invalid_removeable_handle(): ...
def nn_module_proxy(mod): ...

class GmWrapper(torch.nn.Module):
    gm: Incomplete
    unflatten_fn: Incomplete
    def __init__(self, gm, unflatten_fn) -> None: ...
    def forward(self, *args): ...

def flatten_graph_inputs(gm: torch.fx.GraphModule, inputs, compile_gm): ...
def get_locals_to_steal(maybe_gm): ...
def set_locals_to_steal(gm, locals_to_steal) -> None: ...

class Lit:
    s: Incomplete
    def __init__(self, s) -> None: ...

warn_once_cache: set[str]

def warn_once(msg, stacklevel: int = 1) -> None: ...
def strip_color_from_string(text): ...
def is_parameter_freezing(): ...
def get_torch_function_mode_stack(): ...
def get_torch_function_mode_stack_at(ind): ...
def set_torch_function_mode_stack(stack) -> None: ...
def clear_torch_function_mode_stack() -> None: ...
def verify_guard_fn_signature(value) -> None: ...
def does_not_override_dict_iter_methods(user_cls): ...
def call_size(x, i): ...
def call_stride(x, i): ...
def call_storage_offset(x): ...

user_obj_id_to_weakref: dict[int, weakref.ReferenceType[object]]

def get_user_object_from_id(obj_id): ...
def store_user_object_weakref(obj) -> None: ...

class CompileTimeInstructionCounter:
    @classmethod
    def start(cls) -> None: ...
    @classmethod
    def end(cls) -> None: ...
    @classmethod
    def clear(cls) -> None: ...
    @classmethod
    def value(cls) -> int: ...
    @classmethod
    def record(cls) -> Generator[None]: ...

def set_feature_use(feature: str, usage: bool): ...
