import torch
import torch.nn as nn
from . import config as config
from ._aot_autograd.autograd_cache import AOTAutogradCache as AOTAutogradCache, autograd_cache_key as autograd_cache_key, should_use_local_autograd_cache as should_use_local_autograd_cache, should_use_remote_autograd_cache as should_use_remote_autograd_cache
from ._aot_autograd.collect_metadata_analysis import run_functionalized_fw_and_collect_metadata as run_functionalized_fw_and_collect_metadata
from ._aot_autograd.functional_utils import are_all_mutations_hidden_from_autograd as are_all_mutations_hidden_from_autograd, are_all_mutations_under_no_grad_or_inference_mode as are_all_mutations_under_no_grad_or_inference_mode, assert_functional_graph as assert_functional_graph, from_fun as from_fun, gen_alias_from_base as gen_alias_from_base, has_data_mutation as has_data_mutation, has_metadata_mutation as has_metadata_mutation, is_fun as is_fun, sync_functional_tensor as sync_functional_tensor, to_fun as to_fun
from ._aot_autograd.input_output_analysis import compute_overlapping_inputs as compute_overlapping_inputs, create_graph_signature as create_graph_signature, create_synthetic_base_metadata as create_synthetic_base_metadata, remove_dupe_metadata as remove_dupe_metadata
from ._aot_autograd.jit_compile_runtime_wrappers import aot_dispatch_autograd as aot_dispatch_autograd, aot_dispatch_base as aot_dispatch_base, aot_dispatch_export as aot_dispatch_export
from ._aot_autograd.logging_utils import callback_set as callback_set, describe_input as describe_input, format_guard_bug_msg as format_guard_bug_msg, get_aot_compilation_context as get_aot_compilation_context, get_aot_graph_name as get_aot_graph_name, get_graph_being_compiled as get_graph_being_compiled, graph_being_compiled as graph_being_compiled, model_name as model_name, nth_graph as nth_graph, set_model_name as set_model_name, setup_stacktrace_preservation_hooks as setup_stacktrace_preservation_hooks, track_graph_compiling as track_graph_compiling
from ._aot_autograd.runtime_wrappers import AOTDedupeWrapper as AOTDedupeWrapper, AOTSyntheticBaseWrapper as AOTSyntheticBaseWrapper
from ._aot_autograd.schemas import AOTConfig as AOTConfig, BackwardSignature as BackwardSignature, FQN as FQN, GraphInputName as GraphInputName, GraphOutputName as GraphOutputName, GraphSignature as GraphSignature, InputAliasInfo as InputAliasInfo, MutationType as MutationType, OutputAliasInfo as OutputAliasInfo, OutputType as OutputType, SubclassCreationMeta as SubclassCreationMeta, SubclassMeta as SubclassMeta, TensorAlias as TensorAlias, ViewAndMutationMeta as ViewAndMutationMeta
from ._aot_autograd.subclass_utils import requires_subclass_dispatch as requires_subclass_dispatch, unwrap_tensor_subclasses as unwrap_tensor_subclasses, unwrap_tensor_subclasses_with_indices_to_original as unwrap_tensor_subclasses_with_indices_to_original, wrap_tensor_subclasses as wrap_tensor_subclasses, wrap_tensor_subclasses_maybe_joint as wrap_tensor_subclasses_maybe_joint
from ._aot_autograd.traced_function_transforms import aot_dispatch_subclass as aot_dispatch_subclass, create_functional_call as create_functional_call, create_functionalized_fn as create_functionalized_fn, create_functionalized_rng_ops_wrapper as create_functionalized_rng_ops_wrapper, create_joint as create_joint, fn_input_mutations_to_outputs as fn_input_mutations_to_outputs, fn_prepped_for_autograd as fn_prepped_for_autograd
from ._aot_autograd.utils import KNOWN_TYPES as KNOWN_TYPES, call_func_at_runtime_with_args as call_func_at_runtime_with_args, create_tree_flattened_fn as create_tree_flattened_fn, make_boxed_compiler as make_boxed_compiler, make_boxed_func as make_boxed_func, maybe_to_fresh_input as maybe_to_fresh_input, normalize_as_list as normalize_as_list, partial_flatten_asdict as partial_flatten_asdict, root_module_when_exporting_non_strict as root_module_when_exporting_non_strict, strict_zip as strict_zip
from .partitioners import default_partition as default_partition
from _typeshed import Incomplete
from torch import Tensor as Tensor
from torch._decomp.decompositions_for_rng import PhiloxStateTracker as PhiloxStateTracker, rng_decompositions as rng_decompositions
from torch._dispatch.python import enable_python_dispatcher as enable_python_dispatcher
from torch._dynamo import compiled_autograd as compiled_autograd
from torch._dynamo.utils import dynamo_timed as dynamo_timed, get_chromium_event_logger as get_chromium_event_logger, preserve_rng_state as preserve_rng_state
from torch._guards import detect_fake_mode as detect_fake_mode
from torch._inductor.output_code import OutputCode as OutputCode
from torch._inductor.utils import BoxedBool as BoxedBool, InputType as InputType
from torch._subclasses import FakeTensor as FakeTensor, FakeTensorMode as FakeTensorMode
from torch.fx.experimental.proxy_tensor import make_fx as make_fx
from torch.fx.experimental.symbolic_shapes import ShapeEnv as ShapeEnv
from torch.utils._python_dispatch import is_traceable_wrapper_subclass as is_traceable_wrapper_subclass
from typing import Any, Callable, Protocol, Sequence, TypeVar

static_inputs_log: Incomplete
zip = strict_zip
AOT_COUNTER: Incomplete
aot_autograd_decompositions: Incomplete
FakifiedFlatArgs: Incomplete
TOutputCode = TypeVar('TOutputCode', bound=OutputCode)

class AOTDispatchCompiler(Protocol):
    def __call__(self, gm: torch.fx.GraphModule, example_inputs: Sequence[InputType]) -> Any: ...

class SerializableAOTDispatchCompiler(AOTDispatchCompiler):
    output_code_ty: Incomplete
    compiler_fn: Incomplete
    def __init__(self, output_code_ty: type[TOutputCode], compiler_fn: Callable[[torch.fx.GraphModule, Sequence[InputType]], TOutputCode]) -> None: ...
    def __call__(self, gm: torch.fx.GraphModule, example_inputs: Sequence[InputType]) -> OutputCode: ...

def process_inputs(flat_args: list[Any], aot_config: AOTConfig, fake_mode: FakeTensorMode, shape_env: ShapeEnv | None) -> FakifiedFlatArgs: ...
def construct_fake_mode(flat_args: list[Any], aot_config: AOTConfig) -> tuple[FakeTensorMode, ShapeEnv | None]: ...
def create_aot_dispatcher_function(flat_fn, fake_flat_args: FakifiedFlatArgs, aot_config: AOTConfig, fake_mode: FakeTensorMode, shape_env: ShapeEnv | None) -> tuple[Callable, ViewAndMutationMeta]: ...
def aot_function(fn: Callable, fw_compiler: Callable, bw_compiler: Callable | None = None, partition_fn: Callable = ..., decompositions: dict | None = None, num_params_buffers: int = 0, keep_inference_input_mutations: bool = False, inference_compiler: Callable | None = None, *, dynamic: bool = False, enable_log: bool = True) -> Callable: ...
def aot_module(mod: nn.Module, *args, **kwargs) -> nn.Module: ...
def aot_module_simplified(mod: nn.Module, args, fw_compiler: AOTDispatchCompiler, bw_compiler: AOTDispatchCompiler | None = None, partition_fn: Callable = ..., decompositions: dict | None = None, keep_inference_input_mutations: bool = False, inference_compiler: AOTDispatchCompiler | None = None, cudagraphs: BoxedBool | None = None) -> nn.Module: ...
def aot_export_module(mod: nn.Module, args, *, decompositions: dict | None = None, trace_joint: bool, output_loss_index: int | None = None, pre_dispatch: bool = False, dynamic_shapes: bool | None = None, kwargs: Incomplete | None = None) -> tuple[torch.fx.GraphModule, GraphSignature]: ...
def aot_export_joint_simple(func: Callable, args, *, trace_joint: bool, num_params_buffers: int = 0, decompositions: dict | None = None) -> torch.fx.GraphModule: ...
compiled_function = aot_function
compiled_module = aot_module
