import functools
import logging
import torch
import torch.fx as fx
from . import config as config
from ._activation_checkpointing.knapsack import dp_knapsack as dp_knapsack, greedy_knapsack as greedy_knapsack, ilp_knapsack as ilp_knapsack
from ._aot_autograd.logging_utils import get_aot_graph_name as get_aot_graph_name
from ._aot_autograd.utils import is_with_effects as is_with_effects
from .compile_utils import fx_graph_cse as fx_graph_cse, get_aten_target as get_aten_target
from _typeshed import Incomplete
from dataclasses import dataclass
from torch.fx.experimental._backward_state import BackwardState as BackwardState
from torch.fx.experimental.proxy_tensor import is_sym_node as is_sym_node, py_sym_types as py_sym_types
from torch.fx.experimental.sym_node import magic_methods as magic_methods, method_to_operator as method_to_operator
from torch.fx.experimental.symbolic_shapes import find_symbol_binding_fx_nodes as find_symbol_binding_fx_nodes, free_symbols as free_symbols, hint_int as hint_int, is_symbol_binding_fx_node as is_symbol_binding_fx_node
from torch.fx.passes import graph_drawer as graph_drawer
from torch.utils._mode_utils import no_dispatch as no_dispatch
from torch.utils.checkpoint import CheckpointPolicy as CheckpointPolicy
from typing import Callable

AOT_PARTITIONER_DEBUG: bool
log: logging.Logger
aten: Incomplete
prims: Incomplete

@dataclass
class OpTypes:
    fusible_ops: set[Callable]
    compute_intensive_ops: set[Callable]
    random_ops: set[Callable]
    view_ops: set[Callable]
    recomputable_ops: set[Callable]
    def is_fusible(self, node: fx.Node): ...
    def is_compute_intensive(self, node: fx.Node): ...
    def is_random(self, node: fx.Node): ...
    def is_view(self, node: fx.Node): ...
    def is_recomputable(self, node: fx.Node): ...

@dataclass
class NodeInfo:
    inputs: list[fx.Node]
    required_bw_nodes: set[fx.Node]
    unclaimed_nodes: set[fx.Node]
    fw_order: dict[fx.Node, int]
    @functools.cached_property
    def required_fw_nodes(self) -> list[fx.Node]: ...
    def is_required_fw(self, n: fx.Node) -> bool: ...
    def is_required_bw(self, n: fx.Node) -> bool: ...
    def is_unclaimed(self, n: fx.Node) -> bool: ...
    def get_fw_order(self, n: fx.Node) -> int: ...

@dataclass
class MinCutOptions:
    ban_if_used_far_apart: bool
    ban_if_long_fusible_chains: bool
    ban_if_materialized_backward: bool
    ban_if_not_in_allowlist: bool
    ban_if_reduction: bool

def must_recompute(node: fx.Node) -> bool: ...
def has_recomputable_ops(fx_g: fx.GraphModule) -> bool: ...
def has_recomputable_rng_ops(fx_g: fx.GraphModule) -> bool: ...
def sym_node_size(node: fx.Node) -> int: ...

class InvalidNodeBase: ...

InvalidNode: Incomplete

def default_partition(joint_module: fx.GraphModule, _joint_inputs, *, num_fwd_outputs) -> tuple[fx.GraphModule, fx.GraphModule]: ...

INT_INF: Incomplete

def pointwise_ops(): ...
def sort_depths(args, depth_map: dict[fx.Node, int]) -> list[tuple[fx.Node, int]]: ...
def reordering_to_mimic_autograd_engine(gm: fx.GraphModule) -> fx.GraphModule: ...
def functionalize_rng_ops(joint_module: fx.GraphModule, fw_module: fx.GraphModule, bw_module: fx.GraphModule, num_sym_nodes: int) -> tuple[fx.GraphModule, fx.GraphModule]: ...
def cleanup_recompute_tags(joint_module: fx.GraphModule) -> fx.GraphModule: ...
def solve_min_cut(joint_graph: fx.Graph, node_info: NodeInfo, min_cut_options: MinCutOptions, dont_ban: Incomplete | None = None): ...
def visualize_min_cut_graph(nx_graph) -> None: ...
def get_default_op_list() -> OpTypes: ...
def get_name_to_node(graph: fx.Graph): ...
def estimate_runtime(node): ...
def choose_saved_values_set(joint_graph: fx.Graph, node_info: NodeInfo, memory_budget: int = 1) -> list[fx.Node]: ...
def min_cut_rematerialization_partition(joint_module: fx.GraphModule, _joint_inputs, compiler: str = 'inductor', *, num_fwd_outputs) -> tuple[fx.GraphModule, fx.GraphModule]: ...
def draw_graph(traced: torch.fx.GraphModule, fname: str, figname: str = 'fx_graph', clear_meta: bool = True, prog: str | list[str] | None = None, parse_stack_trace: bool = False, dot_graph_shape: str | None = None) -> None: ...
