import abc
import torch
from _typeshed import Incomplete
from abc import ABC, abstractmethod
from dataclasses import dataclass
from torch import Tensor as Tensor
from torch._C import DispatchKey as DispatchKey
from torch._ops import HigherOrderOperator as HigherOrderOperator, OpOverload as OpOverload, OperatorBase as OperatorBase
from torch._prims_common import clone_preserve_strides as clone_preserve_strides
from torch._subclasses.fake_tensor import FakeTensorMode as FakeTensorMode
from torch.fx.experimental.proxy_tensor import ProxyTorchDispatchMode as ProxyTorchDispatchMode, disable_proxy_modes_tracing as disable_proxy_modes_tracing, track_tensor_tree as track_tensor_tree
from typing import Any, Sequence

def get_base(tensor): ...

class ViewInfo(ABC, metaclass=abc.ABCMeta):
    base_index: int
    def __init__(self, base_index) -> None: ...
    @abstractmethod
    def regenerate_view(self, bases_list: list[Tensor]): ...

@dataclass
class AsStridedViewInfo(ViewInfo):
    size: Sequence[int | torch.SymInt]
    stride: Sequence[int | torch.SymInt]
    storage_offset: int
    def __init__(self, base_index, size, stride, storage_offset) -> None: ...
    def regenerate_view(self, bases_list: list[Tensor]): ...

@dataclass
class SliceViewInfo(ViewInfo):
    dim: int | torch.SymInt
    start: int | torch.SymInt
    end: int | torch.SymInt
    def __init__(self, base_index, dim, start, end) -> None: ...
    def regenerate_view(self, bases_list: list[Tensor]): ...

@dataclass
class AliasViewInfo(ViewInfo):
    def __init__(self, base_index) -> None: ...
    def regenerate_view(self, bases_list: list[Tensor]): ...

@dataclass
class NotView(ViewInfo):
    def __init__(self, base_index) -> None: ...
    def regenerate_view(self, bases_list: list[Tensor]): ...

def is_alias(base, tensor): ...
def try_use_slice(base, tensor): ...
def write_view_information_to_args(mutable_arg_names: list[str], mutable_arg_types: list[torch.Type], kwargs: dict[str, Any], arg_to_base_index: dict[str, Any]): ...
def read_view_information_from_args(mutable_arg_names: list[str], mutable_arg_types: list[torch.Type], kwargs: dict[str, Any], all_bases: list[Tensor]): ...

class AutoFunctionalized(HigherOrderOperator):
    def __init__(self) -> None: ...
    def __call__(self, /, _mutable_op: OpOverload, **kwargs: Any) -> tuple[Any, tuple[Tensor, ...]]: ...

auto_functionalized: Incomplete

class AutoFunctionalizedV2(HigherOrderOperator):
    def __init__(self) -> None: ...
    def __call__(self, /, _mutable_op: OpOverload, **kwargs: Any) -> tuple[Any, tuple[Tensor, ...]]: ...

auto_functionalized_v2: Incomplete

def can_auto_functionalize(op: OperatorBase) -> bool: ...
def get_mutable_args(op: OpOverload) -> tuple[list[str], list[torch.Type]]: ...
def do_auto_functionalize(op: OpOverload, args: tuple[Any, ...], kwargs: dict[str, Any]) -> Any: ...
def do_auto_functionalize_v2(op: OpOverload, args: tuple[Any, ...], kwargs: dict[str, Any]) -> Any: ...
def auto_functionalized_dense(_mutable_op: OpOverload, _only_clone_these_tensors: tuple[str, ...] | None = None, **kwargs: Any) -> tuple[Any, tuple[Tensor, ...]]: ...
def auto_functionalized_fake(mode, _mutable_op: OpOverload, **kwargs: Any) -> tuple[Any, tuple[Tensor, ...]]: ...
def auto_functionalized_proxy(mode, _mutable_op: OpOverload, **kwargs: Any) -> tuple[Any, tuple[Tensor, ...]]: ...
def auto_functionalized_func(ctx, _mutable_op, **kwargs): ...
def auto_functionalized_v2_dense(_mutable_op: OpOverload, _only_clone_these_bases: tuple[int, ...] | None = None, **kwargs: Any) -> tuple[Any, tuple[Tensor, ...]]: ...
def auto_functionalized_v2_fake(mode, _mutable_op: OpOverload, **kwargs: dict[str, Any]) -> tuple[Any, tuple[Tensor, ...]]: ...
def auto_functionalized_v2_proxy(mode, _mutable_op: OpOverload, **kwargs: dict[str, Any]) -> tuple[Any, tuple[Tensor, ...]]: ...
def auto_functionalized_v2_func(ctx, _mutable_op, **kwargs): ...
