import dataclasses
import sympy
from _typeshed import Incomplete
from torch import SymInt as SymInt, Tensor as Tensor
from torch._C import DispatchKey as DispatchKey
from torch._dynamo.symbolic_convert import InstructionTranslator as InstructionTranslator
from torch._dynamo.variables.constant import ConstantVariable as ConstantVariable
from torch._dynamo.variables.functions import TritonKernelVariable as TritonKernelVariable
from torch._ops import HigherOrderOperator as HigherOrderOperator
from torch._prims_common import clone_preserve_strides as clone_preserve_strides
from torch._subclasses.fake_tensor import FakeTensorMode as FakeTensorMode
from torch._subclasses.functional_tensor import BaseFunctionalizeAPI as BaseFunctionalizeAPI
from torch.fx.experimental.proxy_tensor import ProxyTorchDispatchMode as ProxyTorchDispatchMode, disable_proxy_modes_tracing as disable_proxy_modes_tracing, track_tensor_tree as track_tensor_tree
from torch.fx.experimental.symbolic_shapes import guard_scalar as guard_scalar
from torch.fx.proxy import Proxy as Proxy
from torch.utils._triton import has_triton as has_triton
from triton._C.libtriton.ir import module as TritonIRModule
from triton.runtime.autotuner import Autotuner
from triton.runtime.jit import JITFunction
from typing import Any, Callable, Sequence
from typing_extensions import Never

TritonMetaParamsType = dict[str, int]
TritonGridTupleType: Incomplete
TritonGridCallableType = Callable[[TritonMetaParamsType], tuple[int, ...]]
TritonGridType = TritonGridTupleType | TritonGridCallableType

class Autotuner: ...
class JITFunction: ...
TritonKernelType = Autotuner | JITFunction
log: Incomplete
TMADescriptorMetadata = dict[str, tuple[list[int | SymInt], list[int | SymInt], int | SymInt]]

class KernelSideTable:
    id_to_kernel: dict[int, 'TritonKernelType']
    kernel_to_id: dict['TritonKernelType', int]
    constant_args: dict[int, dict[str, Any]]
    lock: Incomplete
    def add_kernel(self, kernel: TritonKernelType) -> int: ...
    def get_kernel(self, idx: int) -> TritonKernelType: ...
    def add_constant_args(self, args: dict[str, Any]) -> int: ...
    def get_constant_args(self, idx: int) -> dict[str, Any]: ...
    def reset_table(self) -> None: ...

kernel_side_table: Incomplete

@dataclasses.dataclass(frozen=True)
class Param:
    idx: int

@dataclasses.dataclass(frozen=True)
class Intermediate:
    idx: int
    def fake(self) -> bool: ...

@dataclasses.dataclass(frozen=True)
class Op:
    name: str
    fn_call_name: str | None
    args: list[Param | Intermediate]
    ret: Intermediate = dataclasses.field(repr=False)
    def __post_init__(self) -> None: ...

def generate_ttir(kernel: TritonKernelType, kwargs: dict[str, Any]) -> tuple['TritonIRModule', list[str]]: ...
def ttir_to_functions(ttir_module: TritonIRModule) -> dict[str, dict[Intermediate, list[Op]]]: ...

class MemoizeWithCycleCheck:
    fn: Callable[..., Any]
    cache: dict[tuple[str, int], Any]
    def __init__(self, fn: Callable[..., Any]) -> None: ...
    def __call__(self, functions: dict[str, dict[Intermediate, list[Op]]], fn_name: str, num_args: int) -> list[bool]: ...
    def reset(self) -> None: ...

def analyze_kernel_mutations(functions: dict[str, dict[Intermediate, list[Op]]], fn_name: str, num_args: int) -> list[bool]: ...
def identify_mutated_tensors(kernel: TritonKernelType, kwargs: dict[str, Any]) -> list[str]: ...

class TritonKernelWrapperMutation(HigherOrderOperator):
    def __init__(self) -> None: ...
    def __call__(self, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any]) -> Any: ...

triton_kernel_wrapper_mutation: Incomplete

class TritonKernelWrapperFunctional(HigherOrderOperator):
    def __init__(self) -> None: ...
    def __call__(self, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any], tensors_to_clone: list[str]) -> dict[str, Any]: ...

triton_kernel_wrapper_functional: Incomplete

def triton_kernel_wrapper_mutation_dense(*, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any]) -> None: ...
def triton_kernel_wrapper_mutation_fake_tensor_mode(mode: FakeTensorMode, *, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any]) -> None: ...
def _(*, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any]) -> None: ...
def trace_triton_kernel_wrapper(proxy_mode: ProxyTorchDispatchMode, func_overload: Callable[..., Any], node_args: dict[str, Any]) -> dict[str, Any] | None: ...
def triton_kernel_wrapper_mutation_proxy_torch_dispatch_mode(mode: ProxyTorchDispatchMode, *, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any]) -> None: ...
def get_mutated_tensors(kernel_idx: int, constant_args_idx: int, kwargs: dict[str, Any]) -> list[str]: ...
def triton_kernel_wrapper_mutation_functionalize(ctx: BaseFunctionalizeAPI, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any]) -> None: ...
def triton_kernel_wrapper_functional_dense(*, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any], tensors_to_clone: list[str]) -> dict[str, Any]: ...
def triton_kernel_wrapper_functional_fake_tensor_mode(mode: FakeTensorMode, *, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any], tensors_to_clone: list[str]) -> dict[str, Any]: ...
def triton_kernel_wrapper_functional_proxy_torch_dispatch_mode(mode: ProxyTorchDispatchMode, *, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any], tensors_to_clone: list[str]) -> dict[str, Any]: ...
def triton_kernel_wrapper_functional_functionalize(ctx: BaseFunctionalizeAPI, kernel_idx: int, constant_args_idx: int, grid: list['TritonGridType'], tma_descriptor_metadata: TMADescriptorMetadata, kwargs: dict[str, Any], tensors_to_clone: list[str]) -> dict[str, Any]: ...

class TritonHOPifier:
    def raise_unsupported(self, msg: str) -> Never: ...
    def is_callable(self, maybe_callable: Any) -> bool: ...
    def get_value(self, val: Any) -> Any: ...
    def call_grid(self, grid, meta, tx) -> tuple[int | sympy.Expr | SymInt, ...] | tuple['Proxy', ...]: ...
    def call_HOP(self, variable, grids, combined_args: dict[str, Any], tx) -> ConstantVariable | None: ...
    def check_grid(self, grid) -> tuple[int | sympy.Expr | SymInt, ...] | tuple['Proxy', ...]: ...
    def init_variable(self, variable: TraceableTritonKernelWrapper | TritonKernelVariable, kernel: TritonKernelType, kernel_idx: int | None, grid: TritonGridType | None) -> None: ...
    def call_getitem(self, variable: TritonKernelVariable | TraceableTritonKernelWrapper, args: Sequence[Any]) -> TritonKernelVariable | TraceableTritonKernelWrapper: ...
    def call_run(self, variable: TritonKernelVariable | TraceableTritonKernelWrapper, args: Sequence[Any], kwargs: dict[str, Any], tx: InstructionTranslator | None) -> ConstantVariable | None: ...
    def call_triton_kernel(self, variable: TritonKernelVariable | TraceableTritonKernelWrapper, args: Sequence[Any], kwargs: dict[str, Any], tx: InstructionTranslator | None) -> ConstantVariable | None: ...

class TracingTritonHOPifier(TritonHOPifier):
    def raise_unsupported(self, msg: str) -> Never: ...
    def is_callable(self, maybe_callable: Any) -> bool: ...
    def get_value(self, val: Any) -> Any: ...
    def call_grid(self, grid: TritonGridCallableType, meta: TritonMetaParamsType, tx: None) -> tuple[int | sympy.Expr | SymInt, ...]: ...
    def check_grid(self, grid: TritonGridType) -> tuple[int | sympy.Expr | SymInt, ...]: ...
    def call_HOP(self, variable: TraceableTritonKernelWrapper, grids: list['TritonGridTupleType'], combined_args: dict[str, Any], tx: None) -> None: ...

tracing_triton_hopifier_singleton: Incomplete

class TraceableTritonKernelWrapper:
    kernel: TritonKernelType
    kernel_idx: int | None
    grid: TritonGridType | None
    def __init__(self, kernel: TritonKernelType, kernel_idx: int | None, grid: TritonGridType | None) -> None: ...
    def __getitem__(self, *args: Sequence[Any]) -> TraceableTritonKernelWrapper: ...
    def run(self, *args: Sequence[Any], **kwargs: dict[str, Any]) -> Any: ...
    def __call__(self, *args: Sequence[Any], **kwargs: dict[str, Any]) -> Any: ...
    def specialize_symbolic(self, arg: Sequence[Any]) -> Any: ...
