import dataclasses
import queue
import torch
from . import config as config
from .codegen.common import WorkspaceArg as WorkspaceArg, WorkspaceZeroMode as WorkspaceZeroMode
from .runtime.benchmarking import benchmarker as benchmarker
from .virtualized import V as V
from _typeshed import Incomplete
from concurrent.futures import ThreadPoolExecutor
from ctypes import CDLL
from multiprocessing.process import BaseProcess
from multiprocessing.queues import Queue
from torch import multiprocessing as multiprocessing
from torch._dynamo.testing import rand_strided as rand_strided
from torch._inductor import ir as ir
from torch._inductor.codecache import CUDACodeCache as CUDACodeCache, CppCodeCache as CppCodeCache, DLLWrapper as DLLWrapper, PyCodeCache as PyCodeCache, get_hash as get_hash
from torch._inductor.select_algorithm import TritonTemplateCaller as TritonTemplateCaller
from types import ModuleType
from typing import Any, Callable, Iterable, Sequence

CUDA_VISIBLE_DEVICES: str
EXIT_HANDLER_REGISTERED: bool
log: Incomplete

class Ping: ...
class Pong: ...
class NonzeroWorkspaceNotSupportedError(Exception): ...

def set_cuda_visible_device(device: int | None): ...

@dataclasses.dataclass
class TuningProcess:
    device: int | None = ...
    process: BaseProcess | None = ...
    request_queue: Queue[Any] | None = ...
    response_queue: Queue[Any] | None = ...
    @staticmethod
    def process_main(request_queue: Queue[Any], response_queue: Queue[Any]) -> None: ...
    @staticmethod
    def workloop(request_queue: Queue[Any], response_queue: Queue[Any]) -> None: ...
    def valid(self) -> bool: ...
    def clear(self) -> None: ...
    def initialize(self) -> None: ...
    def put(self, obj: Any) -> None: ...
    def get(self, result_timeout: float = 120.0, graceful_timeout: float = 3.0, terminate_timeout: float = 1.0) -> Any: ...
    def terminate(self) -> None: ...
    def wait(self) -> None: ...
    def kill(self, graceful_timeout: float = 5.0, terminate_timeout: float = 1.0) -> None: ...

@dataclasses.dataclass
class TuningProcessPool:
    processes: queue.Queue[TuningProcess] | None = ...
    executor: ThreadPoolExecutor | None = ...
    def initialize(self) -> None: ...
    def get_device_list(self) -> Sequence[int | None]: ...
    def terminate(self) -> None: ...
    def target(self, choice: TritonTemplateCaller) -> float: ...
    def benchmark(self, choices: list[TritonTemplateCaller]) -> dict[TritonTemplateCaller, float]: ...

tuning_pool: Incomplete
LayoutOrBuffer: Incomplete

@dataclasses.dataclass
class TensorMeta:
    device: torch.device
    dtype: torch.dtype
    sizes: torch._prims_common.ShapeType
    strides: torch._prims_common.StrideType
    offset: int
    name: str | None = ...
    @classmethod
    def from_irnodes(cls, irnodes: LayoutOrBuffer | Sequence[LayoutOrBuffer]) -> TensorMeta | list[TensorMeta]: ...
    def to_tensor(self) -> torch.Tensor: ...

@dataclasses.dataclass
class BenchmarkRequest:
    kernel_name = ...
    input_tensor_meta = ...
    output_tensor_meta = ...
    extra_args = ...
    def __init__(self, kernel_name: str, input_tensor_meta: TensorMeta | list[TensorMeta], output_tensor_meta: TensorMeta | list[TensorMeta], extra_args: Iterable[Any]) -> None: ...
    def make_run_fn(self, *input_tensors: torch.Tensor, output_tensor: torch.Tensor) -> Callable[[], None]: ...
    def cleanup_run_fn(self) -> None: ...
    def do_bench(self, fn, *input_tensors: torch.Tensor, output_tensor: torch.Tensor | None = None) -> float: ...
    def benchmark(self, *input_tensors: torch.Tensor, output_tensor: torch.Tensor | None = None) -> float: ...

class TestBenchmarkRequest(BenchmarkRequest):
    value: Incomplete
    def __init__(self, value: float | None = None) -> None: ...
    def benchmark(self, *input_tensors: torch.Tensor, output_tensor: torch.Tensor | None = None) -> float: ...

class GPUDeviceBenchmarkMixin:
    def do_bench(self, fn, *input_tensors: torch.Tensor, output_tensor: torch.Tensor | None = None) -> float: ...

class CPUDeviceBenchmarkMixin:
    def do_bench(self, fn, *input_tensors: torch.Tensor, output_tensor: torch.Tensor | None = None) -> float: ...

class TritonBenchmarkRequest(BenchmarkRequest):
    module_path: Incomplete
    module_cache_key: Incomplete
    grid: Incomplete
    num_stages: Incomplete
    num_warps: Incomplete
    matrix_instr_nonkdim: Incomplete
    workspace_arg: Incomplete
    def __init__(self, kernel_name: str, input_tensor_meta: TensorMeta | list[TensorMeta], output_tensor_meta: TensorMeta | list[TensorMeta], extra_args: Iterable[Any], module_path: str, module_cache_key: str, grid: list[int], num_stages: int, num_warps: int, matrix_instr_nonkdim: int = 0, workspace_arg: WorkspaceArg | None = None) -> None: ...
    def make_run_fn(self, *input_tensors: torch.Tensor, output_tensor: torch.Tensor) -> Callable[[], None]: ...
    def precompile(self) -> None: ...

class TritonGPUBenchmarkRequest(GPUDeviceBenchmarkMixin, TritonBenchmarkRequest): ...
class TritonCPUBenchmarkRequest(CPUDeviceBenchmarkMixin, TritonBenchmarkRequest): ...

class CUDABenchmarkRequest(GPUDeviceBenchmarkMixin, BenchmarkRequest):
    source_code: Incomplete
    workspace_size: int
    workspace: torch.Tensor | None
    DLL: DLLWrapper | None
    hash_key: str
    source_file: str
    def __init__(self, kernel_name: str, input_tensor_meta: TensorMeta | list[TensorMeta], output_tensor_meta: TensorMeta | list[TensorMeta], extra_args: Iterable[Any], source_code: str) -> None: ...
    def precompile(self) -> None: ...
    def make_run_fn(self, *input_tensors: torch.Tensor, output_tensor: torch.Tensor) -> Callable[[], None]: ...
    def update_workspace_size(self) -> None: ...
    def ensure_dll_loaded(self) -> None: ...
    def cleanup_run_fn(self) -> None: ...

class CppBenchmarkRequest(CPUDeviceBenchmarkMixin, BenchmarkRequest):
    source_code: Incomplete
    hash_key: Incomplete
    DLL: CDLL | ModuleType | None
    def __init__(self, kernel_name: str, input_tensor_meta: TensorMeta | list[TensorMeta], output_tensor_meta: TensorMeta | list[TensorMeta], extra_args: Iterable[Any], source_code: str) -> None: ...
    def precompile(self) -> None: ...
    def make_run_fn(self, *input_tensors: torch.Tensor, output_tensor: torch.Tensor) -> Callable[[], None]: ...
    def cleanup_run_fn(self) -> None: ...

def benchmark_in_sub_process(choices: list[TritonTemplateCaller]) -> dict[TritonTemplateCaller, float]: ...
