import sympy
import torch
import typing
from . import config as config
from .codecache import write_text as write_text
from .codegen.simd_kernel_features import SIMDKernelFeatures as SIMDKernelFeatures
from .codegen.triton import TritonKernel as TritonKernel
from .metrics import get_metric_table as get_metric_table, is_metric_table_enabled as is_metric_table_enabled
from .runtime.hints import DeviceProperties as DeviceProperties, ReductionHint as ReductionHint
from .scheduler import BaseSchedulerNode as BaseSchedulerNode, Scheduler as Scheduler, WhyNoFuse as WhyNoFuse
from .virtualized import V as V
from typing import Any

class Sortable(typing.Protocol):
    def __lt__(self, other: typing.Self) -> bool: ...

class InductorChoices:
    def triton_kernel_kwargs(self, kernel_cls: type[TritonKernel], features: SIMDKernelFeatures, groups: list[sympy.Expr], kernel_kwargs: dict[str, Any]) -> dict[str, Any]: ...
    @staticmethod
    def should_use_cooperative_reduction(features: SIMDKernelFeatures) -> bool: ...
    @staticmethod
    def should_use_persistent_reduction(features: SIMDKernelFeatures, cooperative_reduction: bool) -> bool: ...
    @staticmethod
    def want_no_x_dim(features: SIMDKernelFeatures) -> bool: ...
    @staticmethod
    def reduction_split_factor(device: torch.device, reduction_numel_hint: int, numel_hint: int, inner_reduction: bool) -> int: ...
    @staticmethod
    def can_fuse(scheduler: Scheduler, node1: BaseSchedulerNode, node2: BaseSchedulerNode, shared_data_score: int) -> bool: ...
    @staticmethod
    def can_fuse_vertical(scheduler: Scheduler, node1: BaseSchedulerNode, node2: BaseSchedulerNode, shared_data_score: int) -> bool: ...
    @staticmethod
    def can_fuse_horizontal(scheduler: Scheduler, node1: BaseSchedulerNode, node2: BaseSchedulerNode, shared_data_score: int) -> bool: ...
    @staticmethod
    def score_fusion(scheduler: Scheduler, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> Sortable: ...
