import dataclasses
import hashlib
import pickle
import torch
from .compile_fx import CompiledFxGraph as CompiledFxGraph, _CompileFxKwargs
from .output_code import CompiledFxGraphConstants as CompiledFxGraphConstants, OutputCode as OutputCode
from .remote_cache import JsonDataTy as JsonDataTy, RemoteCache as RemoteCache, create_cache as create_cache
from .runtime import autotune_cache as autotune_cache
from .runtime.autotune_cache import AutotuneCacheBundler as AutotuneCacheBundler
from .triton_bundler import TritonBundler as TritonBundler
from .utils import InputType as InputType
from _typeshed import Incomplete
from collections.abc import KeysView
from concurrent.futures import Future
from ctypes import CDLL, c_void_p
from pathlib import Path
from torch import SymInt as SymInt, Tensor as Tensor
from torch._dynamo.utils import counters as counters, dynamo_timed as dynamo_timed, get_chromium_event_logger as get_chromium_event_logger, get_metrics_context as get_metrics_context
from torch._inductor import config as config, exc as exc, metrics as metrics
from torch._inductor.codegen.cuda import cuda_env as cuda_env
from torch._inductor.codegen.rocm.compile_command import rocm_compile_command as rocm_compile_command, rocm_compiler as rocm_compiler
from torch._inductor.cpp_builder import CppBuilder as CppBuilder, CppOptions as CppOptions, CppTorchDeviceOptions as CppTorchDeviceOptions, get_compiler_version_info as get_compiler_version_info, get_name_and_dir_from_output_file_path as get_name_and_dir_from_output_file_path, normalize_path_separator as normalize_path_separator
from torch._inductor.cpu_vec_isa import pick_vec_isa as pick_vec_isa
from torch._inductor.custom_graph_pass import CustomGraphPass as CustomGraphPass, CustomGraphPassType as CustomGraphPassType
from torch._inductor.fb.utils import log_global_cache_errors as log_global_cache_errors, log_global_cache_stats as log_global_cache_stats, log_global_cache_vals as log_global_cache_vals, use_global_cache as use_global_cache
from torch._inductor.graph import GraphLowering as GraphLowering
from torch._inductor.ir import ChoiceCaller as ChoiceCaller
from torch._inductor.output_code import has_frozen_params as has_frozen_params
from torch._inductor.runtime.hints import HalideInputSpec as HalideInputSpec, HalideMeta as HalideMeta
from torch._inductor.runtime.runtime_utils import cache_dir as cache_dir, default_cache_dir as default_cache_dir
from torch._inductor.utils import ALIGN_BYTES as ALIGN_BYTES, clear_on_fresh_inductor_cache as clear_on_fresh_inductor_cache, is_linux as is_linux, is_windows as is_windows
from torch._logging import trace_structured as trace_structured
from torch._subclasses.fake_tensor import FakeTensor as FakeTensor, TensorMetadata as TensorMetadata, extract_tensor_metadata as extract_tensor_metadata
from torch._utils_internal import log_cache_bypass as log_cache_bypass
from torch.fx.experimental.symbolic_shapes import ShapeEnv as ShapeEnv, has_hint as has_hint, hint_int as hint_int
from types import ModuleType
from typing import Any, Callable, Sequence, TypeVar

T = TypeVar('T')
output_code_log: Incomplete
LOCK_TIMEOUT: int
log: Incomplete

def cpp_wrapper_cache_dir(name: str) -> str: ...
def get_cpp_wrapper_cubin_path_name() -> str: ...
def get_global_cache_path_impl(global_cache_dir: str) -> Path | None: ...

class CacheBase:
    @staticmethod
    def get_system() -> dict[str, Any]: ...
    @staticmethod
    def get_local_cache_path() -> Path: ...
    @staticmethod
    def get_global_cache_path() -> Path | None: ...
    system: Incomplete
    def __init__(self) -> None: ...
    def get_local_cache(self) -> dict[str, Any]: ...
    def update_local_cache(self, local_cache: dict[str, Any]) -> None: ...

class LocalCache(CacheBase):
    def lookup(self, *keys: str) -> dict[str, Any] | None: ...
    def set_value(self, *keys: str, value: Any) -> None: ...

class PersistentCache(CacheBase):
    def get_global_cache(self) -> dict[str, Any]: ...
    def lookup(self, choices: list[ChoiceCaller], op: str, inputs: str, benchmark: Callable[[Any], dict[ChoiceCaller, float]] | None) -> dict[ChoiceCaller, float]: ...

def get_lock_dir() -> str: ...
def sha256_hash(data: bytes) -> str: ...
def code_hash(code: str | bytes, extra: str = '') -> str: ...
def get_path(basename: str, extension: str, specified_dir: str = '') -> tuple[str, str, str]: ...
def get_hash(content: str | bytes, extra: str = '', hash_type: str = 'code') -> str: ...
def write(content: str | bytes, extension: str, extra: str = '', hash_type: str = 'code', specified_dir: str = '') -> tuple[str, str]: ...
def write_text(text: str) -> str: ...
def write_atomic(path_: str, content: str | bytes, make_dirs: bool = False, encode_utf_8: bool = False) -> None: ...

@dataclasses.dataclass
class TensorMetadataAndValues:
    tensor_metadata: TensorMetadata
    values: list[Any]

def extract_tensor_metadata_for_cache_key(t: Tensor) -> TensorMetadata: ...

class FxGraphCachePickler(pickle.Pickler):
    include_non_inlined: Incomplete
    dispatch_table: Incomplete
    fast: bool
    def __init__(self, gm: torch.fx.GraphModule, include_non_inlined: bool = True, has_user_defined_triton_kernels: bool = False) -> None: ...
    def dumps(self, obj: Any) -> bytes: ...
    def get_hash(self, obj: Any) -> str: ...
    def debug_lines(self, inp: FxGraphHashDetails) -> list[str]: ...

def build_code_hash(roots: list[str] | None, prefix: str, hasher: hashlib._Hash) -> None: ...
def torch_key() -> bytes: ...
def get_inductor_root() -> str: ...

@dataclasses.dataclass
class OrderedSetHolder:
    items: list[Any]

class BypassFxGraphCache(Exception): ...

class FxGraphHashDetails:
    EXCLUDED_KWARGS: Incomplete
    gm: Incomplete
    example_inputs: Incomplete
    cache_key_tag: Incomplete
    fx_kwargs: dict[str, object]
    user_defined_triton_source: list[Any]
    inputs_to_check: Incomplete
    deterministic_algorithms_settings: Incomplete
    cuda_matmul_settings: Incomplete
    torch_version: Incomplete
    system_info: Incomplete
    inductor_config: Incomplete
    post_grad_custom_pre_pass: Incomplete
    post_grad_custom_post_pass: Incomplete
    def __init__(self, gm: torch.fx.GraphModule, example_inputs: Sequence[InputType], fx_kwargs: _CompileFxKwargs, inputs_to_check: Sequence[int]) -> None: ...

def compiled_fx_graph_hash(gm: torch.fx.GraphModule, example_inputs: Sequence[InputType], fx_kwargs: _CompileFxKwargs, inputs_to_check: Sequence[int]) -> tuple[str, list[str]]: ...
def add_ephemeral_timeout_increase_for_distributed(time_saved_ns: int) -> int: ...

class FxGraphCache:
    @staticmethod
    def prepare_key(gm: torch.fx.GraphModule, example_inputs: Sequence[InputType], fx_kwargs: _CompileFxKwargs, inputs_to_check: Sequence[int], remote: bool) -> tuple[tuple[str, list[str]] | None, dict[str, Any]]: ...
    @staticmethod
    def get_remote_cache() -> RemoteCache[JsonDataTy] | None: ...
    @staticmethod
    def load_with_key(key: str, debug_lines: list[str], example_inputs: Sequence[InputType], local: bool, remote_cache: RemoteCache[JsonDataTy] | None, is_backward: bool, constants: CompiledFxGraphConstants) -> tuple[CompiledFxGraph | None, dict[str, Any]]: ...
    @staticmethod
    def clear() -> None: ...

def run_command_and_check(cmd_: str) -> None: ...
def split_aot_inductor_output_path(path: str) -> tuple[str, str]: ...

class CudaKernelParamCache:
    cache: dict[str, dict[str, str]]
    cache_clear: Incomplete
    @classmethod
    def set(cls, key: str, params: dict[str, str], cubin: str, bin_type: str) -> None: ...
    @classmethod
    def get(cls, key: str) -> dict[str, str] | None: ...
    @classmethod
    def get_keys(cls) -> KeysView[str]: ...

class AotCodeCompiler:
    @classmethod
    def compile(cls, graph: GraphLowering, source_code: str, serialized_extern_kernel_nodes: str | None, device_type: str, additional_files: list[str]) -> list[str] | str: ...

def cpp_prefix_path() -> str: ...
def cpp_prefix() -> str: ...
def compile_file(input_path: str | list[str], output_path: str, cmd: list[str]) -> None: ...
def custom_op_wrapper(op: str, *args: Any) -> list[c_void_p] | c_void_p: ...

class CppCodeCache:
    cache: dict[str, Callable[[], CDLL | ModuleType]]
    cache_clear: Incomplete
    cpp_compile_command_flags: dict[str, Any]
    @classmethod
    def load_async(cls, source_code: str, device_type: str = 'cpu', submit_fn: Any = None, extra_flags: Sequence[str] = ()) -> Any: ...
    @classmethod
    def load(cls, source_code: str, device_type: str = 'cpu') -> Any: ...

class CppPythonBindingsCodeCache(CppCodeCache):
    cache: dict[str, Callable[[], CDLL | ModuleType]]
    cache_clear: Incomplete
    cpp_compile_command_flags: Incomplete
    entry_function: str
    call_entry_function: str
    extra_parse_arg: str
    suffix_template: Incomplete
    @classmethod
    def load_pybinding_async(cls, argtypes: list[str], source_code: str, device_type: str = 'cpu', num_outputs: int = -1, submit_fn: Any = None, extra_flags: Sequence[str] = ()) -> Any: ...
    @classmethod
    def load_pybinding(cls, *args: Any, **kwargs: Any) -> Any: ...

class CppWrapperCodeCache(CppPythonBindingsCodeCache):
    cache: dict[str, Callable[[], CDLL | ModuleType]]
    cache_clear: Incomplete
    cpp_compile_command_flags: Incomplete
    entry_function: str
    call_entry_function: str
    extra_parse_arg: Incomplete

class HalideCodeCache(CppPythonBindingsCodeCache):
    cache: dict[str, Callable[[], ModuleType | CDLL]]
    cache_clear: Incomplete
    prefix: Incomplete
    glue_template_cpp: Incomplete
    glue_template_cuda: Incomplete
    standalone_runtime_cuda_init: Incomplete
    @classmethod
    def config_hash(cls) -> str: ...
    @staticmethod
    def find_libautoschedule(name: str) -> str: ...
    @staticmethod
    def find_header(name: str) -> str: ...
    @classmethod
    def generate_halide_async(cls, meta: HalideMeta, source_code: str, submit_fn: Any = None) -> Callable[[], Any]: ...
    @classmethod
    def generate_halide(cls, *args: Any, **kwargs: Any) -> Callable[[], Any]: ...
    @classmethod
    def build_standalone_runtime(cls) -> str: ...

def touch(filename: str): ...

class PyCodeCache:
    modules: list[ModuleType]
    linemaps: dict[str, list[tuple[Any, ...]]]
    @classmethod
    def write(cls, source_code: str, extra: str = '') -> tuple[str, str]: ...
    @classmethod
    def load(cls, source_code: str, extra: str = '', linemap: list[tuple[int, str]] | None = None, attrs: dict[str, Any] | None = None) -> ModuleType: ...
    @classmethod
    def load_by_key_path(cls, key: str, path: str, linemap: list[tuple[int, str]] | None = None, attrs: dict[str, Any] | None = None) -> ModuleType: ...
    @classmethod
    def cache_clear(cls, purge: bool = False) -> None: ...
    @classmethod
    def stack_frames_for_code(cls, path: str, lineno: int) -> list[dict[str, Any]] | None: ...

class TritonCodeCache:
    @classmethod
    def load(cls, kernel_name: str, source_code: str) -> ModuleType: ...

def cuda_compile_command(src_files: list[str], dst_file: str, dst_file_ext: str, extra_args: list[str] | None = None) -> str: ...

class DLLWrapper:
    lib_path: Incomplete
    is_open: bool
    DLL: Incomplete
    def __init__(self, lib_path: str) -> None: ...
    def close(self) -> None: ...
    def __getattr__(self, name: str) -> Callable[..., None]: ...
    def __enter__(self) -> DLLWrapper: ...
    def __exit__(self, *args: Any) -> None: ...
    def __del__(self) -> None: ...

class CUDACodeCache:
    @dataclasses.dataclass
    class CacheEntry:
        input_path: str
        output_path: str
    cache: dict[str, CacheEntry]
    cache_clear: Incomplete
    @classmethod
    def write(cls, source_code: str, dst_file_ext: str) -> tuple[str, str]: ...
    @classmethod
    def compile(cls, source_code: str, dst_file_ext: str, extra_args: list[str] | None = None) -> tuple[str, str, str]: ...
    @classmethod
    def load(cls, source_code: str, dst_file_ext: str) -> tuple[DLLWrapper, str, str]: ...

class ROCmCodeCache:
    @dataclasses.dataclass
    class CacheEntry:
        input_path: str
        output_path: str
    cache: dict[str, CacheEntry]
    cache_clear: Incomplete
    @classmethod
    def write(cls, source_code: str, dst_file_ext: str) -> tuple[str, str]: ...
    @classmethod
    def compile(cls, source_code: str, dst_file_ext: str, extra_args: list[str] | None = None) -> tuple[str, str, str]: ...
    @classmethod
    def load(cls, source_code: str, dst_file_ext: str) -> tuple[DLLWrapper, str, str]: ...

class CodeCacheFuture:
    def result(self) -> None: ...

class TritonFuture(CodeCacheFuture):
    kernel: ModuleType
    future: Incomplete
    def __init__(self, kernel: Any, future: Future[Any] | None) -> None: ...
    def result(self) -> ModuleType: ...

class LambdaFuture(CodeCacheFuture):
    result_fn: Incomplete
    def __init__(self, result_fn: Callable[..., Any]) -> None: ...
    def result(self) -> Callable[..., Any]: ...
