import abc
import contextlib
import torch
import torch.fx
from . import config as config, metrics as metrics
from .._dynamo.backends.common import aot_autograd as aot_autograd
from ..utils._triton import has_triton as has_triton
from .debug import DebugContext as DebugContext
from .decomposition import select_decomp_table as select_decomp_table
from .fx_passes.joint_graph import joint_graph_passes as joint_graph_passes
from .fx_passes.post_grad import post_grad_passes as post_grad_passes, view_to_reshape as view_to_reshape
from .fx_passes.pre_grad import pre_grad_passes as pre_grad_passes
from .graph import GraphLowering as GraphLowering
from .ir import ExternKernelNode as ExternKernelNode, IRNode as IRNode, get_device_type as get_device_type
from .triton_bundler import TritonBundler as TritonBundler
from .utils import align_inputs_from_check_idxs as align_inputs_from_check_idxs, clone_preserve_strides as clone_preserve_strides, copy_misaligned_inputs as copy_misaligned_inputs, get_cloned_parameter_buffer_name as get_cloned_parameter_buffer_name, get_first_incompatible_cudagraph_node as get_first_incompatible_cudagraph_node, maybe_get_suppress_shape_guards_ctx as maybe_get_suppress_shape_guards_ctx, output_node as output_node, remove_unaligned_input_idxs as remove_unaligned_input_idxs, shape_env_from_inputs as shape_env_from_inputs
from .virtualized import V as V
from _typeshed import Incomplete
from abc import ABC, abstractmethod
from torch import fx as fx
from torch._dispatch.python import enable_python_dispatcher as enable_python_dispatcher
from torch._dynamo import compiled_autograd as compiled_autograd
from torch._dynamo.device_interface import get_interface_for_device as get_interface_for_device
from torch._dynamo.repro.after_aot import wrap_compiler_debug as wrap_compiler_debug
from torch._dynamo.utils import counters as counters, detect_fake_mode as detect_fake_mode, dynamo_timed as dynamo_timed, flatten_graph_inputs as flatten_graph_inputs, get_chromium_event_logger as get_chromium_event_logger, lazy_format_graph_code as lazy_format_graph_code, set_feature_use as set_feature_use
from torch._functorch._aot_autograd.schemas import FQN as FQN, GraphInputName as GraphInputName, GraphSignature as GraphSignature
from torch._functorch.aot_autograd import SerializableAOTDispatchCompiler as SerializableAOTDispatchCompiler, aot_export_module as aot_export_module, make_boxed_func as make_boxed_func
from torch._inductor.codecache import FxGraphCache as FxGraphCache, code_hash as code_hash, output_code_log as output_code_log
from torch._inductor.cudagraph_utils import BoxedDeviceIndex as BoxedDeviceIndex, PlaceholderInfo as PlaceholderInfo
from torch._inductor.debug import save_args_for_compile_fx_inner as save_args_for_compile_fx_inner
from torch._inductor.output_code import CompiledAOTI as CompiledAOTI, CompiledFxGraph as CompiledFxGraph, CompiledFxGraphConstantsWithGm as CompiledFxGraphConstantsWithGm, OutputCode as OutputCode, get_expanded_dims as get_expanded_dims, index_expanded_dims as index_expanded_dims
from torch._inductor.runtime.runtime_utils import cache_dir as cache_dir
from torch._inductor.utils import BoxedBool as BoxedBool, InputType as InputType, count_tangents as count_tangents, fresh_inductor_cache as fresh_inductor_cache, is_gpu as is_gpu, should_assume_input_aligned as should_assume_input_aligned, should_use_remote_fx_graph_cache as should_use_remote_fx_graph_cache, tensor_is_aligned as tensor_is_aligned
from torch._logging import trace_structured as trace_structured
from torch._ops import OpOverload as OpOverload
from torch._utils_internal import compile_time_strobelight_meta as compile_time_strobelight_meta
from torch.fx import GraphModule as GraphModule
from torch.fx.experimental.symbolic_shapes import SymExprPrinter as SymExprPrinter, free_unbacked_symbols as free_unbacked_symbols
from torch.fx.passes.fake_tensor_prop import FakeTensorProp as FakeTensorProp
from torch.utils._ordered_set import OrderedSet as OrderedSet
from typing import Any, Callable, ContextManager, Generator, Sequence
from typing_extensions import Protocol, TypedDict, Unpack

def time_and_log(attr: str) -> Callable[[Callable[_P, _T]], Callable[_P, _T]]: ...
def log_optimus_to_scuba(*args: object, **kwargs: object) -> None: ...

log: Incomplete
perf_hint_log: Incomplete
post_grad_graphs_log: Incomplete
static_inputs_log: Incomplete

def get_static_input_idxs(num_fixed: int) -> list[int]: ...
def record_original_output_strides(gm: GraphModule) -> None: ...
def split_const_gm(gm: GraphModule, skip_constructor: bool = True, lifted_constant_names: list[str] | None = None, skip_folding_node_fn: Callable[[torch.fx.Node], bool] | None = None) -> tuple[GraphModule, dict[str, int]]: ...
def is_tf32_warning_applicable(gm: GraphModule) -> bool: ...
def maybe_disable_comprehensive_padding(example_inputs: Sequence[InputType]) -> contextlib.AbstractContextManager[None, None]: ...
def fake_tensor_prop(gm: GraphModule, example_inputs: Sequence[InputType], force_allow_non_fake_inputs: bool = False) -> torch._subclasses.FakeTensorMode: ...
def get_patched_config_dict(config_patches: str | dict[str, Any] | None = None) -> dict[str, Any]: ...
def with_fresh_cache_if_config() -> Generator[None, None, None]: ...

class _CompileFxKwargs(TypedDict, total=False):
    cudagraphs: BoxedBool | None
    static_input_idxs: Sequence[int]
    is_backward: bool
    graph_id: int | None
    cpp_wrapper: bool
    aot_mode: bool
    is_inference: bool
    layout_opt: bool | None
    extern_node_serializer: Callable[[list[ExternKernelNode]], Any] | None
    boxed_forward_device_index: BoxedDeviceIndex | None

class _CompileFxCallable(Protocol):
    def __call__(self, gm: GraphModule, example_inputs: Sequence[InputType], **kwargs: Unpack[_CompileFxKwargs]) -> OutputCode: ...

def compile_fx_inner(gm: GraphModule, example_inputs: Sequence[InputType], **kwargs: Unpack[_CompileFxKwargs]) -> OutputCode: ...

class FxCompile(ABC, metaclass=abc.ABCMeta):
    @abstractmethod
    def codegen_and_compile(self, gm: GraphModule, example_inputs: Sequence[InputType], inputs_to_check: Sequence[int], graph_kwargs: _CompileFxKwargs) -> OutputCode: ...

class _InProcessFxCompile(FxCompile):
    def codegen_and_compile(self, gm: GraphModule, example_inputs: Sequence[InputType], inputs_to_check: Sequence[int], graph_kwargs: _CompileFxKwargs) -> OutputCode: ...

def fx_codegen_and_compile(gm: GraphModule, example_inputs: Sequence[InputType], inputs_to_check: Sequence[int], **graph_kwargs: Unpack[_CompileFxKwargs]) -> OutputCode: ...
def get_input_idxs_to_check(inputs: Sequence[InputType], static_input_idxs: Sequence[int]) -> Sequence[int]: ...
def cudagraphify(model: Callable[..., Any], static_input_idxs: Sequence[int] = (), *, device_index: int, stack_traces: list[str | None], is_backward: bool, is_inference: bool, constants: tuple[torch.Tensor, ...] = (), placeholders: Sequence[PlaceholderInfo] = (), mutated_input_idxs: tuple[int, ...] = ()) -> Callable[..., Any]: ...
def static_input(x: torch.Tensor) -> torch.Tensor: ...
def index_expanded_dims_and_copy_(dst: torch.Tensor, src: torch.Tensor, expanded_dims: list[int]) -> None: ...
def cudagraphify_impl(model: Callable[..., Any], inputs: list[torch.Tensor], static_input_idxs: Sequence[int] = ()) -> Callable[[list[InputType]], Any]: ...
def compile_fx_aot(model_: GraphModule, example_inputs_: list[InputType], inner_compile: _CompileFxCallable = ..., config_patches: dict[str, str] | None = None) -> list[str] | str: ...
def fw_compiler_freezing(aot_autograd_model: GraphModule, aot_example_inputs: Sequence[InputType], dynamo_model: GraphModule, num_example_inputs: int, inner_compile: Callable[..., Any], cudagraphs: BoxedBool, graph_id: int, forward_device: BoxedDeviceIndex) -> Callable[[list[object]], Sequence[torch.Tensor]]: ...
def get_cpp_wrapper_config() -> dict[str, object]: ...
def get_cuda_device_context(gm: torch.fx.GraphModule) -> ContextManager[None]: ...
def compile_fx(model_: GraphModule, example_inputs_: Sequence[InputType], inner_compile: Callable[..., OutputCode] = ..., config_patches: dict[str, Any] | None = None, decompositions: dict[OpOverload, Callable[..., Any]] | None = None) -> Callable[[list[object]], Sequence[torch.Tensor]] | str | list[str]: ...
def graph_returns_tuple(gm: GraphModule) -> bool: ...
def make_graph_return_tuple(gm: GraphModule, inputs: Sequence[InputType], compile_gm: Callable[..., Any]) -> Callable[..., Any]: ...
def handle_dynamo_export_graph(gm: GraphModule, inputs: Sequence[InputType], compile_gm: Callable[..., Any]) -> Callable[..., Any]: ...
