import dataclasses
import torch
from _typeshed import Incomplete
from enum import Enum
from torch._dynamo.utils import counters as counters
from torch._inductor.utils import InputType as InputType
from typing import Any, Callable, Sequence

perf_hint_log: Incomplete
static_inputs_log: Incomplete
OutputType: Incomplete
ModelType = Callable[[list[InputType]], OutputType]

@dataclasses.dataclass(frozen=True)
class FunctionID:
    id: int

@dataclasses.dataclass(frozen=True)
class PlaceholderInfo:
    name: str
    stack_trace: str | None
    users: list[PlaceholderInfo]
    mutating_use_stack_trace: str | None

@dataclasses.dataclass(frozen=True)
class WrappedFunction:
    model: Callable[..., Any]
    static_input_idxs: Sequence[int]
    id: FunctionID
    constants: tuple[torch.Tensor, ...]
    placeholders: Sequence[PlaceholderInfo]
    mutated_input_idxs: Sequence[int]

def get_mutating_use_stack_trace_from_node(placeholder_node: torch.fx.Node) -> str | None: ...
def get_mutating_use_stack_trace(placeholder_info: PlaceholderInfo) -> str | None: ...
def to_placeholder_info(placeholder_node: torch.fx.Node) -> PlaceholderInfo: ...
def get_placeholder_info(graph: torch.fx.Graph) -> list[PlaceholderInfo]: ...
def format_default_skip_message(reason: str) -> str: ...
def get_mutation_stack_trace(placeholders: Sequence[PlaceholderInfo], mutation_indices: Sequence[int]) -> str: ...
def check_for_mutation(func: WrappedFunction, inputs: list[InputType], is_cuda_graph_recorded_tensor: Callable[[torch.Tensor], bool]) -> str | None: ...
def check_multiple_devices_or_any_cpu_nodes(device_node_mapping: dict[torch.device, torch.fx.Node]) -> str | None: ...
def check_lowering_disable_cudagraph(device_node_mapping: dict[torch.device, torch.fx.Node]): ...
def log_cudagraph_skip_and_bump_counter(msg) -> None: ...

@dataclasses.dataclass
class BoxedDeviceIndex:
    value: int | None
    def set(self, device_idx: int | None): ...

def check_for_mutation_ignore_cuda_graph_managed_tensor(gm: torch.fx.GraphModule, compiled_graph, static_input_idxs: Sequence[int]) -> str | None: ...
def get_placeholder_stack_trace(placeholder: PlaceholderInfo) -> str | None: ...

class CheckInvariantStatus(Enum):
    SUCCESS = 1
    CudagraphManagedIdxMismatch = 2
    StaticInputIdxMismatch = 3
    ExpectedDeadIndicesBeforeGraphMismatch = 4

def log_data_ptr_mismatch(placeholders: Sequence[PlaceholderInfo], inputs: list[InputType], recorded_data_ptr: Sequence[int | None], target_idxs: Sequence[int], mismatch: CheckInvariantStatus) -> str: ...
def maybe_warning_due_to_dynamic_shape(fn_cache: dict[tuple[int, ...], Callable[..., Any]], new_int_key: Any) -> bool: ...

@dataclasses.dataclass(frozen=True)
class CudagraphCachedInfo:
    placeholders: Sequence[PlaceholderInfo]
    stack_traces: list[str | None]
    cudagraph_fail_reasons: list[str]
