import abc
import dataclasses
import sympy
import torch
from ..utils._sympy.symbol import SymT as SymT, make_symbol as make_symbol
from .codegen.common import index_prevent_reordering as index_prevent_reordering
from .utils import VarRanges as VarRanges, get_dtype_size as get_dtype_size, reduction_num_outputs as reduction_num_outputs, sympy_index_symbol as sympy_index_symbol, sympy_str as sympy_str, sympy_subs as sympy_subs
from .virtualized import OpsHandler as OpsHandler, ReductionType as ReductionType, V as V
from _typeshed import Incomplete
from torch.fx.experimental.symbolic_shapes import free_unbacked_symbols as free_unbacked_symbols
from torch.utils._ordered_set import OrderedSet as OrderedSet
from typing import Any, Callable, Sequence, TypeVar

T = TypeVar('T')
log: Incomplete
is_indirect: Incomplete

class Dep(abc.ABC, metaclass=abc.ABCMeta):
    name: str
    index: sympy.Expr
    @abc.abstractmethod
    def rename(self, renames: dict[str, str]) -> Dep: ...
    @abc.abstractmethod
    def get_numel(self) -> sympy.Expr: ...
    @abc.abstractmethod
    def numbytes_hint(self): ...
    @abc.abstractmethod
    def has_unbacked_symbols(self) -> bool: ...
    @abc.abstractmethod
    def is_contiguous(self) -> bool: ...
    def normalize_with_stride_order(self, prefix: str = 't'): ...

@dataclasses.dataclass(frozen=True)
class MemoryDep(Dep):
    name: str
    index: sympy.Expr
    var_names: tuple[sympy.Symbol, ...]
    size: tuple[sympy.Expr, ...]
    mode: str | None = ...
    @property
    def num_vars(self): ...
    def decide_loop_order_to_match(self, other): ...
    def get_offset(self): ...
    def normalize(self) -> MemoryDep: ...
    def normalize_with_stride_order(self, prefix: str = 't'): ...
    @property
    def ranges(self) -> dict[sympy.Symbol, sympy.Expr]: ...
    def simplify_with_ranges(self): ...
    def get_numel(self) -> sympy.Expr: ...
    def rename(self, renames: dict[str, str]) -> MemoryDep: ...
    def numbytes_hint(self) -> int: ...
    def has_unbacked_symbols(self): ...
    def is_contiguous(self) -> bool: ...
    def stride1_for_last_dim(self, result_for_complex_expression: bool = True) -> bool: ...
    def is_scalar(self) -> bool: ...
    def is_indirect(self) -> bool: ...

@dataclasses.dataclass(frozen=True)
class StarDep(Dep):
    name: str
    mode: str | None = ...
    @property
    def index(self) -> None: ...
    def get_numel(self) -> sympy.Expr: ...
    def rename(self, renames: dict[str, str]) -> StarDep: ...
    def numbytes_hint(self): ...
    def has_unbacked_symbols(self): ...
    def is_contiguous(self) -> bool: ...
    def is_scalar(self) -> bool: ...
    def is_indirect(self) -> bool: ...

@dataclasses.dataclass(frozen=True)
class WeakDep(Dep):
    name: str
    mutating_buf: str
    @property
    def index(self) -> None: ...
    def get_numel(self) -> sympy.Expr: ...
    def rename(self, renames: dict[str, str]) -> WeakDep: ...
    def numbytes_hint(self): ...
    def has_unbacked_symbols(self): ...
    def is_contiguous(self) -> bool: ...

@dataclasses.dataclass(frozen=True)
class IndexExprDep:
    index: sympy.Expr
    var_names: tuple[sympy.Symbol, ...]
    size: tuple[sympy.Expr, ...]

@dataclasses.dataclass
class ReadWrites:
    reads: OrderedSet[Dep]
    writes: OrderedSet[Dep]
    index_exprs: OrderedSet[IndexExprDep]
    range_vars: list[sympy.Expr] | None = ...
    var_ranges: VarRanges | None = ...
    def rename(self, renames: dict[str, str]) -> ReadWrites: ...
    def with_read(self, dep: Dep | set[Dep]) -> ReadWrites: ...
    def merge(self, other: ReadWrites): ...
    @staticmethod
    def merge_list(read_writes: list['ReadWrites']): ...
    def remove_reads(self, rem_reads): ...
    def reads_and_writes(self): ...
    def buffer_names(self, ignore_integer_index: bool = True): ...

class _RecordLoadStoreInner(V.MockHandler):
    def __init__(self, var_ranges: VarRanges, normalize: bool) -> None: ...
    @staticmethod
    def drop_unused_symbols(index, var_names, sizes) -> None: ...
    def canonicalize(self, index: sympy.Expr) -> tuple[sympy.Expr, tuple[sympy.Symbol, ...], tuple[sympy.Expr, ...]]: ...
    def load(self, name: str, index: sympy.Expr) -> str: ...
    def load_seed(self, name: str, index: int): ...
    def store(self, name: str, index: sympy.Expr, value: str, mode: Incomplete | None = None) -> str: ...
    def store_reduction(self, name: str, index, value) -> str: ...
    def index_expr(self, index: sympy.Expr, dtype) -> str: ...
    def bucketize(self, values: T, boundaries: tuple[str, sympy.Expr, sympy.Expr, sympy.Expr], boundary_indices: T, indexing_dtype: torch.dtype, right: bool, sorter: tuple[str, sympy.Expr] | None = None, sorter_indices: T | None = None) -> None: ...

class RecordLoadStore(V.KernelFormatterHandler):
    def __init__(self, var_ranges: VarRanges, normalize: bool) -> None: ...

def var_builder(prefix: str) -> tuple[VarRanges, Callable[[sympy.Expr], sympy.Symbol]]: ...
def index_vars_no_squeeze(*argsizes: Sequence[sympy.Expr], prefix: str): ...
def index_vars_squeeze(*argsizes: Sequence[sympy.Expr], prefix: str = 'd'): ...
def extract_read_writes(fn: Callable[..., Any], *argsizes: Sequence[sympy.Expr], normalize: bool = False, prefix: str = 'd', hidden_args=()): ...
def extract_loop_body_with_args(fn, args, var_ranges, normalize: bool = False): ...
def extract_input_node_reduction_ranges(input_node: torch._inductor.ir.IRNode) -> tuple[list[sympy.Expr] | None, list[sympy.Expr] | None]: ...
def canonicalization_prefix(): ...

class FreeUnbackedSymbolsOpsHandler:
    symbols: OrderedSet[sympy.Symbol]
    def __init__(self) -> None: ...
    def __getattr__(self, name: str) -> Callable[..., Any]: ...
    def indirect_indexing(self, index_var, size, check: bool = True, wrap_neg: bool = True) -> sympy.Symbol: ...
    def frexp(self, x): ...
    def scan(self, dtypes, combine_fn, values): ...
    def sort(self, dtypes, values, stable, descending): ...
    def reduction(self, dtype: torch.dtype, src_dtype: torch.dtype, reduction_type: ReductionType, value: None | tuple[None, ...]) -> None | tuple[None, ...]: ...
    def masked(self, mask, body, other) -> None: ...

def extract_free_unbacked_symbols(fn: Callable[..., Any], index, rindex: Incomplete | None = None): ...
