import sympy
import torch
import torch.fx
from . import config as config, ir as ir, metrics as metrics
from .codegen.common import BackendFeature as BackendFeature, DeviceOpOverrides as DeviceOpOverrides, get_backend_features as get_backend_features, get_device_op_overrides as get_device_op_overrides, get_wrapper_codegen_for_device as get_wrapper_codegen_for_device, init_backend_registration as init_backend_registration
from .codegen.wrapper import PythonWrapperCodegen as PythonWrapperCodegen
from .exc import CppWrapperCodegenError as CppWrapperCodegenError, LoweringException as LoweringException, MissingOperatorWithDecomp as MissingOperatorWithDecomp, MissingOperatorWithoutDecomp as MissingOperatorWithoutDecomp
from .ir import Constant as Constant, DonatedBuffer as DonatedBuffer, FixedLayout as FixedLayout, InputBuffer as InputBuffer, Pointwise as Pointwise, Reduction as Reduction, StorageBox as StorageBox, TensorBox as TensorBox, TorchBindObject as TorchBindObject, get_device_type as get_device_type
from .lowering import FALLBACK_ALLOW_LIST as FALLBACK_ALLOW_LIST, constrain_to_fx_strides as constrain_to_fx_strides, fallback_handler as fallback_handler, fallback_node_due_to_unsupported_type as fallback_node_due_to_unsupported_type, lowerings as lowerings, make_fallback as make_fallback, maybe_layout_constraints as maybe_layout_constraints, needs_realized_inputs as needs_realized_inputs, require_contiguous as require_contiguous, unsupported_output_tensor as unsupported_output_tensor
from .runtime import autotune_cache as autotune_cache
from .runtime.autotune_cache import AutotuneCacheBundler as AutotuneCacheBundler
from .scheduler import BaseSchedulerNode as BaseSchedulerNode
from .sizevars import SizeVarAllocator as SizeVarAllocator
from .utils import convert_shape_to_inductor as convert_shape_to_inductor, gather_origins as gather_origins, get_cloned_parameter_buffer_name as get_cloned_parameter_buffer_name, get_donated_idxs as get_donated_idxs, get_sympy_Expr_dtype as get_sympy_Expr_dtype, is_same_tensor as is_same_tensor, maybe_get_suppress_shape_guards_ctx as maybe_get_suppress_shape_guards_ctx, normalize_name as normalize_name, should_assume_input_aligned as should_assume_input_aligned
from .virtualized import NullHandler as NullHandler, V as V
from _typeshed import Incomplete
from sympy import Expr as Expr
from torch import Tensor as Tensor, device as device
from torch._decomp import get_decompositions as get_decompositions
from torch._dynamo.utils import defake as defake, dynamo_timed as dynamo_timed
from torch._higher_order_ops.effects import _EffectType
from torch._inductor.codecache import output_code_log as output_code_log
from torch._inductor.fb.utils import log_module_code as log_module_code
from torch._logging import LazyString as LazyString, trace_structured as trace_structured
from torch._prims_common import make_channels_last_strides_for as make_channels_last_strides_for
from torch._subclasses.fake_tensor import FakeTensor as FakeTensor
from torch.fx import GraphModule as GraphModule
from torch.fx.experimental._backward_state import BackwardState as BackwardState
from torch.fx.experimental.sym_node import magic_methods as magic_methods, method_to_operator as method_to_operator
from torch.fx.experimental.symbolic_shapes import RuntimeAssert as RuntimeAssert, ShapeEnv as ShapeEnv, SymTypes as SymTypes, SympyBoolean as SympyBoolean, free_unbacked_symbols as free_unbacked_symbols, has_free_symbols as has_free_symbols, resolve_unbacked_bindings as resolve_unbacked_bindings
from torch.fx.graph import Graph as Graph
from torch.fx.node import Node as Node
from torch.utils._mode_utils import no_dispatch as no_dispatch
from torch.utils._ordered_set import OrderedSet as OrderedSet
from torch.utils._sympy.numbers import int_oo as int_oo
from types import ModuleType
from typing import Any, Callable, DefaultDict, Iterable, Iterator, NoReturn, Sequence

log: Incomplete
perf_hint_log: Incomplete
aten: Incomplete

def supported_dtype_of_cpp_wrapper(dtype: torch.dtype, device_type: str) -> bool: ...
def may_get_constant_buffer_dtype(constant_buffer: sympy.Expr) -> torch.dtype | None: ...
def is_magic_method(op: Any) -> bool: ...
def getattr_recursive(obj: GraphModule, target: str) -> Tensor | torch._C.ScriptObject | GraphModule: ...
def get_user_visible_output_strides(g: Graph) -> dict[Node, tuple[int, ...]]: ...
def mark_nodes_dislike_padding(g: Graph, user_visible_output_strides: dict[Node, tuple[int, ...]]) -> None: ...

class GraphLowering(torch.fx.Interpreter):
    graph_outputs: list[ir.IRNode]
    def symbolic_sizes_strides(self, ex: torch.Tensor) -> tuple[Sequence[int | Expr], Sequence[int | Expr]]: ...
    def static_sizes_strides(self, ex: torch.Tensor) -> tuple[list[sympy.Expr], list[sympy.Expr]]: ...
    example_inputs: Incomplete
    layout_opt: Incomplete
    num_channels_last_conv: int
    is_inference: Incomplete
    is_backward: Incomplete
    is_const_graph: Incomplete
    const_code: Incomplete
    const_module: Incomplete
    inputs_to_check: Incomplete
    extra_traceback: bool
    reuse_shape_env: bool
    ras_by_symbol: dict[sympy.Symbol | None, list[RuntimeAssert]]
    bound_unbacked_symbols: OrderedSet[sympy.Symbol]
    sizevars: Incomplete
    graph_input_names: list[str]
    graph_inputs: dict[str, TensorBox]
    graph_inputs_original: dict[str, InputBuffer]
    zero_dim_cpu_tensor_list: OrderedSet[str]
    device_types: OrderedSet[str]
    device_idxs: OrderedSet[int]
    device_type: str
    buffers: list[ir.Buffer]
    operations: list[ir.Operation]
    const_output_index: dict[str, int]
    folded_constants: OrderedSet[str]
    constants: dict[str, torch.Tensor]
    torchbind_constants: dict[str, torch._C.ScriptObject]
    seen_subgraphs: dict[str, ir.Subgraph]
    constant_reprs: dict[str, str]
    removed_operations: OrderedSet[str]
    removed_buffers: OrderedSet[str]
    removed_inplace_buffers: OrderedSet[str]
    mutated_buffers: OrderedSet[str]
    never_reuse_buffers: OrderedSet[str]
    inplaced_to_remove: OrderedSet[str]
    device_ops: DeviceOpOverrides
    wrapper_code: PythonWrapperCodegen
    extern_kernel_nodes: list[ir.ExternKernelNode]
    extern_node_serializer: Callable[[list[ir.ExternKernelNode]], Any]
    current_node: torch.fx.Node
    lists: dict[str, list[str]]
    mutated_inputs: OrderedSet[str]
    mutated_input_idxs: list[int]
    name_to_buffer: dict[str, ir.Buffer]
    name_to_users: DefaultDict[str, list[ir.IRNode]]
    name_to_op: dict[str, ir.Operation]
    creation_time: Incomplete
    name: Incomplete
    cpp_wrapper: Incomplete
    record_multi_kernel_choice: Incomplete
    multi_kernel_to_choice: dict[str, str]
    aot_mode: Incomplete
    graph_id: Incomplete
    post_grad_graph_id: Incomplete
    scheduler: torch._inductor.scheduler.Scheduler
    current_device: torch.device | None
    nodes_prefer_channels_last: Incomplete
    user_visible_output_strides: Incomplete
    cache_key: str
    cache_path: str
    cache_linemap: list[tuple[int, str]]
    disable_cudagraphs_reason: str | None
    device_node_mapping: dict[torch.device, torch.fx.Node]
    orig_gm: torch.fx.GraphModule
    dynamo_flat_name_to_original_fqn: Incomplete
    allocated_constant_name: dict[str, str]
    get_backend_features: Incomplete
    effectful_ops: dict[_EffectType, ir.Buffer]
    aligned_inputs: OrderedSet[str]
    no_fuse_buffer_names: OrderedSet[str]
    all_codegen_kernel_names: OrderedSet[str]
    workspace_id: Incomplete
    placeholder_idx: int
    bw_donated_idxs: Incomplete
    def __init__(self, gm: torch.fx.GraphModule, example_inputs: Sequence[object] | None = None, shape_env: ShapeEnv | None = None, graph_id: int | None = None, cpp_wrapper: bool = False, aot_mode: bool = False, layout_opt: bool | None = None, extern_node_serializer: Callable[[list[ir.ExternKernelNode]], Any] | None = None, is_inference: bool = False, is_backward: bool = False, is_const_graph: bool = False, const_output_index: dict[str, int] | None = None, const_code: str | None = None, const_module: GraphLowering | None = None, name: str | None = None, inputs_to_check: Sequence[int] | None = None) -> None: ...
    def has_feature(self, device: torch._inductor.ir.IRNode | device | None, feature: BackendFeature) -> bool: ...
    def get_current_device_or_throw(self) -> torch.device: ...
    def set_current_device(self, device: torch.device) -> Iterator[None]: ...
    def get_training_phase(self) -> str: ...
    @staticmethod
    def decide_layout_opt(gm: GraphModule, *, is_inference: bool) -> bool: ...
    def qualify_name(self, name: str) -> str: ...
    def make_subgraph(self, gm: torch.fx.GraphModule, example_inputs: list[torch.Tensor], subgraph_name: str) -> SubgraphLowering: ...
    def find_nodes_prefer_channels_last(self) -> OrderedSet[Node]: ...
    def warn_fallback(self, name: str) -> None: ...
    def add_device_info(self, device: torch.device) -> None: ...
    @property
    def fake_mode(self) -> torch._subclasses.fake_tensor.FakeTensorMode: ...
    def try_get_buffer(self, buffer_name: str) -> ir.TensorBox | ir.Buffer | None: ...
    def add_symbol_graph_input(self, symbol: sympy.Expr) -> None: ...
    def get_buffer(self, buffer_name: str) -> ir.TensorBox | ir.Buffer: ...
    def get_dtype(self, buffer_name: str) -> torch.dtype: ...
    def get_numel(self, buffer_name: str) -> int | Expr: ...
    def run(self, *args: Any) -> Any: ...
    def register_operation(self, op: ir.Operation) -> str: ...
    def register_buffer(self, buffer: ir.Buffer, *, set_name: bool = False) -> str: ...
    def register_operation_list(self, operation_names: list[str]) -> str: ...
    def register_users_of(self, node_output: Iterable[ir.IRNode] | ir.IRNode) -> None: ...
    def mark_buffer_mutated(self, name: str) -> None: ...
    def get_original_value_of_constant(self, name: str) -> torch.Tensor: ...
    def allocate_non_dup_const_name(self, name: str | None, data: Tensor) -> str: ...
    def add_tensor_constant(self, data: Tensor, name: str | None = None) -> TensorBox: ...
    def constant_name(self, name: str, device_override: torch.device | None) -> str: ...
    def placeholder(self, target: str, args: tuple[object], kwargs: dict[str, object]) -> Expr | TensorBox | None: ...
    def call_function(self, target: Callable, args: Any, kwargs: dict[str, Any]) -> Any: ...
    @staticmethod
    def can_inline_constant(t: torch.Tensor) -> bool: ...
    def get_attr(self, target: str, args: tuple[()], kwargs: dict[str, object]) -> Constant | TensorBox | ir.Subgraph | TorchBindObject: ...
    def call_module(self, target: Any, args: Any, kwargs: Any) -> NoReturn: ...
    def call_method(self, target: Any, args: Any, kwargs: Any) -> NoReturn: ...
    def output(self, target: str, args: tuple[object], kwargs: dict[str, object]) -> None: ...
    def finalize(self) -> None: ...
    def set_current_node(self, node: torch.fx.Node): ...
    def try_match_insignificant_strides(self, tensor: ir.TensorBox | ir.BaseView, meta_strides_inp: tuple[int | torch.SymInt, ...]) -> ir.TensorBox | ir.BaseView: ...
    def propagate_mutation(self, fx_node: torch.fx.Node, old_args: tuple[Any], old_kwargs: dict[str, Any], new_args: tuple[Any], new_kwargs: dict[str, Any]) -> None: ...
    def run_node(self, n: torch.fx.Node) -> object: ...
    def validate_can_generate_cpp_wrapper(self) -> None: ...
    def init_wrapper_code(self, is_subgraph: bool = False, subgraph_name: str | None = None, parent_wrapper_code: PythonWrapperCodegen | None = None) -> None: ...
    def codegen_with_cpp_wrapper(self) -> tuple[str, list[tuple[int, Node]]]: ...
    def codegen(self) -> tuple[str, list[tuple[int, Node]]]: ...
    def codegen_subgraph(self, parent_graph: GraphLowering) -> None: ...
    def count_bytes(self) -> tuple[int, list[tuple[BaseSchedulerNode, int]], list[tuple[BaseSchedulerNode, float]]]: ...
    @staticmethod
    def save_output_code(code: str) -> None: ...
    def compile_to_module(self) -> ModuleType: ...
    def get_output_names(self) -> list[str]: ...
    def is_unspec_arg(self, name: str) -> bool: ...

class SubgraphLowering(GraphLowering):
    parent: Incomplete
    def __init__(self, parent: GraphLowering, *args: Any, **kwargs: Any) -> None: ...
    def init_wrapper_code(self, is_subgraph: bool = False, subgraph_name: str | None = None, parent_wrapper_code: PythonWrapperCodegen | None = None) -> None: ...
