import sympy
import torch
from .sizevars import evaluate_expr as evaluate_expr
from .utils import generate_assert as generate_assert
from .virtualized import V as V
from _typeshed import Incomplete
from dataclasses import dataclass
from torch._prims_common import dtype_to_type as dtype_to_type, is_integer_dtype as is_integer_dtype
from torch.utils._sympy.functions import FloorDiv as FloorDiv, ModularIndexing as ModularIndexing, Where as Where
from torch.utils._sympy.value_ranges import ValueRanges as ValueRanges, bound_sympy as bound_sympy
from typing import Any, Callable, Literal, overload
from typing_extensions import TypeAlias

def upper_bound(val: _ExprType): ...

@dataclass
class TypedExpr:
    expr: _ExprType
    dtype: torch.dtype
    def is_constant(self): ...
    def __post_init__(self) -> None: ...

class SymPyOps:
    @staticmethod
    def identity(value: Any) -> Any: ...
    @staticmethod
    def constant(value: int | float | bool, dtype: torch.dtype) -> TypedExpr: ...
    @staticmethod
    def index_expr(value: sympy.Expr | int, dtype: torch.dtype) -> TypedExpr: ...
    @staticmethod
    def to_dtype(value: TypedExpr, dtype: torch.dtype, src_dtype: torch.dtype | None = None, use_compute_types: bool = False) -> TypedExpr: ...
    @staticmethod
    def abs(x: TypedExpr) -> TypedExpr: ...
    @staticmethod
    def square(x: TypedExpr) -> TypedExpr: ...
    @staticmethod
    def add(x: TypedExpr, y: TypedExpr) -> TypedExpr: ...
    @staticmethod
    def sub(x: TypedExpr, y: TypedExpr) -> TypedExpr: ...
    @staticmethod
    def mul(x: TypedExpr, y: TypedExpr) -> TypedExpr: ...
    @staticmethod
    def neg(x: TypedExpr) -> TypedExpr: ...
    @staticmethod
    def floordiv(x: TypedExpr, y: TypedExpr) -> TypedExpr: ...
    @staticmethod
    def mod(x: TypedExpr, y: TypedExpr) -> TypedExpr | None: ...
    @staticmethod
    def remainder(x: TypedExpr, y: TypedExpr) -> TypedExpr | None: ...
    @staticmethod
    def minimum(x: TypedExpr, y: TypedExpr) -> TypedExpr: ...
    @staticmethod
    def maximum(x: TypedExpr, y: TypedExpr) -> TypedExpr: ...

@dataclass
class IndexPropVar:
    value: Any
    is_symbolic: bool = ...
    @staticmethod
    def new_symbolic(expr: TypedExpr) -> IndexPropVar: ...
    def __post_init__(self) -> None: ...

IndexPropResult: TypeAlias

class IndexPropagation:
    shape_env: Incomplete
    var_to_range: Incomplete
    indirect_var_ranges: Incomplete
    axioms: Incomplete
    def __init__(self, inner: Any, iter_ranges: dict[sympy.Symbol, sympy.Expr], indirect_var_ranges: dict[sympy.Symbol, sympy.Expr]) -> None: ...
    def materialize_expr(self, expr: sympy.Expr, dtype: torch.dtype) -> Any: ...
    def unwrap(self, a: Any | IndexPropVar) -> Any: ...
    def wrap(self, a) -> IndexPropResult: ...
    @overload
    def fallback(self, name: Literal['indirect_indexing'], args: tuple[Any, ...], kwargs: dict[str, Any]) -> IndexPropVar: ...
    @overload
    def fallback(self, name: str, args: tuple[Any, ...], kwargs: dict[str, Any]) -> IndexPropResult: ...
    def propagate_sympy(self, name: str, args: tuple[Any, ...], kwargs: dict[str, Any]) -> IndexPropResult: ...
    def __getattr__(self, name: str) -> Callable[..., IndexPropResult]: ...
    def statically_true(self, e): ...
    def indirect_indexing(self, index: Any | IndexPropVar, size: Any, check: bool = True, wrap_neg: bool = True) -> Any: ...
