import dataclasses
import sympy
import torch
import torch._export.serde.schema as export_schema
import torch.fx
import torch.utils._pytree as pytree
from . import config as config, dependencies as dependencies
from .codegen.common import BackendFeature as BackendFeature, index_prevent_reordering as index_prevent_reordering
from .codegen.cuda.cuda_template import CUDATemplate as CUDATemplate
from .dependencies import Dep as Dep, extract_free_unbacked_symbols as extract_free_unbacked_symbols, extract_input_node_reduction_ranges as extract_input_node_reduction_ranges, extract_read_writes as extract_read_writes, var_builder as var_builder
from .graph import GraphLowering as GraphLowering
from .loop_body import LoopBody as LoopBody
from .ops_handler import OpCountResult as OpCountResult, OpCounterCSE as OpCounterCSE
from .runtime.benchmarking import benchmarker as benchmarker
from .runtime.hints import DeviceProperties as DeviceProperties, ReductionHint as ReductionHint
from .utils import IndentedBuffer as IndentedBuffer, argsort as argsort, argsort_sym as argsort_sym, cache_on_self as cache_on_self, ceildiv as ceildiv, convert_shape_to_inductor as convert_shape_to_inductor, convert_shape_to_symint as convert_shape_to_symint, developer_warning as developer_warning, get_kernel_metadata as get_kernel_metadata, ir_dataclass as ir_dataclass, is_dynamic as is_dynamic, is_gpu as is_gpu, sympy_dot as sympy_dot, sympy_index_symbol as sympy_index_symbol, sympy_index_symbol_with_prefix as sympy_index_symbol_with_prefix, sympy_product as sympy_product, sympy_subs as sympy_subs
from .virtualized import OpsValue as OpsValue, V as V, ops as ops
from _typeshed import Incomplete
from enum import Enum
from sympy import Expr, Integer, Symbol as Symbol
from torch._dynamo.utils import identity as identity
from torch._export.serde.serialize import GraphModuleSerializer as GraphModuleSerializer
from torch._higher_order_ops.auto_functionalize import can_auto_functionalize as can_auto_functionalize
from torch._inductor import metrics as metrics
from torch._prims_common import StrideType as StrideType, compute_required_storage_length as compute_required_storage_length, is_boolean_dtype as is_boolean_dtype, is_float_dtype as is_float_dtype, make_channels_last_strides_for as make_channels_last_strides_for
from torch._subclasses.fake_tensor import get_schema_info as get_schema_info
from torch.fx.experimental.symbolic_shapes import CallMethodKey as CallMethodKey, DivideByKey as DivideByKey, ShapeEnv as ShapeEnv, SymTypes as SymTypes, compute_unbacked_bindings as compute_unbacked_bindings, free_unbacked_symbols as free_unbacked_symbols, rebind_unbacked as rebind_unbacked, resolve_unbacked_bindings as resolve_unbacked_bindings
from torch.fx.node import Node as Node
from torch.utils._ordered_set import OrderedSet as OrderedSet
from torch.utils._sympy.functions import CleanDiv as CleanDiv, FloorDiv as FloorDiv, ModularIndexing as ModularIndexing
from torch.utils._sympy.symbol import SymT as SymT
from typing import Any, Callable, Generator, Iterable, Literal, Sequence, overload
from typing_extensions import Never

log: Incomplete
indent: Incomplete
aten: Incomplete

def validate_ir(node_or_nodes: _NodeOrNodes | None) -> None: ...
def ops_wrapper(name: str) -> Callable[..., OpsValue]: ...
def inverse_reorder(order: Sequence[int]) -> Callable[[Sequence[_T]], Sequence[_T]]: ...
def same_reorder(order: Sequence[int]) -> Callable[[Sequence[_T]], Sequence[_T]]: ...
def fuse_reindexing(reindex1: Callable[[Sequence[_U]], Sequence[_V]], reindex2: Callable[[Sequence[_T]], Sequence[_U]]) -> Callable[[Sequence[_T]], Sequence[_V]]: ...

NHWC_STRIDE_ORDER: Incomplete
NHWDC_STRIDE_ORDER: Incomplete

def get_fill_order(seq: Sequence[int | torch.SymInt | Expr], shape_env: ShapeEnv | None = None) -> Sequence[int]: ...
def stride_order2fill_order(order: Sequence[int | Integer]) -> Sequence[int]: ...
def get_stride_order(seq: Sequence[int | torch.SymInt | Expr], shape_env: ShapeEnv | None = None) -> Sequence[int]: ...
@overload
def ir_node_to_tensor(x: Literal[None], guard_shape: bool = True) -> None: ...
@overload
def ir_node_to_tensor(x: IRNode, guard_shape: bool = True) -> torch.Tensor: ...
def may_convert_to_optional(value: Sequence[_T] | None) -> Sequence[_T | None] | None: ...
def get_device_type(x: IRNode | OutputSpec | torch.device | None | str) -> str | None: ...
def is_triton(x: IRNode | torch.device | None | str) -> bool: ...
def is_cpu(x: IRNode | torch.device | None | str) -> bool: ...

class IRNode:
    origins: OrderedSet[Any]
    traceback: list[str] | None
    origin_node: torch.fx.Node | None
    @staticmethod
    def current_origins(origins: OrderedSet[Node]) -> Generator[None, None, None]: ...
    def __post_init__(self) -> None: ...
    def get_read_names(self) -> OrderedSet[str]: ...
    def get_traceback(self) -> list[str] | None: ...
    def get_origin_node(self) -> torch.fx.Node | None: ...
    def get_defining_op(self) -> Operation | None: ...
    def common_repr(self, shorten: bool = True) -> Sequence[str]: ...
    def str_helper(self, lines: Sequence[object], shorten: bool = True, multiline: bool = True) -> str: ...
    def get_dtype(self) -> torch.dtype: ...
    def maybe_get_dtype(self) -> torch.dtype | None: ...
    def get_layout(self) -> Layout: ...
    def maybe_get_layout(self) -> Layout | None: ...
    def get_output_spec(self) -> OutputSpec: ...
    def maybe_get_output_spec(self) -> OutputSpec | None: ...
    def has_tensor_output(self) -> bool: ...
    def get_size(self) -> Sequence[Expr]: ...
    def maybe_get_size(self) -> Sequence[_IntLike] | None: ...
    @property
    def shape(self) -> _IntLike | sympy.Rel | Sequence[_IntLike]: ...
    def get_numel(self) -> Expr: ...
    def is_zero_elements(self) -> bool: ...
    def realize(self) -> str | None: ...
    def codegen_reference(self, writer: IndentedBuffer | None = None) -> str: ...
    def get_device(self) -> torch.device | None: ...
    def get_device_or_error(self) -> torch.device: ...
    def has_exceeded_max_reads(self) -> bool: ...
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...
    def make_indexer(self) -> Callable[[Sequence[Expr]], Expr]: ...
    def get_stride(self) -> Sequence[_IntLike]: ...
    def maybe_get_stride(self) -> Sequence[_IntLike] | None: ...
    def get_name(self) -> str: ...
    def maybe_get_name(self) -> str | None: ...
    def has_large_inner_fn(self, threshold: int | None = None) -> bool: ...
    def mark_reuse(self, users: int) -> None: ...
    def realize_hint(self) -> None: ...
    def unwrap_view(self) -> IRNode: ...
    def freeze_layout(self) -> None: ...
    def freeze_layout_with_stride_order(self, order: list[int], allow_padding: bool = False) -> None: ...
    def freeze_layout_with_fill_order(self, order: list[int]) -> None: ...
    def freeze_layout_with_same_order(self, stride: list[_IntLike]) -> None: ...
    def freeze_layout_with_exact_strides(self, exact_strides: list[_IntLike], allow_padding: bool = False) -> None: ...
    def get_read_writes(self) -> dependencies.ReadWrites: ...
    def get_reads(self) -> OrderedSet[Dep]: ...
    def num_reads(self) -> int: ...
    def get_storage_numel(self) -> _IntLike: ...
    def get_unbacked_symbol_uses(self) -> OrderedSet[Symbol]: ...
    def get_reduction_type(self) -> str | None: ...
    def get_reduction_size(self) -> Sequence[sympy.Expr]: ...
    def is_extern(self) -> bool: ...
    def is_no_op(self) -> bool: ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...
    def get_mutation_names(self) -> Sequence[str]: ...
    def get_operation_name(self) -> str: ...
    def get_inputs_that_alias_output(self) -> Sequence[str]: ...
    @property
    def dtype(self) -> torch.dtype: ...

@ir_dataclass(frozen=False)
class Operation:
    operation_name: str | None = ...
    def __post_init__(self) -> None: ...
    def get_device(self) -> torch.device | None: ...
    def get_origin_node(self) -> torch.fx.Node | None: ...
    def get_origins(self) -> OrderedSet[Any]: ...
    def get_operation_name(self) -> str: ...
    def is_extern(self) -> bool: ...
    def is_no_op(self) -> bool: ...
    def get_read_writes(self) -> dependencies.ReadWrites: ...
    def is_user_of(self, name: str) -> bool: ...
    def get_read_names(self) -> OrderedSet[str]: ...
    def get_reads(self) -> OrderedSet[Dep]: ...
    def get_outputs(self) -> list[Buffer]: ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    def get_unbacked_symbol_uses(self) -> OrderedSet[sympy.Symbol]: ...
    def get_workspace_size(self) -> int: ...

@ir_dataclass
class Loops(IRNode):
    device: torch.device
    dtype: torch.dtype
    inner_fn: Callable[..., Any]
    ranges: Sequence[_IntLike]
    def get_unbacked_symbol_uses(self) -> OrderedSet[Symbol]: ...
    def __post_init__(self) -> None: ...
    def get_device(self) -> torch.device | None: ...
    def get_origin_node(self) -> torch.fx.Node | None: ...
    def get_size(self) -> Sequence[Expr]: ...
    def get_pointwise_size(self) -> Sequence[Expr]: ...
    @classmethod
    def create(cls, *args: Any, **kwargs: Any) -> TensorBox: ...
    def inner_fn_opcount(self) -> OpCountResult: ...
    def inner_fn_args(self) -> Sequence[Sequence[_IntLike]]: ...
    def inner_fn_str(self) -> str: ...
    def has_large_inner_fn(self, threshold: int | None = None) -> bool: ...
    def inner_fn_free_unbacked_symbols(self) -> set[Symbol]: ...
    def get_reads(self) -> OrderedSet[Dep]: ...
    def get_read_names(self) -> OrderedSet[str]: ...
    def num_reads(self) -> int: ...
    def get_reduction_size(self) -> Sequence[sympy.Expr]: ...
    def get_reduction_type(self) -> str | None: ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...

def nop_loader_fn(idx: Expr | Sequence[Expr], *, dtype: torch.dtype) -> OpsValue: ...

@ir_dataclass
class Pointwise(Loops):
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...
    def get_reduction_size(self) -> Sequence[sympy.Expr]: ...
    def get_reduction_type(self) -> str | None: ...
    def store_output(self, output_name: str | None, indexer: Callable[[Sequence[Expr]], Never], vars: Sequence[Expr]) -> OpsValue: ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...

@ir_dataclass
class Scatter(Pointwise):
    output_indexer: Callable[[Sequence[Expr]], Expr]
    scatter_mode: str | None = ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...
    def store_output(self, output_name: str | None, indexer: Callable[[Sequence[Expr]], Never], vars: Sequence[Expr]) -> OpsValue: ...

REDUCTION_COMBINE_FN: dict[str, Callable[..., OpsValue]]

def get_reduction_combine_fn(reduction_type: str, dtype: torch.dtype, arg_break_ties_left: bool = True) -> Callable[..., object]: ...
def significant_strides_equal(strides1: Sequence[_IntLike], strides2: Sequence[_IntLike], size: Sequence[_IntLike]) -> bool: ...

@ir_dataclass
class Reduction(Loops):
    reduction_ranges: Sequence[_IntLike]
    reduction_type: str
    src_dtype: torch.dtype
    reduction_hint: ReductionHint
    def get_unbacked_symbol_uses(self) -> OrderedSet[Symbol]: ...
    def get_reduction_size(self) -> Sequence[sympy.Expr]: ...
    def get_reduction_type(self) -> str | None: ...
    def store_reduction(self, output_name: str | None, indexer: Callable[[Sequence[Expr]], Never], vars: Sequence[Expr], reduction_vars: Sequence[Symbol]) -> OpsValue: ...
    def index_length(self) -> int: ...
    def inner_fn_args(self) -> Sequence[Sequence[Expr]]: ...
    def inner_fn_free_unbacked_symbols(self) -> set[Symbol]: ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...
    @staticmethod
    def num_splits(device: torch.device, dst_dtype: torch.dtype, src_dtype: torch.dtype, inner_fn: Callable[..., OpsValue], ranges: Sequence[_IntLike], reduction_ranges: Sequence[_IntLike], reduction_type: str, reduction_numel: Expr, input_node: IRNode | None = None) -> tuple[ReductionHint, _IntLike]: ...
    @classmethod
    def create(cls, device: torch.device, dst_dtype: torch.dtype, src_dtype: torch.dtype, inner_fn: Callable[..., Any], ranges: Sequence[Expr], reduction_ranges: Sequence[Expr], reduction_type: str, reduction_hint: ReductionHint = ..., input_node: IRNode | None = None) -> TensorBox: ...
    @staticmethod
    def default_accumulator(reduction_type: str, dtype: torch.dtype) -> _NumLike | Sequence[_NumLike]: ...
    @staticmethod
    def default_value(reduction_type: str, dtype: torch.dtype) -> _NumLike | Sequence[_NumLike]: ...
    @classmethod
    def create_multilayer_helper(cls, device: torch.device, dst_dtype: torch.dtype, src_dtype: torch.dtype, wrapper_fn: Callable[..., Any], original_ranges: Sequence[Expr], original_reduction_ranges: Sequence[Expr], new_ranges: list[Expr], new_reduction_ranges: list[Integer], reduction_type: str, split: _IntLike, reduction_hint: ReductionHint) -> TensorBox: ...
    @classmethod
    def create_multilayer(cls, device: torch.device, dst_dtype: torch.dtype, src_dtype: torch.dtype, inner_fn: Callable[..., Any], ranges: Sequence[Expr], reduction_ranges: Sequence[Expr], reduction_type: str, split: _IntLike, reduction_hint: ReductionHint) -> TensorBox: ...
    @classmethod
    def create_multilayer_existing_ranges(cls, device: torch.device, dst_dtype: torch.dtype, src_dtype: torch.dtype, inner_fn: Callable[..., Any], original_ranges: Sequence[Expr], original_reduction_ranges: Sequence[Expr], new_ranges: list[Integer], new_reduction_ranges: list[Integer], reduction_type: str, reduction_hint: ReductionHint) -> TensorBox: ...

class WelfordReduction(Reduction):
    output_index: int
    def __init__(self, device: torch.device, dtype: torch.dtype, inner_fns: Sequence[Callable[[Sequence[Expr], Sequence[Expr]], OpsValue]], ranges: Sequence[Integer], reduction_ranges: Sequence[Integer], reduction_type: str, reduction_hint: ReductionHint, output_index: int) -> None: ...
    def store_reduction(self, output_name: str | None, indexer: Callable[[Sequence[Expr]], Never], vars: Sequence[Expr], reduction_vars: Sequence[Symbol]) -> OpsValue: ...
    @classmethod
    def create(cls, device: torch.device, dtype: torch.dtype, inner_fns: Sequence[Callable[..., Any]], ranges: list[Integer], reduction_ranges: list[Integer], reduction_type: str, reduction_hint: ReductionHint = ...) -> Sequence[TensorBox]: ...
    @staticmethod
    def default_value(reduction_type: str, dtype: torch.dtype) -> _NumLike | Sequence[_NumLike]: ...
    @classmethod
    def create_multilayer(cls, device: torch.device, dtype: torch.dtype, inner_fns: Sequence[Callable[..., Any]], ranges: list[Integer], reduction_ranges: list[Integer], reduction_type: str, split: _IntLike, reduction_hint: ReductionHint) -> Sequence[TensorBox]: ...

@ir_dataclass
class Scan(Loops):
    scan_ranges: list[Integer]
    size: list[Integer]
    combine_fn: Callable[[tuple[Any, ...], tuple[Any, ...]], tuple[Any, ...]]
    reindex: Callable[[Sequence[_IntLike], Sequence[_IntLike]], Sequence[_IntLike]]
    reduction_hint: ReductionHint
    output_index: int
    dtypes: tuple[torch.dtype, ...]
    inner_fns: tuple[Callable[..., Any], ...]
    def get_unbacked_symbol_uses(self) -> OrderedSet[Symbol]: ...
    def __post_init__(self) -> None: ...
    def store_reduction(self, output_name: str | None, indexer: Callable[[Sequence[_IntLike]], Never], vars: Sequence[Expr], scan_vars: Sequence[Symbol]) -> OpsValue: ...
    def get_reduction_type(self) -> str | None: ...
    def get_reduction_size(self) -> Sequence[sympy.Expr]: ...
    def get_size(self) -> Sequence[Expr]: ...
    def get_pointwise_size(self) -> Sequence[Expr]: ...
    def index_length(self) -> int: ...
    def inner_fn_args(self) -> Sequence[Sequence[_IntLike]]: ...
    def inner_fn_free_unbacked_symbols(self) -> set[Symbol]: ...
    @classmethod
    def create(cls, device: torch.device, dtypes: tuple[torch.dtype, ...], inner_fns: tuple[Callable[[Sequence[Expr]], Any], ...], size: list[Integer], axis: int, combine_fn: Callable[[tuple[Any, ...], tuple[Any, ...]], tuple[Any, ...]], reduction_hint: ReductionHint = ..., *, can_fallback_to_aten: bool = True, **kwargs: Any) -> Sequence[TensorBox | None]: ...
    @classmethod
    def num_splits(cls, device: torch.device, dtype: torch.dtype, inner_fn: Callable[[Sequence[Expr]], OpsValue], axis: int, pointwise_ranges: list[Integer], scan_ranges: list[Integer], combine_fn: Callable[[tuple[Any, ...], tuple[Any, ...]], tuple[Any, ...]], scan_numel: Expr) -> tuple[ReductionHint, _IntLike]: ...

@ir_dataclass
class SplitScan(Scan): ...

@ir_dataclass
class Sort(Loops):
    sort_ranges: list[Integer]
    size: list[Integer]
    reindex: Callable[[Sequence[Expr], Sequence[Expr]], Sequence[Expr]]
    reduction_hint: ReductionHint
    output_index: int
    dtypes: tuple[torch.dtype, ...]
    inner_fns: tuple[Callable[..., Any], ...]
    stable: bool
    descending: bool
    def get_unbacked_symbol_uses(self) -> OrderedSet[Symbol]: ...
    def __post_init__(self) -> None: ...
    def store_reduction(self, output_name: str | None, indexer: Callable[[Sequence[Expr]], Expr], vars: Sequence[Expr], reduction_vars: Sequence[Expr]) -> OpsValue: ...
    def get_reduction_type(self) -> str | None: ...
    def get_reduction_size(self) -> Sequence[Expr]: ...
    def get_size(self) -> Sequence[Expr]: ...
    def get_pointwise_size(self) -> Sequence[Expr]: ...
    def index_length(self) -> int: ...
    def inner_fn_args(self) -> Sequence[Sequence[Expr]]: ...
    def inner_fn_free_unbacked_symbols(self) -> set[Symbol]: ...
    @classmethod
    def create(cls, device: torch.device, dtypes: tuple[torch.dtype, ...], inner_fns: tuple[Callable[[list[Expr]], Any], ...], size: list[Integer], axis: int, stable: bool, descending: bool, reduction_hint: ReductionHint = ..., **kwargs: Any) -> Sequence[TensorBox | None]: ...

def is_storage_and_layout(x: IRNode) -> bool: ...
def is_contiguous_storage_and_layout(x: IRNode) -> bool: ...
def as_storage_and_layout(x: IRNode, freeze: bool = True, want_contiguous: bool = False, stride_order: Sequence[int | Integer] | None = None, allow_padding: bool = False, exact_strides: Sequence[int | Integer] | None = None) -> tuple[StorageBox, Layout]: ...

as_contiguous_storage_and_layout: Incomplete

def is_stride_order_storage_and_layout(x: IRNode, stride_order: Sequence[int | Integer]) -> bool: ...

@ir_dataclass
class BaseView(IRNode):
    data: IRNode
    def get_unbacked_symbol_uses(self) -> OrderedSet[Symbol]: ...
    def make_reindexer(self) -> Callable[[Sequence[Expr]], Sequence[Expr]]: ...
    def make_indexer(self) -> Callable[[Sequence[Expr]], Expr]: ...
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...
    @property
    def dtype(self) -> torch.dtype: ...
    def get_layout(self) -> Layout: ...
    def get_device(self) -> torch.device | None: ...
    def get_origin_node(self) -> torch.fx.Node | None: ...
    def get_name(self) -> str: ...
    def get_pointwise_size(self) -> Sequence[Expr]: ...
    def mark_reuse(self, users: int) -> None: ...
    def has_exceeded_max_reads(self) -> bool: ...
    def realize(self) -> str | None: ...
    def realize_hint(self): ...
    def get_storage_numel(self): ...
    def is_extern(self) -> bool: ...
    def is_module_buffer(self) -> bool: ...
    def get_read_names(self) -> OrderedSet[str]: ...
    def get_reads(self) -> OrderedSet[Dep]: ...
    def unwrap_view(self): ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...

@ir_dataclass
class ExpandView(BaseView):
    size: list[Expr]
    @classmethod
    def create(cls, x, new_size): ...
    def get_size(self) -> Sequence[Expr]: ...
    def make_reindexer(self): ...

@ir_dataclass
class PermuteView(BaseView):
    dims: list[Expr]
    @classmethod
    def create(cls, x, dims): ...
    def get_size(self) -> Sequence[Expr]: ...
    def make_reindexer(self): ...

@ir_dataclass
class SqueezeView(BaseView):
    @classmethod
    def create(cls, x, *, dim: Incomplete | None = None): ...
    @staticmethod
    def squeezer(size: Sequence[sympy.Expr]): ...
    def __init__(self, data) -> None: ...

@ir_dataclass
class GenericView(BaseView):
    size: list[Expr]
    reindex: Callable[..., Any]
    def make_reindexer(self): ...
    def reindex_str(self) -> str: ...
    @classmethod
    def create(cls, x, new_size, reindex): ...
    def get_size(self) -> Sequence[Expr]: ...

@ir_dataclass
class View(GenericView):
    @staticmethod
    def handle_negative_index(idx, size): ...
    @classmethod
    def create(cls, x, new_size): ...
    @staticmethod
    def resolve_negative_size(old_size, new_size): ...
    @classmethod
    def dynamic_reshape_indexer(cls, old_size, new_size): ...

@ir_dataclass
class ReinterpretView(BaseView):
    layout: Layout
    def __post_init__(self) -> None: ...
    def get_name(self): ...
    def get_device(self) -> torch.device | None: ...
    def get_origin_node(self) -> torch.fx.Node | None: ...
    @property
    def dtype(self): ...
    def get_size(self) -> Sequence[Expr]: ...
    def get_stride(self): ...
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...
    def make_indexer(self) -> Callable[[Sequence[Expr]], Expr]: ...
    def get_layout(self) -> Layout: ...
    def freeze_layout(self) -> None: ...
    def get_unbacked_symbol_uses(self) -> OrderedSet[sympy.Symbol]: ...
    def codegen_reference(self, writer: IndentedBuffer | None = None) -> str: ...
    def num_reads(self) -> int: ...

@ir_dataclass
class DtypeView(BaseView):
    target_dtype: torch.dtype
    @classmethod
    def create(cls, x, new_dtype): ...
    @property
    def dtype(self): ...
    def get_size(self) -> Sequence[Expr]: ...
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...

class SliceView(View):
    @classmethod
    def normalize_start_end(cls, x, dim, start, end): ...
    @classmethod
    def create(cls, x, dim, start, end, step: int = 1, clamp: bool = True): ...

@ir_dataclass
class BaseConstant(IRNode):
    dtype: torch.dtype
    device: torch.device
    def get_size(self) -> Sequence[Expr]: ...
    def get_device(self) -> torch.device | None: ...
    def get_origin_node(self) -> torch.fx.Node | None: ...
    def get_reads(self) -> OrderedSet[Dep]: ...

@ir_dataclass
class Constant(BaseConstant):
    value: Any
    dtype: torch.dtype
    device: torch.device
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...
    def realize(self) -> str | None: ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...

@ir_dataclass
class IndexingConstant(BaseConstant):
    index: Any
    dtype: torch.dtype
    device: torch.device
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...

def is_contiguous_strides_for_shape(stride: Sequence[_IntLike], shape: Sequence[_IntLike]) -> bool: ...
def get_align_for_dtype(dtype: torch.dtype) -> int: ...

class OutputSpec:
    def get_device(self) -> torch.device | None: ...
    def storage_size(self) -> int: ...

@ir_dataclass
class Layout(OutputSpec):
    device = ...
    dtype = ...
    size: list[Expr] = ...
    stride: list[Expr] = ...
    offset: Expr = ...
    def __init__(self, device: torch.device, dtype: torch.dtype, size: list[Expr], stride: list[Expr] | None = None, offset: Expr = ...) -> None: ...
    def get_device(self) -> torch.device: ...
    def is_contiguous(self) -> bool: ...
    @staticmethod
    def is_channels_last_contiguous(shape: Sequence[_IntLike], strides: Sequence[_IntLike]) -> bool: ...
    def is_transposed(self) -> bool: ...
    def is_stride_ordered(self, order) -> bool: ...
    def is_channels_last_stride_ordered(self): ...
    def pad_strides(self) -> None: ...
    def should_pad_strides(self): ...
    def as_fixed(self): ...
    def make_indexer(self) -> Callable[[Sequence[Expr]], Expr]: ...
    def __eq__(self, other) -> bool: ...
    def storage_size(self) -> sympy.Expr: ...

class FixedLayout(Layout):
    def make_indexer(self) -> Callable[[Sequence[Expr]], Expr]: ...

class FlexibleLayout(Layout):
    allow_indexing: bool
    @staticmethod
    def contiguous_strides(sizes): ...
    @staticmethod
    def fill_ordered(sizes, order): ...
    @staticmethod
    def stride_ordered(sizes, order): ...
    @staticmethod
    def stride_ordered_for_memory_format(sizes, memory_format): ...
    @staticmethod
    def same_ordered(sizes, stride): ...
    def as_stride_order(self, order, allow_padding: bool = False): ...
    def as_exact_strides(self, exact_strides, allow_padding: bool = False): ...
    def as_fill_order(self, order): ...
    def as_same_order(self, stride): ...
    def __init__(self, device, dtype, size, stride_order: Incomplete | None = None) -> None: ...

class NonOwningLayout(Layout):
    view: Incomplete
    def __init__(self, view: BaseView | TensorBox) -> None: ...
    def make_indexer(self) -> Callable[[Sequence[Expr]], Expr]: ...
    def maybe_guard_aligned(self): ...

class CommBufferType(Enum):
    SYMM_MEM = 'symm_mem'

class CommBufferLayout(FixedLayout):
    comm_buffer_type: CommBufferType
    group_name: str
    def __init__(self, layout: FlexibleLayout, comm_buffer_type: CommBufferType, group_name: str) -> None: ...

@ir_dataclass
class NoneLayout(OutputSpec):
    device: torch.device | None
    size: list[int] = dataclasses.field(default_factory=Incomplete)
    stride: list[int] = dataclasses.field(default_factory=Incomplete)
    def storage_size(self) -> int: ...
    def as_fixed(self): ...
    def get_device(self) -> torch.device | None: ...

class MutationLayoutSHOULDREMOVE(Layout):
    target: Incomplete
    def __init__(self, target: IRNode) -> None: ...
    @property
    def stride(self) -> list[Expr]: ...
    @stride.setter
    def stride(self, value: Never) -> None: ...
    def storage_size(self) -> sympy.Expr: ...
    def get_buffer(self) -> Buffer: ...
    def real_layout(self): ...
    @classmethod
    def realize_into(cls, src, dst, unsafe_alias: bool = False): ...
    def as_fixed(self): ...
    def make_indexer(self) -> Callable[[Sequence[Expr]], Expr]: ...

@ir_dataclass(frozen=False)
class Buffer(IRNode):
    name: str | None
    layout: OutputSpec
    def __post_init__(self) -> None: ...
    def make_indexer(self) -> Callable[[Sequence[Expr]], Expr]: ...
    def get_name(self) -> str: ...
    def get_device(self) -> torch.device | None: ...
    def get_defining_op(self) -> Operation | None: ...
    @property
    def dtype(self) -> torch.dtype: ...
    def get_size(self) -> Sequence[Expr]: ...
    def get_stride(self) -> list[Expr]: ...
    def get_offset(self) -> Expr: ...
    def get_layout(self) -> Layout: ...
    def get_output_spec(self) -> OutputSpec: ...
    def get_storage_numel(self): ...
    def freeze_layout(self) -> None: ...
    def freeze_layout_with_stride_order(self, order, allow_padding: bool = False) -> None: ...
    def freeze_layout_with_fill_order(self, order) -> None: ...
    def freeze_layout_with_same_order(self, stride) -> None: ...
    def freeze_layout_with_exact_strides(self, exact_strides, allow_padding: bool = False) -> None: ...
    def is_zero_elements(self): ...
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...
    def codegen_reference(self, writer: IndentedBuffer | None = None) -> str: ...
    def decide_layout(self) -> None: ...
    def get_inputs_that_alias_output(self) -> Sequence[str]: ...
    def get_mutation_names(self) -> Sequence[str]: ...
    def get_read_names(self) -> OrderedSet[str]: ...
    def get_unbacked_symbol_uses(self) -> OrderedSet[sympy.Symbol]: ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    def realize(self) -> str | None: ...
    def should_allocate(self) -> bool: ...

@ir_dataclass(frozen=False)
class OperationBuffer(Buffer, Operation):
    def get_outputs(self) -> list[Buffer]: ...
    def get_defining_op(self) -> Operation: ...
    get_operation_name = ...
    def __post_init__(self) -> None: ...

class InputBuffer(Buffer):
    def num_reads(self) -> int: ...

class DonatedBuffer(InputBuffer): ...

class ConstantBuffer(InputBuffer):
    override_device: torch.device | None
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...

@ir_dataclass
class NoneAsConstantBuffer(IRNode):
    def get_unbacked_symbol_uses(self) -> OrderedSet[sympy.Symbol]: ...
    def codegen_reference(self, writer: IndentedBuffer | None = None) -> str: ...
    def get_output_spec(self) -> OutputSpec: ...
    def has_tensor_output(self) -> bool: ...

@ir_dataclass
class ShapeAsConstantBuffer(IRNode):
    expr: Expr
    def get_unbacked_symbol_uses(self) -> OrderedSet[sympy.Symbol]: ...
    def codegen_reference(self, writer: IndentedBuffer | None = None) -> str: ...
    def has_tensor_output(self) -> bool: ...

@ir_dataclass(frozen=False)
class ComputedBuffer(OperationBuffer):
    data: Loops
    def get_computed_buffer_name(self) -> str | None: ...
    def num_reads(self) -> int: ...
    def get_reads(self) -> OrderedSet[Dep]: ...
    def get_read_names(self) -> OrderedSet[str]: ...
    def get_read_writes(self) -> dependencies.ReadWrites: ...
    def get_unbacked_symbol_uses(self) -> OrderedSet[sympy.Symbol]: ...
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...
    def get_store_function(self) -> Callable[..., OpsValue]: ...
    def get_fill_order(self) -> list[int] | None: ...
    def decide_layout(self) -> None: ...
    def get_default_sizes_body(self) -> tuple[tuple[list[sympy.Expr], list[sympy.Expr]], LoopBody, tuple[list[sympy.Expr], list[sympy.Expr]]]: ...
    def simplify_and_reorder(self, extra_indexing_constraints: tuple[dict[Any, Any], list[Any]] | None = None, recompute_sizes_body_func: Callable[..., Any] | None = None) -> tuple[tuple[list[sympy.Expr], list[sympy.Expr]], LoopBody]: ...
    def get_reduction_size(self) -> Sequence[sympy.Expr]: ...
    def get_reduction_type(self) -> str | None: ...
    def is_no_op(self) -> bool: ...
    def should_allocate(self) -> bool: ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...

class TemplateBuffer(OperationBuffer):
    inputs: Incomplete
    make_kernel_render: Incomplete
    name: Incomplete
    def __init__(self, layout: Layout, inputs: list[IRNode], make_kernel_render: Callable[..., Any]) -> None: ...
    def get_read_writes(self) -> dependencies.ReadWrites: ...
    def extract_read_writes(self, normalize): ...
    def get_reduction_size(self) -> Sequence[sympy.Expr]: ...
    def get_reduction_type(self) -> str | None: ...
    def should_allocate(self) -> bool: ...
    def simplify_and_reorder(self, extra_indexing_constraints: tuple[dict[Any, Any], list[Any]] | None = None, recompute_sizes_body_func: Callable[..., Any] | None = None): ...

class TritonTemplateBuffer(TemplateBuffer):
    mutated_inputs: Incomplete
    outputs: list[Buffer]
    def __init__(self, layout, inputs, make_kernel_render, mutated_inputs: Iterable[IRNode] | None = None) -> None: ...
    def get_outputs(self) -> list[Buffer]: ...
PrimitiveInfoType = int | float | bool | str | list[int | str | float | bool]

class ChoiceCaller:
    name: Incomplete
    layout: Incomplete
    input_nodes: Incomplete
    description: Incomplete
    def __init__(self, name: str, input_nodes: list[Buffer], layout: Layout, description: str) -> None: ...
    def benchmark(self, *args, out) -> float: ...
    def call_name(self) -> str: ...
    def to_callable(self) -> None: ...
    def hash_key(self) -> str: ...
    def output_node(self) -> TensorBox: ...
    def info_dict(self) -> dict[str, PrimitiveInfoType | list[PrimitiveInfoType]]: ...
    def autoheuristic_id(self) -> str: ...

class TritonTemplateCallerBase(ChoiceCaller):
    def get_make_kernel_render(self) -> Any: ...

class MultiTemplateBuffer(TritonTemplateBuffer):
    original_inputs: Incomplete
    def __init__(self, layout: Layout, inputs: list[IRNode], choice_timings: Callable[[], dict[ChoiceCaller, float]], unfiltered_choices: list[ChoiceCaller]) -> None: ...
    @property
    def output_plannable(self) -> bool: ...
    @property
    def choice_timings(self) -> dict[ChoiceCaller, float]: ...
    make_kernel_render: Incomplete
    def swap_as_triton_caller(self, caller: TritonTemplateCallerBase): ...
    def finalize_as_triton_caller(self, caller: TritonTemplateCallerBase) -> None: ...
    def get_min_choice(self) -> tuple[ChoiceCaller, float]: ...

class CUDATemplateBuffer(TemplateBuffer):
    workspace_size: Incomplete
    template: Incomplete
    def __init__(self, layout, inputs, make_kernel_render, workspace_size: int, template: CUDATemplate) -> None: ...
    def get_workspace_size(self): ...

class CppTemplateBuffer(TemplateBuffer):
    template: Incomplete
    choice: Incomplete
    def __init__(self, layout, inputs, make_kernel_render, template, choice) -> None: ...

@ir_dataclass(frozen=False)
class InputsKernel(OperationBuffer):
    inputs: list[Buffer]
    def get_read_writes(self) -> dependencies.ReadWrites: ...
    def get_reads(self) -> OrderedSet[Dep]: ...
    @classmethod
    def unwrap_storage_for_input(cls, x: IRNode) -> IRNode: ...
    @staticmethod
    def unwrap_storage(inputs): ...
    def is_extern(self) -> bool: ...
    def num_reads(self) -> int: ...

class NopKernel(InputsKernel):
    def is_no_op(self) -> bool: ...
    def get_reads(self) -> OrderedSet[Dep]: ...

class ConcatKernel(NopKernel):
    @classmethod
    def create(cls, inputs, dim): ...
    @classmethod
    def can_realize_into_without_copy(cls, src, dst: Incomplete | None = None): ...
    @classmethod
    def realize_into(cls, src, dst): ...
    def should_allocate(self) -> bool: ...

@ir_dataclass(frozen=False)
class ExternKernel(InputsKernel):
    constant_args: tuple[Any, ...] = ...
    kwargs: dict[str, Any] = dataclasses.field(default_factory=dict)
    output_view: ReinterpretView | None = ...
    python_kernel_name: str | None = ...
    cpp_kernel_name: str | None = ...
    ordered_kwargs_for_cpp_kernel: Iterable[str] = dataclasses.field(default_factory=list)
    op_overload: torch._ops.OpOverload | torch._ops.HigherOrderOperator | None = ...
    arg_properties: list[dict[str, Any]] | None = ...
    kwarg_properties: dict[str, dict[str, Any]] | None = ...
    unbacked_bindings: dict[sympy.Symbol, pytree.KeyPath] = dataclasses.field(default_factory=dict)
    mutation_outputs: list[MutationOutput] = dataclasses.field(default_factory=list)
    fx_node = ...
    def __init__(self, name, layout, inputs, constant_args=(), kwargs: Incomplete | None = None, output_view: Incomplete | None = None, python_kernel_name: Incomplete | None = None, cpp_kernel_name: Incomplete | None = None, ordered_kwargs_for_cpp_kernel=(), op_overload: Incomplete | None = None) -> None: ...
    def get_outputs(self) -> list[Buffer]: ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    allarg_properties = ...
    schema_kwargs = ...
    def collect_arg_kwarg_properties(self) -> None: ...
    def decide_layout(self) -> None: ...
    def codegen_comment(self, wrapper) -> None: ...
    def codegen(self, wrapper) -> None: ...
    def set_cpp_kernel_name(self, cpp_kernel_name: str | None = None) -> None: ...
    def set_python_kernel_name(self, python_kernel_name: str | None) -> None: ...
    def get_kernel_name(self): ...
    @staticmethod
    def copy_input(x): ...
    @classmethod
    def process_kernel(cls, kernel, *args, **kwargs) -> tuple[Any, list[Any], list[Any], Callable[[Any, Any], Any], dict[sympy.Symbol, pytree.KeyPath] | None]: ...
    @classmethod
    def convert_to_reinterpret_view(cls, x): ...
    @classmethod
    def realize_input(cls, x): ...
    @classmethod
    def require_stride1(cls, x): ...
    @classmethod
    def require_strides(cls, x, order: Sequence[int] | None = None, exact_strides: Sequence[_IntLike] | None = None, allow_padding: bool = False): ...
    @classmethod
    def require_exact_strides(cls, x, exact_strides, allow_padding: bool = False): ...
    @classmethod
    def require_stride_order(cls, x, order, allow_padding: bool = False): ...
    @classmethod
    def require_channels_last(cls, x): ...
    @classmethod
    def require_channels_last_3d(cls, x): ...
    @classmethod
    def require_contiguous(cls, x): ...
    def apply_constraint(self) -> None: ...
    def fill_non_provided_args(self, args, kwargs): ...
    def codegen_const_args(self, names: list[str] | None = None): ...
    def codegen_args(self): ...
    def get_kwargs_value(self, arg_name): ...
    def codegen_kwargs(self, skip_out: bool = False): ...
    def codegen_size_asserts(self, wrapper) -> None: ...
    def get_group_stride(self): ...
    def canonicalize(self): ...
    def get_unbacked_symbol_uses(self) -> OrderedSet[sympy.Symbol]: ...

@ir_dataclass(frozen=False)
class ExternKernelOut(ExternKernel):
    def codegen(self, wrapper) -> None: ...
    name = ...
    def __init__(self, layout, inputs, constant_args=(), kwargs: Incomplete | None = None, output_view: Incomplete | None = None, python_kernel_name: Incomplete | None = None, cpp_kernel_name: Incomplete | None = None, ordered_kwargs_for_cpp_kernel=(), op_overload: Incomplete | None = None) -> None: ...
    def should_allocate(self) -> bool: ...

class RandomSeeds(ExternKernelOut):
    def __init__(self, count: int, device: torch.device) -> None: ...

class ExternKernelAlloc(ExternKernel):
    def codegen(self, wrapper) -> None: ...
    outputs: Sequence[Any]
    name: Incomplete
    def __init__(self, layout, inputs, constant_args=(), kwargs: Incomplete | None = None, python_kernel_name: Incomplete | None = None, cpp_kernel_name: Incomplete | None = None, ordered_kwargs_for_cpp_kernel=(), op_overload: Incomplete | None = None) -> None: ...
    def should_allocate(self) -> bool: ...
    def apply_constraint(self) -> None: ...

class MutationOutput(Buffer):
    mutation_names: Incomplete
    mutating_node: Operation
    name: Incomplete
    def __init__(self, layout, mutated_node, mutating_node: Operation) -> None: ...
    def get_defining_op(self) -> Operation: ...
    def get_mutation_names(self): ...
    def should_allocate(self) -> bool: ...

class TMADescriptor(ExternKernel):
    @classmethod
    def create(cls, tensor: IRNode, dims: list[int | torch.SymInt], block_dims: list[int | torch.SymInt], element_size: int | None = None): ...
    tensor: Incomplete
    dims: Incomplete
    block_dims: Incomplete
    element_size: Incomplete
    rank: Incomplete
    name: Incomplete
    def __init__(self, tensor: IRNode, dims: list[int | torch.SymInt], block_dims: list[int | torch.SymInt], element_size: int | None = None) -> None: ...
    def codegen(self, wrapper) -> None: ...

class UserDefinedTritonKernel(ExternKernel):
    def get_kernel_and_metadata(self): ...
    def codegen(self, wrapper) -> None: ...
    def get_unbacked_symbol_uses(self) -> OrderedSet[sympy.Symbol]: ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    device: Incomplete
    kernel_idx: Incomplete
    grid: Incomplete
    ordered_kwargs_for_cpp_kernel: Incomplete
    mutable_args: Incomplete
    mutation_outputs: Incomplete
    def __init__(self, *, kernel_idx, grid, tma_descriptor_metadata, kernel_args) -> None: ...
    def get_outputs(self) -> list[Buffer]: ...
    def get_device(self) -> torch.device | None: ...

class InplaceBernoulliFallback(ExternKernel):
    def codegen(self, wrapper) -> None: ...
    def should_allocate(self) -> bool: ...
    def get_mutation_names(self): ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    name: Incomplete
    def __init__(self, op_overload, x, *constant_args) -> None: ...

class InplaceCopyFallback(ExternKernel):
    def codegen(self, wrapper) -> None: ...
    def should_allocate(self) -> bool: ...
    def get_mutation_names(self): ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    name: Incomplete
    def __init__(self, layout, inputs, constant_args) -> None: ...
    @classmethod
    def create(cls, dst, src, non_blocking: bool = False): ...

class MutatingFirstArgExternKernel(ExternKernel):
    def codegen(self, wrapper) -> None: ...
    def should_allocate(self) -> bool: ...
    def get_mutation_names(self): ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    def has_side_effects(self) -> bool: ...

class ResizeStorageBytes(MutatingFirstArgExternKernel):
    name: Incomplete
    python_kernel_name: str
    cpp_kernel_name: str
    def __init__(self, variable, new_size) -> None: ...

class SetSourceTensorKernel(ExternKernelAlloc):
    mutation_outputs: Incomplete
    def __init__(self, self_tensor, storage_tensor) -> None: ...
    def get_inputs_that_alias_output(self) -> Sequence[str]: ...

class ScatterFallback(ExternKernel):
    def codegen(self, wrapper) -> None: ...
    def should_allocate(self) -> bool: ...
    def get_mutation_names(self): ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    src_is_tensor: Incomplete
    name: Incomplete
    def __init__(self, op_overload, x, dim: int, index, src, *, reduce: str | None = None, include_self: bool = True) -> None: ...

class IndexPutFallback(ExternKernel):
    def codegen(self, wrapper) -> None: ...
    def should_allocate(self) -> bool: ...
    def get_mutation_names(self): ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    indices: Incomplete
    name: Incomplete
    def __init__(self, op_overload, x, indices, values, accumulate) -> None: ...

class DeviceCopy(ExternKernelOut):
    @classmethod
    def create(cls, x, device, non_blocking): ...
    def codegen(self, wrapper) -> None: ...

class DynamicScalar(ExternKernel):
    def get_reads(self) -> OrderedSet[Dep]: ...
    def should_allocate(self) -> bool: ...
    sym: Incomplete
    keypath: Incomplete
    def __init__(self, sym, keypath, data) -> None: ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    def codegen(self, wrapper) -> None: ...

class AssertScalar(ExternKernel):
    def get_reads(self) -> OrderedSet[Dep]: ...
    def should_allocate(self) -> bool: ...
    scalar: Incomplete
    msg: Incomplete
    def __init__(self, scalar, msg) -> None: ...
    def has_side_effects(self) -> bool: ...
    def get_unbacked_symbol_uses(self): ...
    def codegen(self, wrapper) -> None: ...

@ir_dataclass(frozen=False)
class ExternKernelNode:
    name: str
    node: export_schema.Node

class FallbackKernel(ExternKernelAlloc):
    use_runtime_dispatch: bool
    unbacked_bindings: Incomplete
    op_overload: Incomplete
    unflatten_args: Incomplete
    kwargs: Incomplete
    alias_names: list[str]
    mutation_names: list[str]
    def __init__(self, layout, kernel, tensor_args, nontensor_args, unflatten_args, kwargs: Incomplete | None = None, *, unbacked_bindings: Incomplete | None = None) -> None: ...
    def codegen_unbacked_symbol_defs(self, wrapper) -> None: ...
    def get_unbacked_symbol_defs(self) -> OrderedSet[sympy.Symbol]: ...
    def codegen_args(self): ...
    @staticmethod
    def find_device(tensor_args, example_output): ...
    def has_side_effects(self): ...
    def get_inputs_that_alias_output(self): ...
    def get_mutation_names(self): ...
    def export_extern_kernel_node(self): ...
    def codegen(self, wrapper) -> None: ...
    @staticmethod
    def tensor_to_layout(output: torch.Tensor): ...
    @classmethod
    def create(cls, kernel, *args, **kwargs): ...
    def apply_constraint(self): ...

@ir_dataclass(frozen=False)
class ComplexView(FallbackKernel):
    def should_allocate(self) -> bool: ...
    def get_inputs_that_alias_output(self) -> Sequence[str]: ...
    def __init__(self, layout, kernel, tensor_args, nontensor_args, unflatten_args, *, unbacked_bindings: Incomplete | None = None) -> None: ...

@ir_dataclass
class MultiOutputLayout(OutputSpec):
    device: torch.device
    def get_device(self) -> torch.device | None: ...

class MultiOutput(ExternKernel):
    def codegen_list_tuple_access(self, basename, indices): ...
    def codegen(self, wrapper) -> None: ...
    name: Incomplete
    indices: Incomplete
    def __init__(self, layout: OutputSpec, input, indices: list[tuple[Any, ...]]) -> None: ...
    def get_unbacked_symbol_uses(self) -> OrderedSet[sympy.Symbol]: ...
    def should_allocate(self) -> bool: ...
    def get_inputs_that_alias_output(self) -> Sequence[str]: ...

@dataclasses.dataclass
class MutableBox(IRNode):
    data: IRNode
    def has_exceeded_max_reads(self) -> bool: ...
    def get_device(self) -> torch.device | None: ...
    def make_loader(self) -> Callable[[Sequence[Expr]], OpsValue]: ...
    def make_indexer(self) -> Callable[[Sequence[Expr]], Expr]: ...
    def get_stride(self) -> Sequence[_IntLike]: ...
    def get_name(self) -> str: ...
    def has_large_inner_fn(self, threshold: int | None = None) -> bool: ...
    def mark_reuse(self, users: int) -> None: ...
    def realize_hint(self) -> None: ...
    def unwrap_view(self) -> IRNode: ...
    def freeze_layout(self) -> None: ...
    def freeze_layout_with_stride_order(self, order: list[int], allow_padding: bool = False) -> None: ...
    def freeze_layout_with_fill_order(self, order: list[int]) -> None: ...
    def freeze_layout_with_same_order(self, stride: list[_IntLike]) -> None: ...
    def freeze_layout_with_exact_strides(self, exact_strides: list[_IntLike], allow_padding: bool = False) -> None: ...
    def get_read_writes(self) -> dependencies.ReadWrites: ...
    def get_reads(self) -> OrderedSet[Dep]: ...
    def num_reads(self) -> int: ...
    def get_storage_numel(self) -> _IntLike: ...
    def get_reduction_type(self) -> str | None: ...
    def get_reduction_size(self) -> Sequence[sympy.Expr]: ...
    def is_extern(self) -> bool: ...
    def is_no_op(self) -> bool: ...
    def constant_to_device(self, device: torch.device) -> IRNode: ...
    def get_mutation_names(self) -> Sequence[str]: ...
    def get_operation_name(self) -> str: ...
    def get_inputs_that_alias_output(self) -> Sequence[str]: ...
    def realize(self) -> str | None: ...
    def get_unbacked_symbol_uses(self) -> OrderedSet[sympy.Symbol]: ...
    def get_read_names(self) -> OrderedSet[str]: ...
    def get_defining_op(self) -> Operation | None: ...
    def codegen_reference(self, writer: IndentedBuffer | None = None) -> str: ...
    @property
    def layout(self) -> OutputSpec: ...
    def get_layout(self) -> Layout: ...
    def get_output_spec(self) -> OutputSpec: ...
    def get_size(self) -> Sequence[Expr]: ...
    @property
    def dtype(self): ...

class TensorBox(MutableBox):
    @staticmethod
    def create(data): ...

class StorageBox(MutableBox):
    def is_input_buffer(self): ...
    def is_module_buffer(self): ...
    data: Incomplete
    def realize(self) -> str | None: ...
    def realize_hint(self) -> None: ...
    def has_exceeded_max_reads(self) -> bool: ...
    def should_realize_on_reuse(self, users): ...
    def mark_reuse(self, users: int) -> None: ...
    def num_reads(self): ...

@ir_dataclass(frozen=False)
class Subgraph(IRNode):
    name: str
    graph_module: torch.fx.GraphModule
    graph: GraphLowering | None = ...

@ir_dataclass(frozen=False)
class InvokeSubgraph(ExternKernel):
    subgraph: Subgraph | None = ...
    operands: list[TensorBox] | None = ...
    outputs: list[MultiOutput] | None = ...
    name = ...
    def __init__(self, subgraph: Subgraph, operands: list[TensorBox], layout: MultiOutputLayout) -> None: ...
    @classmethod
    def create(cls, subgraph: Subgraph, operands): ...
    def codegen(self, wrapper) -> None: ...

@ir_dataclass(frozen=False)
class Conditional(ExternKernel):
    predicate: IRNode | None = ...
    operands: list[TensorBox] | None = ...
    true_subgraph: Subgraph | None = ...
    false_subgraph: Subgraph | None = ...
    outputs: list[MultiOutput] | None = ...
    name = ...
    def __init__(self, predicate: IRNode, operands: list[TensorBox], true_subgraph: Subgraph, false_subgraph: Subgraph, layout: MultiOutputLayout) -> None: ...
    @classmethod
    def create(cls, predicate: TensorBox, true_fn: Subgraph, false_fn: Subgraph, operands: list[TensorBox]): ...
    def codegen(self, wrapper) -> None: ...

@ir_dataclass(frozen=False)
class WhileLoop(ExternKernel):
    carried_inputs: list[TensorBox] | None = ...
    additional_inputs: list[TensorBox] | None = ...
    cond_subgraph: Subgraph | None = ...
    body_subgraph: Subgraph | None = ...
    outputs: list[MultiOutput] | None = ...
    name = ...
    def __init__(self, carried_inputs: list[TensorBox], additional_inputs: list[TensorBox], cond_subgraph: Subgraph, body_subgraph: Subgraph, layout: MultiOutputLayout) -> None: ...
    @classmethod
    def create(cls, cond_fn: Subgraph, body_fn: Subgraph, carried_inputs: list[TensorBox], additional_inputs: list[TensorBox]): ...
    def codegen(self, wrapper) -> None: ...

class EffectfulKernel(FallbackKernel):
    effect_type: Incomplete
    prev_effect_buffer: Incomplete
    def __init__(self, layout, kernel, tensor_args, nontensor_args, unflatten_args, kwargs: Incomplete | None = None, *, unbacked_bindings: Incomplete | None = None) -> None: ...
    def get_read_writes(self) -> dependencies.ReadWrites: ...
    def has_side_effects(self) -> bool: ...

@ir_dataclass
class TorchBindObject(IRNode):
    name: str
    value: torch._C.ScriptObject
    def get_name(self): ...
    def codegen_reference(self, writer: IndentedBuffer | None = None) -> str: ...

class _CollectiveKernel(FallbackKernel):
    def should_allocate(self) -> bool: ...
    def has_side_effects(self) -> bool: ...
    cpp_kernel_name: Incomplete
    ordered_kwargs_for_cpp_kernel: Incomplete
    def set_cpp_kernel_name(self, cpp_kernel_name: str | None = None) -> None: ...
    @classmethod
    def create_inplace(cls, kernel, inputs: TensorBox | list[TensorBox], *args, **kwargs) -> None: ...
    @classmethod
    def create_out_of_place(cls, kernel, inputs: TensorBox | list[TensorBox], *args, **kwargs): ...

class _WaitKernel(_CollectiveKernel):
    def get_volatile_reads(self): ...
    @classmethod
    def create_wait(cls, kernel, inp: TensorBox) -> None: ...
    def get_read_writes(self) -> dependencies.ReadWrites: ...

def maybe_free_unbacked_symbols(s: object) -> OrderedSet[Symbol]: ...
