import collections
import sympy
import torch.fx
from . import config as config, dependencies as dependencies
from .codegen.common import index_prevent_reordering as index_prevent_reordering
from .utils import cache_on_self as cache_on_self, sympy_index_symbol_with_prefix as sympy_index_symbol_with_prefix, sympy_subs as sympy_subs
from .virtualized import V as V, ops as ops
from _typeshed import Incomplete
from enum import Enum
from torch._dynamo.utils import identity as identity
from torch.fx.proxy import Scope as Scope, TracerBase as TracerBase
from torch.utils._sympy.symbol import SymT as SymT
from typing import Any, Callable, NamedTuple, TypeVar

T = TypeVar('T')

class InterpreterShim(torch.fx.Interpreter):
    module: Incomplete
    graph: Incomplete
    submodules: Incomplete
    extra_traceback: bool
    fetch_attr: Incomplete
    current_node: Incomplete
    def __init__(self, graph, submodules) -> None: ...
    def run_node(self, n: torch.fx.Node) -> Any: ...
    def run(self, *args, **kwargs): ...

class LightTracer(TracerBase):
    graph: Incomplete
    scope: Incomplete
    module_stack: Incomplete
    node_name_to_scope: Incomplete
    def __init__(self) -> None: ...

class MemoryEntry(NamedTuple):
    index_name: str
    buffer_name: str | None
    mode: str | None

class MemoryUsageType(Enum):
    LOAD = ...
    LOAD_SEED = ...
    STORE = ...
    STORE_REDUCTION = ...
    INDEX_EXPR = ...
    CHECK_BOUNDS = ...
    BUCKETIZE = ...

class LoopBody:
    indexing_exprs: dict[str, sympy.Expr]
    indexing_exprs_name: dict[sympy.Expr, str]
    submodules: dict[str, Any]
    subblocks: dict[str, LoopBodyBlock]
    indirect_vars: list[sympy.Symbol]
    indirect_var_ranges: dict[sympy.Symbol, sympy.Expr]
    root_block: LoopBodyBlock
    memory_usage: dict[MemoryUsageType, list[MemoryEntry]]
    op_counts: collections.Counter[str]
    sizes: Incomplete
    iter_vars: Incomplete
    reduce_vars: Incomplete
    var_ranges: Incomplete
    indexing: Incomplete
    def __init__(self, fn, args, var_ranges, iter_vars, reduce_vars) -> None: ...
    def has_op(self, name: str): ...
    def merge_loops(self) -> LoopBody: ...
    def reorder_iter_loops(self, new_order) -> LoopBody: ...
    @property
    def vars(self): ...
    def get_nodes(self): ...
    def bounds(self): ...
    def get_read_expr(self, buffer_name): ...
    def get_write_expr(self, buffer_name): ...
    def get_read_exprs(self): ...
    def get_write_exprs(self): ...
    def debug_str(self): ...
    def is_memory_copy(self) -> bool: ...
    def add_index_expr(self, expr: sympy.Expr, mtype: MemoryUsageType, buffer_name: str | None = None, mode: str | None = None): ...
    def add_submodule(self, block, prefix): ...
    def add_indirect(self, size): ...
    def replace_indirect(self, old, new) -> None: ...
    def get_index(self, name): ...
    def indexing_from_args(self, indices): ...
    def __call__(self, *indices): ...
    def bind_set_indirect_shim(self, var, size, check, wrap_neg): ...
    def bind_scan_shim(self, combine_fn): ...
    def bind_masked_shim(self, name): ...

class LoopBodyBlock:
    body: Incomplete
    name: str
    graph: Incomplete
    def __init__(self, body: LoopBody, fn: Callable[..., Any], args: list[Any]) -> None: ...
    def __call__(self): ...
    def debug_str(self, name: str = 'block'): ...
    def contains_only_ops(self, allowed_ops) -> bool: ...
    def clone(self, body: LoopBody): ...

class CountOps:
    def __init__(self, inner: Any, counts: collections.Counter[str]) -> None: ...
    def __getattr__(self, name): ...
