import sympy
import torch
from ..utils._ordered_set import OrderedSet as OrderedSet
from .utils import IndentedBuffer as IndentedBuffer, reduction_num_outputs as reduction_num_outputs, sympy_index_symbol as sympy_index_symbol, sympy_str as sympy_str
from _typeshed import Incomplete
from typing import Any, Callable, Generic, NamedTuple, TypeVar
from typing_extensions import Protocol

T = TypeVar('T')
StoreMode: Incomplete
ReductionType: Incomplete

class OpsHandler(Protocol[T]):
    def constant(self, value: bool | float | int, dtype: torch.dtype) -> T: ...
    def load_seed(self, name: str, offset: T): ...
    def rand(self, seed: T, offset: T) -> T: ...
    def randn(self, seed: T, offset: T) -> T: ...
    def randint64(self, seed: T, offset: T, low: T, high: T) -> T: ...
    def masked(self, mask: T, body: Callable[[], T], other: T) -> T: ...
    def where(self, condition: T, input: T, other: T) -> T: ...
    def index_expr(self, expr: sympy.Expr, dtype: torch.dtype) -> T: ...
    def to_dtype(self, x: T, dtype: torch.dtype, src_dtype: torch.dtype | None = None, use_compute_types: bool = True) -> T: ...
    def trunc_to_int(self, x: T, dtype: torch.dtype) -> T: ...
    def ceil_to_int(self, x: T, dtype: torch.dtype) -> T: ...
    def floor_to_int(self, x: T, dtype: torch.dtype) -> T: ...
    def round_to_int(self, x: T, dtype: torch.dtype) -> T: ...
    def to_dtype_bitcast(self, x: T, dtype: torch.dtype, src_dtype: torch.dtype) -> T: ...
    def identity(self, x: T) -> T: ...
    def indirect_indexing(self, x: T, size: sympy.Expr, check: bool = True, wrap_neg: bool = True) -> sympy.Expr: ...
    def load(self, name: str, index: sympy.Expr) -> T: ...
    def store(self, name: str, index: sympy.Expr, value: T, mode: StoreMode = None) -> None: ...
    def reduction(self, dtype: torch.dtype, src_dtype: torch.dtype, reduction_type: ReductionType, value: T) -> T | tuple[T, ...]: ...
    def store_reduction(self, name: str, index: sympy.Expr, value: T) -> T: ...
    def scan(self, dtypes: tuple[torch.dtype, ...], combine_fn: Callable[[tuple[T, ...], tuple[T, ...]], tuple[T, ...]], values: tuple[T, ...]) -> tuple[T, ...]: ...
    def sort(self, dtypes: tuple[torch.dtype, ...], values: tuple[T, ...], stable: bool, descending: bool) -> tuple[T, ...]: ...
    def bucketize(self, values: T, boundaries: tuple[str, sympy.Expr, sympy.Expr, sympy.Expr], boundary_indices: T, indexing_dtype: torch.dtype, right: bool, sorter: tuple[str, sympy.Expr] | None = None, sorter_indices: T | None = None) -> T: ...
    def abs(self, x0: T) -> T: ...
    def exp(self, x0: T) -> T: ...
    def exp2(self, x0: T) -> T: ...
    def expm1(self, x0: T) -> T: ...
    def sqrt(self, x0: T) -> T: ...
    def relu(self, x0: T) -> T: ...
    def minimum(self, x0: T, x1: T) -> T: ...
    def maximum(self, x0: T, x1: T) -> T: ...
    def cos(self, x0: T) -> T: ...
    def sin(self, x0: T) -> T: ...
    def lgamma(self, x0: T) -> T: ...
    def erf(self, x0: T) -> T: ...
    def cosh(self, x0: T) -> T: ...
    def sinh(self, x0: T) -> T: ...
    def acos(self, x0: T) -> T: ...
    def acosh(self, x0: T) -> T: ...
    def asin(self, x0: T) -> T: ...
    def asinh(self, x0: T) -> T: ...
    def atan2(self, x0: T, x1: T) -> T: ...
    def atan(self, x0: T) -> T: ...
    def atanh(self, x0: T) -> T: ...
    def copysign(self, x0: T, x1: T) -> T: ...
    def erfc(self, x0: T) -> T: ...
    def erfinv(self, x0: T) -> T: ...
    def frexp(self, x0: T): ...
    def hypot(self, x0: T, x1: T) -> T: ...
    def log10(self, x0: T) -> T: ...
    def log2(self, x0: T) -> T: ...
    def nextafter(self, x0: T, x1: T) -> T: ...
    def logical_and(self, x0: T, x1: T) -> T: ...
    def logical_not(self, x0: T) -> T: ...
    def logical_or(self, x0: T, x1: T) -> T: ...
    def logical_xor(self, x0: T, x1: T) -> T: ...
    def bitwise_and(self, x0: T, x1: T) -> T: ...
    def bitwise_not(self, x0: T) -> T: ...
    def bitwise_or(self, x0: T, x1: T) -> T: ...
    def bitwise_xor(self, x0: T, x1: T) -> T: ...
    def bitwise_left_shift(self, x0: T, x1: T) -> T: ...
    def bitwise_right_shift(self, x0: T, x1: T) -> T: ...
    def rsqrt(self, x0: T) -> T: ...
    def log1p(self, x0: T) -> T: ...
    def tan(self, x0: T) -> T: ...
    def tanh(self, x0: T) -> T: ...
    def sigmoid(self, x0: T) -> T: ...
    def signbit(self, x0: T) -> T: ...
    def fmod(self, x0: T, x1: T) -> T: ...
    def log(self, x0: T) -> T: ...
    def isinf(self, x0: T) -> T: ...
    def isnan(self, x0: T) -> T: ...
    def round(self, x0: T) -> T: ...
    def floor(self, x0: T) -> T: ...
    def sign(self, x0: T) -> T: ...
    def trunc(self, x0: T) -> T: ...
    def ceil(self, x0: T) -> T: ...
    def neg(self, x0: T) -> T: ...
    def reciprocal(self, x0: T) -> T: ...
    def eq(self, x0: T, x1: T) -> T: ...
    def ne(self, x0: T, x1: T) -> T: ...
    def lt(self, x0: T, x1: T) -> T: ...
    def gt(self, x0: T, x1: T) -> T: ...
    def le(self, x0: T, x1: T) -> T: ...
    def ge(self, x0: T, x1: T) -> T: ...
    def add(self, x0: T, x1: T) -> T: ...
    def sub(self, x0: T, x1: T) -> T: ...
    def mul(self, x0: T, x1: T) -> T: ...
    def pow(self, x0: T, x1: T) -> T: ...
    def and_(self, x0: T, x1: T) -> T: ...
    def or_(self, x0: T, x1: T) -> T: ...
    def xor(self, x0: T, x1: T) -> T: ...
    def lshift(self, x0: T, x1: T) -> T: ...
    def rshift(self, x0: T, x1: T) -> T: ...
    def getitem(self, x0: T, x1: T) -> T: ...
    def matmul(self, x0: T, x1: T) -> T: ...
    def invert(self, x0: T) -> T: ...
    def airy_ai(self, x: T) -> T: ...
    def bessel_j0(self, x: T) -> T: ...
    def bessel_j1(self, x: T) -> T: ...
    def bessel_y0(self, x: T) -> T: ...
    def bessel_y1(self, x: T) -> T: ...
    def digamma(self, x: T) -> T: ...
    def erfcx(self, x: T) -> T: ...
    def fma(self, x: T, y: T, z: T) -> T: ...
    def igamma(self, x: T, y: T) -> T: ...
    def igammac(self, x: T, y: T) -> T: ...
    def gammainc(self, x: T, y: T) -> T: ...
    def gammaincc(self, x: T, y: T) -> T: ...
    def i0(self, x: T) -> T: ...
    def i0e(self, x: T) -> T: ...
    def i1(self, x: T) -> T: ...
    def i1e(self, x: T) -> T: ...
    def log_ndtr(self, x: T) -> T: ...
    def modified_bessel_i0(self, x: T) -> T: ...
    def modified_bessel_i1(self, x: T) -> T: ...
    def modified_bessel_k0(self, x: T) -> T: ...
    def modified_bessel_k1(self, x: T) -> T: ...
    def ndtr(self, x: T) -> T: ...
    def ndtri(self, x: T) -> T: ...
    def polygamma(self, x: T, y: T) -> T: ...
    def scaled_modified_bessel_k0(self, x: T) -> T: ...
    def scaled_modified_bessel_k1(self, x: T) -> T: ...
    def spherical_bessel_j0(self, x: T) -> T: ...
    def zeta(self, x: T, y: T) -> T: ...
    def chebyshev_polynomial_t(self, x: T, y: T) -> T: ...
    def chebyshev_polynomial_u(self, x: T, y: T) -> T: ...
    def chebyshev_polynomial_v(self, x: T, y: T) -> T: ...
    def chebyshev_polynomial_w(self, x: T, y: T) -> T: ...
    def legendre_polynomial_p(self, x: T, y: T) -> T: ...
    def shifted_chebyshev_polynomial_t(self, x: T, y: T) -> T: ...
    def shifted_chebyshev_polynomial_u(self, x: T, y: T) -> T: ...
    def shifted_chebyshev_polynomial_v(self, x: T, y: T) -> T: ...
    def shifted_chebyshev_polynomial_w(self, x: T, y: T) -> T: ...
    def hermite_polynomial_h(self, x: T, y: T) -> T: ...
    def hermite_polynomial_he(self, x: T, y: T) -> T: ...
    def laguerre_polynomial_l(self, x: T, y: T) -> T: ...
    def truncdiv(self, x0: T, x1: T) -> T: ...
    def floordiv(self, x0: T, x1: T) -> T: ...
    def truediv(self, x0: T, x1: T) -> T: ...
    def int_truediv(self, x0: T, x1: T) -> T: ...
    def div(self, x0: T, x1: T) -> T: ...
    def mod(self, x0: T, x1: T) -> T: ...
    def remainder(self, x0: T, x1: T) -> T: ...
    def round_decimal(self, x0: T, x1: T) -> T: ...
    def libdevice_abs(self, x0: T) -> T: ...
    def libdevice_exp(self, x0: T) -> T: ...
    def libdevice_sqrt(self, x0: T) -> T: ...
    def libdevice_cos(self, x0: T) -> T: ...
    def libdevice_sin(self, x0: T) -> T: ...
    def libdevice_sigmoid(self, x0: T) -> T: ...
    def libdevice_log(self, x0: T) -> T: ...

class NoopHandler:
    def __getattr__(self, name): ...
    @staticmethod
    def masked(mask, body, other) -> None: ...
    @staticmethod
    def frexp(x) -> tuple[None, None]: ...
    @staticmethod
    def scan(dtypes, combine_fn, values) -> tuple[None, ...]: ...
    @staticmethod
    def sort(dtypes, values, stable, descending) -> tuple[None, ...]: ...
    @staticmethod
    def indirect_indexing(index_var, size, check: bool = True, wrap_neg: bool = True) -> sympy.Symbol: ...

class MockHandler:
    def __getattr__(self, name): ...
    @staticmethod
    def masked(mask, body, other) -> str: ...
    @staticmethod
    def frexp(x): ...
    @staticmethod
    def scan(dtypes, combine_fn, values): ...
    @staticmethod
    def sort(dtypes, values, stable, descending): ...
    @staticmethod
    def indirect_indexing(index_var, size, check: bool = True, wrap_neg: bool = True) -> sympy.Symbol: ...

class KernelFormatterHandler:
    parent_handler: Incomplete
    output: Incomplete
    var_counter: Incomplete
    def __init__(self, parent_handler) -> None: ...
    @staticmethod
    def ir_to_string(ir_fn, index, rindex: Incomplete | None = None) -> str: ...
    def __getattr__(self, name) -> Callable[..., Any]: ...
    def reduction(self, dtype: torch.dtype, src_dtype: torch.dtype, reduction_type: ReductionType, value: str | tuple[str, ...]) -> str | tuple[str, ...]: ...
    def getvalue(self, result): ...

class WrapperHandler(Generic[T]):
    def __init__(self, inner: OpsHandler[T]) -> None: ...
    def __getattr__(self, item): ...

class AddParenHandler(WrapperHandler[T]):
    def __getattr__(self, name): ...

class OpCountResult(NamedTuple):
    num_ops: int
    used_ops: OrderedSet[str]
    read_buffers: list[str]
    nontrivial_read_count: int

class OpCounterCSE:
    parent_handler: Incomplete
    op_count: int
    var_names: Incomplete
    def __init__(self, inner) -> None: ...
    def __getattr__(self, name): ...
    def indirect_indexing(self, *args, **kwargs): ...
    def load(self, name: str, index: sympy.Expr) -> str: ...
    def load_seed(self, name: str, offset: T): ...
    def bucketize(self, values: T, boundaries: tuple[str, sympy.Expr, sympy.Expr, sympy.Expr], boundary_indices: T, indexing_dtype: torch.dtype, right: bool, sorter: tuple[str, sympy.Expr] | None = None, sorter_indices: T | None = None) -> T: ...
    def getvalue(self): ...

class ExtractConstantsHandler(NoopHandler):
    device: Incomplete
    def __init__(self, device) -> None: ...
    def constant(self, value: Any, dtype: torch.dtype) -> torch._inductor.ir.Constant: ...

class SimpleCSEHandler(WrapperHandler[T]):
    cse_cache: dict[str, T | tuple[T, ...]]
    mock: Incomplete
    def __init__(self, inner: OpsHandler[T]) -> None: ...
    def indirect_indexing(self, *args, **kwargs) -> sympy.Expr: ...
    def store(self, *args, **kwargs) -> T: ...
    def store_reduction(self, *args, **kwargs) -> T: ...
    def __getattr__(self, name) -> Callable[..., Any]: ...
