import dataclasses
import torch
from . import config as config
from .compile_fx import _CompileFxKwargs
from .runtime.autotune_cache import AutotuneCacheBundler as AutotuneCacheBundler
from .triton_bundler import TritonKernelArtifacts as TritonKernelArtifacts
from _typeshed import Incomplete
from torch._dynamo.utils import counters as counters
from torch._inductor import metrics as metrics
from torch._inductor.cudagraph_utils import BoxedDeviceIndex as BoxedDeviceIndex, CudagraphCachedInfo as CudagraphCachedInfo, get_placeholder_info as get_placeholder_info, log_cudagraph_skip_and_bump_counter as log_cudagraph_skip_and_bump_counter
from torch._inductor.graph import GraphLowering as GraphLowering
from torch._inductor.utils import BoxedBool as BoxedBool, InputType as InputType, align_inputs_from_check_idxs as align_inputs_from_check_idxs, output_node as output_node, set_tracing_context_output_strides as set_tracing_context_output_strides
from typing import Any, Callable, Counter, Sequence

log: Incomplete

@dataclasses.dataclass
class OutputCode:
    def __call__(self, inputs: Sequence[Any]) -> Any: ...
    def post_compile(self, example_inputs: Sequence[InputType], cudagraphs: BoxedBool, constants: CompiledFxGraphConstants) -> None: ...
    def set_triton_bundle(self, triton_bundle: Any) -> None: ...

def has_frozen_params(gm: torch.fx.GraphModule) -> bool: ...
def get_expanded_dims(t: torch.Tensor) -> list[int]: ...
def index_expanded_dims(t: torch.Tensor, expanded_dims: list[int]) -> torch.Tensor: ...
def complex_memory_overlap(t: torch.Tensor) -> bool: ...
def cudagraph_post_compile(example_inputs: Sequence[InputType], compiled_graph: CompiledFxGraph, cudagraphs: BoxedBool, constants: dict[str, torch.Tensor]) -> None: ...
def maybe_realign_inputs(ran_cudagraphs: BoxedBool, compiled_graph: CompiledFxGraph, inputs_to_check: Sequence[int]) -> None: ...

class CompiledFxGraphConstants:
    def unwrap(self, g: CompiledFxGraph) -> dict[str, torch.Tensor]: ...

class CompiledFxGraphConstantsWithGm(CompiledFxGraphConstants):
    gm: Incomplete
    def __init__(self, gm: torch.fx.GraphModule) -> None: ...
    def unwrap(self, g: CompiledFxGraph) -> dict[str, torch.Tensor]: ...

@dataclasses.dataclass
class CompiledFxGraph(OutputCode):
    current_callable: Callable[..., Any] | None
    cache_key: str
    source_code: str = dataclasses.field(repr=False)
    cache_linemap: list[tuple[int, str]] | None
    device_types: set[str]
    device_idxs: set[int]
    mutated_inputs: set[str]
    mutated_input_idxs: set[int]
    allocated_constant_name: dict[str, str] | None
    constants: dict[str, torch.Tensor] | None
    torchbind_constants: dict[str, torch._C.ScriptObject]
    output_strides: list[tuple[_StrideExprStr, ...] | None] | None
    disabled_cudagraphs_reason: str | None
    metrics_deltas: metrics.CachedMetricsDeltas
    counter_deltas: Counter[str]
    guards_expr: str | None
    cudagraph_info: CudagraphCachedInfo | None
    fx_kwargs: _CompileFxKwargs
    inputs_to_check: Sequence[int]
    boxed_forward_device_index: BoxedDeviceIndex | None
    def __init__(self, current_callable: Callable[..., Any] | None, graph: GraphLowering, gm: torch.fx.GraphModule, output_strides: list[tuple[_StrideExprStr, ...] | None], disabled_cudagraphs_reason: str | None, metrics_deltas: metrics.CachedMetricsDeltas, counter_deltas: Counter[str], cudagraphs: BoxedBool, example_inputs: Sequence[InputType], static_input_idxs: Sequence[int], fx_kwargs: _CompileFxKwargs, inputs_to_check: Sequence[int], boxed_forward_device_index: BoxedDeviceIndex | None) -> None: ...
    def __call__(self, inputs: Sequence[Any]) -> Any: ...
    def post_compile(self, example_inputs: Sequence[InputType], cudagraphs: BoxedBool, constants: CompiledFxGraphConstants) -> None: ...
    def set_triton_bundle(self, triton_bundle: Any) -> None: ...
    def prepare_for_serialization(self) -> None: ...
    def after_deserialization(self, constants: CompiledFxGraphConstants) -> str: ...

@dataclasses.dataclass
class CompiledAOTI(OutputCode):
    filename: str | list[str]
    def __call__(self, inputs: Sequence[Any]) -> Any: ...
    def post_compile(self, example_inputs: Sequence[InputType], cudagraphs: BoxedBool, constants: CompiledFxGraphConstants) -> None: ...
    def set_triton_bundle(self, triton_bundle: Any) -> None: ...

@dataclasses.dataclass
class MockFXGraphCacheOutput(OutputCode):
    gm: Any = ...
    def __post_init__(self) -> None: ...
    def post_compile(self, example_inputs: Sequence[InputType], cudagraphs: BoxedBool, constants: CompiledFxGraphConstants) -> None: ...
    def __call__(self, inputs: Sequence[Any]) -> Any: ...
    def set_triton_bundle(self, triton_bundle: Any) -> None: ...
