import dataclasses
import sympy
import torch
from . import comms as comms, config as config, dependencies as dependencies, ir as ir, metrics as metrics
from .codegen.common import BackendFeature as BackendFeature, Kernel as Kernel, get_scheduling_for_device as get_scheduling_for_device
from .comm_analysis import estimate_nccl_collective_runtime as estimate_nccl_collective_runtime
from .dependencies import Dep as Dep, MemoryDep as MemoryDep, StarDep as StarDep, WeakDep as WeakDep
from .ir import ComputedBuffer as ComputedBuffer, MultiOutput as MultiOutput, MultiOutputLayout as MultiOutputLayout, get_device_type as get_device_type
from .loop_body import LoopBody as LoopBody
from .memory import MemoryPlanningInfoForBuffer as MemoryPlanningInfoForBuffer, MemoryPlanningInfoForNode as MemoryPlanningInfoForNode
from .runtime.runtime_utils import green_text as green_text, red_text as red_text
from .sizevars import SimplifyIndexing as SimplifyIndexing
from .utils import IndentedBuffer as IndentedBuffer, cache_on_self as cache_on_self, cmp as cmp, device_need_guard as device_need_guard, get_device_tflops as get_device_tflops, get_dtype_size as get_dtype_size, get_gpu_dram_gbps as get_gpu_dram_gbps, is_collective as is_collective, is_gpu as is_gpu, is_wait as is_wait, sympy_product as sympy_product
from .virtualized import V as V
from _typeshed import Incomplete
from torch._dynamo.utils import counters as counters, dynamo_timed as dynamo_timed
from torch._inductor.metrics import get_metric_table as get_metric_table, is_metric_table_enabled as is_metric_table_enabled
from torch.fx.experimental.symbolic_shapes import free_unbacked_symbols as free_unbacked_symbols
from torch.utils._ordered_set import OrderedSet as OrderedSet
from torch.utils._sympy.symbol import SymT as SymT, free_symbol_is_type as free_symbol_is_type
from torch.utils._triton import has_triton as has_triton
from typing import Any, Callable, Sequence

log: Incomplete
fusion_log: Incomplete
loop_ordering_log: Incomplete

@dataclasses.dataclass
class SchedulerBuffer:
    scheduler: Scheduler
    node: ir.Buffer
    defining_op: BaseSchedulerNode
    users: list[NodeUser] = dataclasses.field(default_factory=list)
    mpi_buffer: MemoryPlanningInfoForBuffer = dataclasses.field(default_factory=MemoryPlanningInfoForBuffer)
    def __hash__(self) -> int: ...
    def debug_str(self) -> str: ...
    def get_name(self) -> str: ...
    def allocate(self) -> None: ...
    def can_free(self) -> bool: ...
    def set_users(self, users: list[NodeUser]) -> None: ...
    def get_aliases(self) -> Sequence[str]: ...
    def get_mutations(self) -> Sequence[str]: ...

@dataclasses.dataclass
class SchedulerDonatedBuffer(SchedulerBuffer):
    defining_op: BaseSchedulerNode | None = ...

class BaseSchedulerNode:
    group: tuple[torch.device, tuple[tuple[sympy.Expr, ...], ...]]
    read_writes: dependencies.ReadWrites
    unmet_dependencies: OrderedSet[Dep]
    min_order: int
    max_order: int
    mpi_node: MemoryPlanningInfoForNode
    scheduler: Scheduler
    debug_device_str: Callable[[BaseSchedulerNode], list[str]]
    def __init__(self, scheduler: Scheduler) -> None: ...
    def debug_str(self) -> str: ...
    def debug_str_extra(self) -> str: ...
    def debug_str_short(self) -> str: ...
    def log_details(self) -> None: ...
    def reorder_loops_by_dep_pair(self, self_dep: MemoryDep, other_dep: MemoryDep) -> None: ...
    def update_mutated_names(self, renames: dict[str, str]) -> None: ...
    def add_fake_dep(self, dep: Dep) -> None: ...
    def has_aliasing_or_mutation(self) -> bool: ...
    def set_read_writes(self, rw: dependencies.ReadWrites) -> None: ...
    last_usage: Incomplete
    def set_last_usage(self, future_used_buffers: OrderedSet[str], mutation_real_name: dict[str, str]) -> None: ...
    def mark_run(self) -> None: ...
    def used_buffer_names(self) -> OrderedSet[str]: ...
    def used_or_aliased_buffer_names(self) -> OrderedSet[str]: ...
    def prune_deps(self) -> None: ...
    def prune_weak_deps(self) -> None: ...
    def prune_redundant_deps(self, name_to_fused_node: dict[str, BaseSchedulerNode]) -> None: ...
    def get_name(self) -> str: ...
    def get_first_name(self) -> str: ...
    def get_operation_names(self) -> OrderedSet[str]: ...
    def get_buffer_names(self) -> OrderedSet[str]: ...
    def get_nodes(self) -> Sequence[BaseSchedulerNode]: ...
    def get_outputs(self) -> Sequence[SchedulerBuffer]: ...
    def get_output(self, buf_name: str) -> SchedulerBuffer: ...
    def get_device(self) -> torch.device | None: ...
    def is_cpu(self) -> bool: ...
    def is_gpu(self) -> bool: ...
    def is_reduction(self) -> bool: ...
    def is_split_scan(self) -> bool: ...
    def is_template(self) -> bool: ...
    def is_extern(self) -> bool: ...
    def is_foreach(self) -> bool: ...
    def can_inplace(self, read_dep: dependencies.Dep) -> bool: ...
    def has_side_effects(self) -> bool: ...
    def decide_inplace_update(self) -> None: ...
    written: bool
    def codegen_originating_info(self, buffer: IndentedBuffer, only_once: bool = True) -> None: ...
    def get_read_write_buffers_sizes(self) -> int: ...
    def get_estimated_runtime(self) -> float: ...
    def get_template_node(self) -> ir.TemplateBuffer | None: ...

class WhyNoFuse:
    reason: str
    args: tuple[Any, ...]
    node1: Incomplete
    node2: Incomplete
    def __init__(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> None: ...
    def __call__(self, reason: str, *args: Any) -> None: ...

def pformat(obj: Any) -> str: ...

class OutputNode:
    unmet_dependencies: Incomplete
    def __init__(self, dep: StarDep) -> None: ...
    def is_reduction(self) -> bool: ...
    def get_inputs_that_alias_output(self) -> Sequence[str]: ...
    def get_name(self) -> str: ...

kernel_name_to_op: Incomplete

class ExternKernelSchedulerNode(BaseSchedulerNode):
    def __init__(self, scheduler: Scheduler, node: ir.Operation) -> None: ...
    def debug_str_extra(self) -> str: ...
    def is_extern(self) -> bool: ...
    def has_side_effects(self) -> bool: ...

class NopKernelSchedulerNode(BaseSchedulerNode):
    def __init__(self, scheduler: Scheduler, node: ir.Operation) -> None: ...

class SchedulerNode(BaseSchedulerNode):
    def __init__(self, scheduler: Scheduler, node: ir.ComputedBuffer | ir.TemplateBuffer) -> None: ...
    def recompute_size_and_body(self, extra_indexing_constraints: tuple[dict[Any, Any], list[Any]] | None = None, recompute_sizes_body_func: Callable[..., Any] | None = None) -> None: ...
    def refresh_dependencies(self, normalize: bool) -> None: ...
    def apply_new_loop_order(self, new_order: Sequence[int]) -> None: ...
    def reorder_loops_by_dep_pair(self, self_dep: MemoryDep, other_dep: MemoryDep) -> None: ...
    def debug_str_extra(self) -> str: ...
    def get_ranges(self) -> Sequence[Sequence[sympy.Expr]]: ...
    def is_reduction(self) -> bool: ...
    def is_split_scan(self) -> bool: ...
    def is_template(self) -> bool: ...
    def get_template_node(self) -> ir.TemplateBuffer | None: ...
    def run(self, *index_vars: Sequence[sympy.Expr]) -> None: ...
    def ranges_from_index_vars(self, index_vars: Sequence[Sequence[sympy.Expr]]) -> dict[sympy.Expr, sympy.Expr]: ...
    def codegen(self, index_vars: Sequence[Sequence[sympy.Expr]]) -> None: ...
    def pointwise_read_writes(self) -> dependencies.ReadWrites: ...
    def can_inplace(self, read_dep: dependencies.Dep) -> bool: ...

def refresh_group_node_dependencies(group_snode: BaseSchedulerNode) -> None: ...
def init_group_node(group_snode: BaseSchedulerNode, scheduler: Scheduler, snodes: list[BaseSchedulerNode]) -> None: ...

class FusedSchedulerNode(BaseSchedulerNode):
    snodes: list[BaseSchedulerNode]
    @classmethod
    def fuse(cls, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> FusedSchedulerNode: ...
    def reorder_loops_by_dep_pair(self, self_dep: MemoryDep, other_dep: MemoryDep) -> None: ...
    users: list[NodeUser]
    group: Incomplete
    def __init__(self, scheduler: Scheduler, snodes: list[BaseSchedulerNode]) -> None: ...
    def get_name(self) -> str: ...
    def get_first_name(self) -> str: ...
    def get_buffer_names(self) -> OrderedSet[str]: ...
    def get_outputs(self) -> list[SchedulerBuffer]: ...
    def debug_str_extra(self) -> str: ...
    def debug_str_short(self) -> str: ...
    def set_last_usage(self, future_used_buffers: OrderedSet[str], mutation_real_name: dict[str, str]) -> None: ...
    def used_buffer_names(self) -> OrderedSet[str]: ...
    def used_or_aliased_buffer_names(self) -> OrderedSet[str]: ...
    def get_nodes(self) -> Sequence[BaseSchedulerNode]: ...
    def is_reduction(self) -> bool: ...
    def is_split_scan(self) -> bool: ...
    def is_template(self) -> bool: ...
    def get_template_node(self) -> ir.TemplateBuffer | None: ...
    def get_device(self) -> torch.device: ...
    def has_aliasing_or_mutation(self) -> bool: ...
    def update_mutated_names(self, renames: dict[str, str]) -> None: ...
    def add_fake_dep(self, name: Dep) -> None: ...
    def can_inplace(self, read_dep: dependencies.Dep) -> bool: ...
    def debug_str(self) -> str: ...

class ForeachKernelSchedulerNode(FusedSchedulerNode):
    def get_consumer_subnode_for(self, producer: BaseSchedulerNode) -> BaseSchedulerNode | None: ...
    def get_producer_subnode_for(self, consumer: BaseSchedulerNode) -> BaseSchedulerNode | None: ...
    @classmethod
    def can_fuse(cls, producer: BaseSchedulerNode, consumer: BaseSchedulerNode) -> bool: ...
    @classmethod
    def fuse(cls, producer: BaseSchedulerNode, consumer: BaseSchedulerNode) -> ForeachKernelSchedulerNode: ...
    read_to_node: Incomplete
    name_to_node: Incomplete
    scheduler: Incomplete
    snodes: Incomplete
    node: Incomplete
    users: list[NodeUser]
    unmet_dependencies: Incomplete
    min_order: Incomplete
    max_order: Incomplete
    ancestors: Incomplete
    use_custom_partition_algo: Incomplete
    group: Incomplete
    origins: OrderedSet[torch.fx.Node]
    enable_autotune: Incomplete
    def __init__(self, scheduler: Scheduler, snodes: list[BaseSchedulerNode], use_custom_partition_algo: bool, prev_node_1: BaseSchedulerNode | None = None, prev_node_2: BaseSchedulerNode | None = None, enable_autotune: bool = False) -> None: ...
    @classmethod
    def combinable_nodes(cls, nodes: list[BaseSchedulerNode]) -> list[BaseSchedulerNode]: ...
    group_algorithm_for_combo_kernels: Callable[[Scheduler], list[list[BaseSchedulerNode]]]
    @staticmethod
    def set_group_algorithm_for_combo_kernels(custom_group_algorithm: Callable[[Scheduler], list[list[BaseSchedulerNode]]]) -> None: ...
    @staticmethod
    def group_nodes_for_combo_kernels(scheduler: Scheduler) -> list[list[BaseSchedulerNode]]: ...
    def mark_run(self) -> None: ...
    def codegen(self) -> None: ...
    def is_foreach(self) -> bool: ...
    def get_subkernel_nodes(self) -> list[BaseSchedulerNode]: ...
    def get_nodes(self) -> Sequence[BaseSchedulerNode]: ...
    def get_first_name(self) -> str: ...
    def prune_redundant_deps(self, name_to_fused_node: dict[str, BaseSchedulerNode]) -> None: ...

class GroupedSchedulerNode(BaseSchedulerNode):
    snodes: list[BaseSchedulerNode]
    @classmethod
    def create(cls, snodes: list[BaseSchedulerNode]) -> GroupedSchedulerNode: ...
    def __init__(self, scheduler: Scheduler, snodes: list[BaseSchedulerNode]) -> None: ...
    def unpack(self) -> list[BaseSchedulerNode]: ...
    def add_fake_dep(self, fake_dep: Dep) -> None: ...
    def get_name(self) -> str: ...
    def get_first_name(self) -> str: ...
    def get_buffer_names(self) -> OrderedSet[str]: ...
    def get_outputs(self) -> list[SchedulerBuffer]: ...
    def get_nodes(self) -> Sequence[BaseSchedulerNode]: ...
    @classmethod
    def can_fuse(cls, producer: BaseSchedulerNode, consumer: BaseSchedulerNode) -> bool: ...

def pick_loop_order(stride_lengths: list[list[int]], sizes: Sequence[sympy.Expr], priority_idx: tuple[int, ...] = ()) -> list[int]: ...

@dataclasses.dataclass
class NodeUser:
    node: BaseSchedulerNode | OutputNode
    can_inplace: bool = ...
    is_weak: bool = ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    def get_name(self) -> str: ...
    def merge(self, other: NodeUser) -> NodeUser: ...

class Scheduler:
    def __init__(self, nodes: list[ir.Operation]) -> None: ...
    def get_donated_buffers(self) -> dict[str, SchedulerDonatedBuffer]: ...
    @property
    def current_device(self) -> torch.device | None: ...
    @current_device.setter
    def current_device(self, device: torch.device | None) -> None: ...
    def debug_draw_graph(self) -> None: ...
    def debug_print_nodes(self, label: str) -> None: ...
    def create_scheduler_node(self, node: ir.Operation) -> BaseSchedulerNode: ...
    nodes: Incomplete
    def create_foreach_nodes(self) -> None: ...
    items: Incomplete
    membership: Incomplete
    def compute_dependencies(self) -> None: ...
    def dead_node_elimination(self) -> None: ...
    def topological_sort_schedule(self, nodes: list[BaseSchedulerNode]) -> list[BaseSchedulerNode]: ...
    def compute_ancestors(self) -> None: ...
    def merge_loops(self) -> None: ...
    def fuse_nodes(self, nodes: list[BaseSchedulerNode]) -> list[BaseSchedulerNode]: ...
    def process_grouped_nodes(self) -> None: ...
    def benchmark_fused_nodes(self, nodes: Sequence[BaseSchedulerNode]) -> tuple[float, str]: ...
    def finalize_multi_template_buffers(self) -> None: ...
    def speedup_by_fusion(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> bool: ...
    def fuse_nodes_once(self, nodes: list[BaseSchedulerNode]) -> list[BaseSchedulerNode]: ...
    def create_combo_kernel_nodes(self, num_ck_nodes: int | None = None) -> None: ...
    def prune_redundant_deps(self, nodes: list[BaseSchedulerNode]) -> None: ...
    def get_possible_fusions(self, nodes: list[BaseSchedulerNode]) -> list[tuple[BaseSchedulerNode, BaseSchedulerNode]]: ...
    def will_fusion_create_cycle(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> bool: ...
    def can_fusion_increase_peak_memory(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> bool: ...
    def are_long_distant_nodes(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> bool: ...
    def decide_fusion_fail_reason(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode, common_buf_names: tuple[str, ...]) -> str: ...
    def shared_data_after_reordering_loop(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> int: ...
    def unfusable_node(self, node: BaseSchedulerNode) -> bool: ...
    def can_fuse(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> bool: ...
    def can_fuse_vertical(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> bool: ...
    def fusable_weak_dep(self, weak_dep: WeakDep, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> bool: ...
    def fusable_read_and_write(self, read: Dep, write: MemoryDep) -> bool: ...
    def dep_size_hint(self, dep: Dep) -> int: ...
    def score_fusion_memory(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> int: ...
    def get_possible_fusions_with_highest_priority(self, possible_fusions: list[tuple[BaseSchedulerNode, BaseSchedulerNode]]) -> list[tuple[BaseSchedulerNode, BaseSchedulerNode]]: ...
    def score_fusion_key(self, nodes: tuple[BaseSchedulerNode, BaseSchedulerNode]) -> Any: ...
    def compute_last_usage(self) -> None: ...
    def free_buffers(self) -> None: ...
    def flush(self) -> None: ...
    def codegen_extern_call(self, scheduler_node: ExternKernelSchedulerNode) -> None: ...
    def create_backend(self, device: torch.device) -> BaseScheduling: ...
    def get_backend(self, device: torch.device | None) -> BaseScheduling: ...
    def enter_context(self, node: BaseSchedulerNode) -> None: ...
    def can_buffer_be_removed_through_fusion(self, name: str, fused_node_names: OrderedSet[str]) -> bool: ...
    def codegen(self) -> None: ...
    def benchmark_combo_kernel(self, node_list: Sequence[BaseSchedulerNode]) -> tuple[float, float, str]: ...
    def speedup_by_combo_kernel(self, nodes: list[BaseSchedulerNode]) -> bool: ...
    def get_buffer_layout(self, buf_name: str) -> ir.Layout: ...
    def update_zero_dim_cpu_tensor(self) -> None: ...

class BaseScheduling:
    @classmethod
    def get_backend_features(cls, device: torch.device) -> Sequence[BackendFeature]: ...
    def can_fuse_vertical(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> bool: ...
    def can_fuse_horizontal(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> bool: ...
    def fuse(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> FusedSchedulerNode: ...
    def group_fn(self, sizes: Sequence[Sequence[sympy.Expr]]) -> tuple[tuple[sympy.Expr, ...], ...]: ...
    def codegen_template(self, template_node: BaseSchedulerNode, epilogue_nodes: Sequence[BaseSchedulerNode]) -> str | None: ...
    def codegen_node(self, node: FusedSchedulerNode | SchedulerNode) -> None: ...
    def codegen_sync(self) -> None: ...
    def ready_to_flush(self) -> bool: ...
    def flush(self) -> None: ...
    def benchmark_fused_nodes(self, nodes: Sequence[BaseSchedulerNode]) -> tuple[float, str]: ...
    def get_fusion_pair_priority(self, node1: BaseSchedulerNode, node2: BaseSchedulerNode) -> int: ...
    def benchmark_combo_kernel(self, node_list: Sequence[BaseSchedulerNode]) -> tuple[float, float, str]: ...
