import dataclasses
import sympy
import torch
from . import config as config, ir as ir, lowering as lowering
from .autotune_process import TensorMeta as TensorMeta, TritonBenchmarkRequest as TritonBenchmarkRequest, TritonCPUBenchmarkRequest as TritonCPUBenchmarkRequest, TritonGPUBenchmarkRequest as TritonGPUBenchmarkRequest
from .codecache import PersistentCache as PersistentCache, PyCodeCache as PyCodeCache, code_hash as code_hash
from .codegen.common import CSEVariable as CSEVariable, IndentedBuffer as IndentedBuffer, KernelTemplate as KernelTemplate, OpOverrides as OpOverrides, WorkspaceArg as WorkspaceArg
from .codegen.simd_kernel_features import SIMDKernelFeatures as SIMDKernelFeatures
from .codegen.triton import TritonKernel as TritonKernel, TritonScheduling as TritonScheduling, gen_common_triton_imports as gen_common_triton_imports, texpr as texpr
from .codegen.triton_utils import config_of as config_of, signature_to_meta as signature_to_meta
from .exc import CUDACompileError as CUDACompileError
from .ir import ChoiceCaller as ChoiceCaller, PrimitiveInfoType as PrimitiveInfoType
from .ops_handler import StoreMode as StoreMode
from .runtime.benchmarking import benchmarker as benchmarker
from .runtime.hints import DeviceProperties as DeviceProperties
from .utils import FakeIndentedBuffer as FakeIndentedBuffer, Placeholder as Placeholder, get_dtype_size as get_dtype_size, restore_stdout_stderr as restore_stdout_stderr, sympy_dot as sympy_dot, sympy_index_symbol as sympy_index_symbol, sympy_product as sympy_product, triton_type_to_torch as triton_type_to_torch, unique as unique
from .virtualized import V as V
from _typeshed import Incomplete
from torch._dynamo.testing import rand_strided as rand_strided
from torch._dynamo.utils import counters as counters, dynamo_timed as dynamo_timed, identity as identity, preserve_rng_state as preserve_rng_state
from typing import Any, Callable, NamedTuple

log: Incomplete
VERIFY: dict[str, Any]
PRINT_AUTOTUNE: bool
DEBUG: bool

class KernelNamespace: ...

extern_kernels: Incomplete

@dataclasses.dataclass
class BenchmarkTensors:
    input_tensors: list[torch.Tensor]
    output_tensor: torch.Tensor | None
    def unpack(self): ...

@dataclasses.dataclass
class AutotuneArgs:
    triton: BenchmarkTensors
    extern: BenchmarkTensors
    expected: torch.Tensor | None = ...
    def get_benchmark_tensors(self, extern: bool = False) -> BenchmarkTensors: ...
    @classmethod
    def from_choice_args(cls, example_inputs: list[torch.Tensor], example_inputs_extern: list[torch.Tensor], out: torch.Tensor, out_extern: torch.Tensor, expected: torch.Tensor | None = None) -> _T: ...
    def verify(self, **kwargs) -> None: ...

class PartialRender:
    code: Incomplete
    replacement_hooks: Incomplete
    def __init__(self, code, replacement_hooks) -> None: ...
    def finalize_hook(self, hook_key: str, strict: bool = True) -> None: ...
    def finalize_all(self) -> str: ...

class SubgraphInfo(NamedTuple):
    body: Incomplete
    template_mask: Incomplete
    template_out: Incomplete

class ModificationWrapper(V.WrapperHandler):
    name: Incomplete
    kernel: Incomplete
    fixed_inputs: Incomplete
    mask: Incomplete
    def __init__(self, kernel, subgraph_number: int, fixed_inputs: dict[str, Any], mask: str | None) -> None: ...
    def load(self, name: str, index: sympy.Expr): ...
    def indirect_indexing(self, index_var: str, size, check, wrap_neg: bool = True): ...
    def store(self, name: str, index: sympy.Expr, value: CSEVariable, mode: StoreMode = None) -> str: ...

class TritonTemplateKernel(TritonKernel):
    input_nodes: Incomplete
    output_node: Incomplete
    named_input_nodes: Incomplete
    defines: Incomplete
    kernel_name: Incomplete
    use_jit: Incomplete
    num_stages: Incomplete
    num_warps: Incomplete
    grid_fn: Incomplete
    meta: Incomplete
    call_sizes: Incomplete
    prefix_args: Incomplete
    suffix_args: Incomplete
    epilogue_fn: Incomplete
    render_hooks: Incomplete
    triton_meta: dict[str, object] | None
    subgraphs: list[ir.ComputedBuffer] | None
    workspace_arg: Incomplete
    subgraph_bodies: dict[str, SubgraphInfo]
    body: IndentedBuffer
    template_mask: str | None
    template_out: str | None
    def __init__(self, kernel_name, input_nodes, output_node, defines, num_stages, num_warps, grid_fn, meta, call_sizes, use_jit: bool = False, prefix_args: int = 0, suffix_args: int = 0, epilogue_fn=..., subgraphs: list[ir.ComputedBuffer] | None = None, workspace_arg: WorkspaceArg | None = None) -> None: ...
    def set_subgraph_body(self, body_name: str): ...
    def create_subgraph_body(self, body_name: str): ...
    def need_numel_args(self): ...
    def estimate_kernel_num_bytes(self): ...
    def jit_lines(self): ...
    def gen_argdefs(self): ...
    def gen_defines(self): ...
    def def_kernel(self, *argnames): ...
    def size(self, name: str, index: int): ...
    def stride(self, name, index: Incomplete | None = None): ...
    def modification(self, subgraph_number: int, output_name: str | None, mask: str | None = None, **fixed_inputs) -> str: ...
    template_indices: Incomplete
    def store_output(self, indices: list[Any] | tuple[Any], val: str, mask: str | None = None, indent_width: int = 4): ...
    def render(self, template, kwargs): ...
    def make_load(self, name, indices, mask): ...
    def template_env(self): ...
    def indexing(self, index: sympy.Expr, *, dense_indexing: bool = False, copy_shape: Incomplete | None = None, override_mask: Incomplete | None = None, block_ptr: bool = False): ...
    def codegen_range_tree(self) -> None: ...
    def call_kernel(self, name: str, node: ir.IRNode | None = None): ...

class TritonTemplate(KernelTemplate):
    index_counter: Incomplete
    all_templates: dict[str, 'TritonTemplate']
    grid: Incomplete
    template: Incomplete
    debug: Incomplete
    def __init__(self, name: str, grid: Any, source: str, debug: bool = False) -> None: ...
    def generate(self, input_nodes, layout, num_stages, num_warps, prefix_args: int = 0, suffix_args: int = 0, epilogue_fn=..., subgraphs: Incomplete | None = None, mutated_inputs: Incomplete | None = None, call_sizes: Incomplete | None = None, workspace_arg: WorkspaceArg | None = None, **kwargs): ...

class ExternKernelChoice:
    name: Incomplete
    cpp_kernel_name: Incomplete
    has_out_variant: Incomplete
    op_overload: Incomplete
    use_fallback_kernel: Incomplete
    kernel_creator: Incomplete
    def __init__(self, kernel, cpp_kernel: Incomplete | None = None, *, name: Incomplete | None = None, has_out_variant: bool = True, op_overload: Incomplete | None = None, use_fallback_kernel: bool = False, kernel_creator: Incomplete | None = None) -> None: ...
    def to_callable(self): ...
    def call_name(self): ...
    def hash_key(self): ...
    ordered_kwargs_for_cpp_kernel: Incomplete
    def bind(self, input_nodes, layout, ordered_kwargs_for_cpp_kernel=(), **kwargs): ...

class TritonTemplateCaller(ir.TritonTemplateCallerBase):
    make_kernel_render: Incomplete
    bmreq: TritonBenchmarkRequest
    log_info: dict[str, Any]
    mutated_inputs: Incomplete
    workspace_arg: Incomplete
    def __init__(self, name, input_nodes, layout, make_kernel_render, description, bmreq, log_info: dict[str, PrimitiveInfoType | list[PrimitiveInfoType]] | None = None, mutated_inputs: Incomplete | None = None, workspace_arg: WorkspaceArg | None = None) -> None: ...
    def benchmark(self, *args, out): ...
    def precompile(self) -> None: ...
    def call_name(self): ...
    def hash_key(self): ...
    def output_node(self): ...
    def info_dict(self) -> dict[str, PrimitiveInfoType | list[PrimitiveInfoType]]: ...
    def get_make_kernel_render(self): ...
    def autoheuristic_id(self): ...

class ExternKernelCaller(ChoiceCaller):
    choice: Incomplete
    kwargs: Incomplete
    has_out_variant: Incomplete
    def __init__(self, choice: ExternKernelChoice, input_nodes, layout, kwargs: Incomplete | None = None, *, has_out_variant: bool = True) -> None: ...
    def benchmark(self, *args, out): ...
    def to_callable(self): ...
    def hash_key(self): ...
    def output_node(self): ...
    def info_dict(self) -> dict[str, PrimitiveInfoType | list[PrimitiveInfoType]]: ...
    def autoheuristic_id(self): ...

def get_mm_log_filename() -> str | None: ...
def append_to_log(filename, data) -> None: ...

class DataProcessorChoiceCallerWrapper:
    def __init__(self, wrapped, preprocessor, postprocessor) -> None: ...
    def __getattr__(self, name): ...
    def benchmark(self, *args, out) -> float: ...
    def output_node(self) -> ir.TensorBox: ...

class DataProcessorTemplateWrapper:
    def __init__(self, wrapped_template_cls, preprocessor, postprocessor, **kwargs) -> None: ...
    def __getattr__(self, name): ...
    def maybe_append_choice(self, choices, **kwargs): ...
    def generate(self, **kwargs): ...

class ErrorFromChoice(RuntimeError):
    choice: Incomplete
    def __init__(self, msg, choice: ChoiceCaller, inputs_str) -> None: ...

class NoValidChoicesError(RuntimeError): ...

def get_env_num_workers() -> int | None: ...
def create_inputs_key(input_nodes) -> str: ...
def create_precompile_key(name: str, inputs_key: str, choices: list[ChoiceCaller]) -> str: ...

class AlgorithmSelectorCache(PersistentCache):
    precompile_cache: dict[str, Callable[[], None]]
    feedback_saver_fns: list[Callable[[dict[ChoiceCaller, float], str, list[Any], list[ChoiceCaller]], None]]
    def __init__(self, *args, **kwargs) -> None: ...
    def __call__(self, name, choices: list[ChoiceCaller], input_nodes, layout, input_gen_fns: dict[int, Callable[[ir.Buffer], torch.Tensor]] | None = None, precompilation_timeout_seconds: int = ..., return_multi_template: bool = False): ...
    @classmethod
    def make_benchmark_fn(cls, choices, input_nodes, layout, input_gen_fns: Incomplete | None = None): ...
    @staticmethod
    def log_results(name: str, input_nodes: list[ir.IRNode], timings: dict[ChoiceCaller, float], elapse: float, precompile_elapse: float): ...
    @staticmethod
    def benchmark_example_value(node): ...
    @staticmethod
    def generate_example_value(size, stride, device, dtype, extra_size): ...
    @staticmethod
    def key_of(node): ...
    def add_feedback_saver(self, fn: Callable[[dict[ChoiceCaller, float], str, list[Any], list[ChoiceCaller]], None]): ...

def autotune_select_algorithm(*args, **kwargs): ...
def add_feedback_saver(fn: Callable[[dict[ChoiceCaller, float], str, list[Any], list[ChoiceCaller]], None]): ...
def realize_inputs(*args): ...
