import logging
from _typeshed import Incomplete
from dataclasses import dataclass, field
from torch._utils_internal import log_trace_structured_event as log_trace_structured_event
from torch.utils._traceback import CapturedTraceback as CapturedTraceback
from typing import Any, Callable

log: Incomplete
trace_log: Incomplete
DEFAULT_LOG_LEVEL: Incomplete
LOG_ENV_VAR: str
LOG_OUT_ENV_VAR: str
LOG_FORMAT_ENV_VAR: str
TRACE_ENV_VAR: str
LOG_TRACE_HANDLER: LazyTraceHandler | None

@dataclass
class LogRegistry:
    log_alias_to_log_qnames: dict[str, list[str]] = field(default_factory=dict)
    artifact_log_qnames: set[str] = field(default_factory=set)
    child_log_qnames: set[str] = field(default_factory=set)
    artifact_names: set[str] = field(default_factory=set)
    visible_artifacts: set[str] = field(default_factory=set)
    artifact_descriptions: dict[str, str] = field(default_factory=dict)
    off_by_default_artifact_names: set[str] = field(default_factory=set)
    artifact_log_formatters: dict[str, logging.Formatter] = field(default_factory=dict)
    def is_artifact(self, name): ...
    def is_log(self, alias): ...
    def register_log(self, alias, log_qnames: str | list[str]): ...
    def register_artifact_name(self, name, description, visible, off_by_default, log_format) -> None: ...
    def register_artifact_log(self, artifact_log_qname) -> None: ...
    def register_child_log(self, log_qname) -> None: ...
    def get_log_qnames(self) -> set[str]: ...
    def get_artifact_log_qnames(self): ...
    def get_child_log_qnames(self): ...
    def is_off_by_default(self, artifact_qname): ...

@dataclass
class LogState:
    log_qname_to_level: dict[str, str] = field(default_factory=dict)
    artifact_names: set[str] = field(default_factory=set)
    def enable_artifact(self, artifact_name) -> None: ...
    def is_artifact_enabled(self, name): ...
    def enable_log(self, log_qnames, log_level) -> None: ...
    def get_log_level_pairs(self): ...
    def clear(self) -> None: ...

log_registry: Incomplete
log_state: Incomplete
DEFAULT_LOGGING: Incomplete

def set_logs(*, all: int | None = None, dynamo: int | None = None, aot: int | None = None, autograd: int | None = None, dynamic: int | None = None, inductor: int | None = None, distributed: int | None = None, c10d: int | None = None, ddp: int | None = None, fsdp: int | None = None, dtensor: int | None = None, onnx: int | None = None, bytecode: bool = False, aot_graphs: bool = False, aot_joint_graph: bool = False, ddp_graphs: bool = False, graph: bool = False, graph_code: bool = False, graph_breaks: bool = False, graph_sizes: bool = False, guards: bool = False, recompiles: bool = False, recompiles_verbose: bool = False, trace_source: bool = False, trace_call: bool = False, trace_bytecode: bool = False, output_code: bool = False, kernel_code: bool = False, schedule: bool = False, perf_hints: bool = False, post_grad_graphs: bool = False, onnx_diagnostics: bool = False, fusion: bool = False, overlap: bool = False, export: int | None = None, modules: dict[str, int | bool] | None = None, cudagraphs: bool = False, sym_node: bool = False, compiled_autograd: bool = False, compiled_autograd_verbose: bool = False, cudagraph_static_inputs: bool = False, benchmarking: bool = False, graph_region_expansion: bool = False): ...
def get_loggers(): ...
def register_log(setting_name, log_name) -> None: ...
def register_artifact(setting_name, description, visible: bool = False, off_by_default: bool = False, log_format: Incomplete | None = None) -> None: ...
def getArtifactLogger(module_qname, artifact_name): ...

INCR_VERBOSITY_CHAR: str
DECR_VERBOSITY_CHAR: str
VERBOSITY_REGEX: Incomplete

def configure_artifact_log(log) -> None: ...
def help_message(verbose: bool = False): ...
def make_module_path_relative(abs_path): ...

class TorchLogsFormatter(logging.Formatter):
    def __init__(self, *, trace: bool = False) -> None: ...
    def format(self, record): ...

DEFAULT_FORMATTER: Incomplete
handlers: Incomplete

class LazyTraceHandler(logging.StreamHandler):
    root_dir: Incomplete
    stream: Incomplete
    def __init__(self, root_dir: str | None) -> None: ...
    def close(self) -> None: ...
    def emit(self, record) -> None: ...

def warning_once(logger_obj, *args, **kwargs) -> None: ...

class LazyString:
    func: Incomplete
    args: Incomplete
    kwargs: Incomplete
    def __init__(self, func, *args, **kwargs) -> None: ...

structured_logging_overhead: dict[str, float]

def add_structured_logging_overhead(time_spent: float) -> None: ...
def get_structured_logging_overhead() -> float | None: ...
def trace_structured_artifact(name: str, encoding: str, payload_fn: Callable[[], str | object | None] = ...) -> None: ...
def trace_structured(name: str, metadata_fn: Callable[[], dict[str, Any] | tuple[str, int]] = ..., *, payload_fn: Callable[[], str | object | None] = ..., suppress_context: bool = False, expect_trace_id: bool = True, record_logging_overhead: bool = True) -> None: ...

GET_DTRACE_STRUCTURED: bool

def dtrace_structured(name: str, metadata_fn: Callable[[], dict[str, Any] | tuple[str, int]] = ..., *, payload_fn: Callable[[], str | object | None] = ..., suppress_context: bool = False, expect_trace_id: bool = True, record_logging_overhead: bool = True): ...
