import abc
import torch
import types
from _typeshed import Incomplete
from collections.abc import Generator
from torch._C import DispatchKey as DispatchKey
from torch._functorch.pyfunctorch import dispatch_functorch as dispatch_functorch
from torch.utils._python_dispatch import TorchDispatchMode as TorchDispatchMode
from typing import Any, Callable

def dl_open_guard() -> Generator[None]: ...

class OperatorBase:
    py_kernels: dict[DispatchKey, Callable[..., Any]]
    python_key_table: dict[type[TorchDispatchMode] | type[torch.Tensor], Callable[..., Any]]
    functorch_table: Incomplete
    def __init__(self) -> None: ...
    def __call__(self, *args, **kwargs) -> None: ...
    def has_kernel_for_dispatch_key(self, k): ...
    def has_kernel_for_any_dispatch_key(self, ks): ...
    def py_impl(self, k: Any) -> Callable[[_F], _F]: ...
    def py_functionalize_impl(self, fn: _F) -> _F: ...
    def name(self) -> None: ...

def resolve_key(op: OperatorBase, k: DispatchKey): ...

class HigherOrderOperator(OperatorBase, abc.ABC, metaclass=abc.ABCMeta):
    __module__: str
    non_fallthrough_keys: Incomplete
    def __init__(self, name, *, cacheable: bool = False) -> None: ...
    def py_impl(self, k: Any) -> Callable[[_F], _F]: ...
    @property
    def namespace(self): ...
    def cacheable(self): ...
    def fallthrough(self, dispatch_key) -> None: ...
    def dispatch(self, /, dispatch_key, *args, **kwargs): ...
    @abc.abstractmethod
    def __call__(self, /, *args, **kwargs): ...
    def name(self): ...

def key_extractor(tensors, key_mask): ...

class _ModeStackStateForPreDispatch:
    def __init__(self) -> None: ...
    def set(self, index, mode) -> None: ...
    def get(self, index): ...
    def count(self): ...

def unset_mode_pre_dispatch(mode_key, schema_check: bool = False): ...
def mode_stack_state_for_pre_dispatch(): ...

cached_ops: set['OpOverload']

def add_cached_op(op_overload) -> None: ...
def reset_cached_ops() -> None: ...
def get_cached_ops(): ...

class OpOverload(OperatorBase):
    __module__: Incomplete
    __qualname__: Incomplete
    __annotations__: Incomplete
    is_view: Incomplete
    def __init__(self, overloadpacket, op, op_dk, schema, tags) -> None: ...
    def __deepcopy__(self, memo: Incomplete | None = None): ...
    def __call__(self, /, *args, **kwargs): ...
    def redispatch(self, /, keyset, *args, **kwargs): ...
    def __hash__(self): ...
    def has_kernel_for_dispatch_key(self, k): ...
    def has_kernel_for_any_dispatch_key(self, ks): ...
    @property
    def namespace(self): ...
    def decompose(self, *args, **kwargs): ...
    def name(self): ...
    @property
    def overloadpacket(self): ...
    @property
    def op(self): ...
    @property
    def tags(self): ...

class TorchBindOpOverload(OpOverload):
    def __call__(self, /, *args, **kwargs): ...

class OpOverloadPacket:
    def __init__(self, qualified_op_name, op_name, op, overload_names) -> None: ...
    def __deepcopy__(self, memo: Incomplete | None = None): ...
    def __hash__(self): ...
    @property
    def op(self): ...
    def __getattr__(self, key): ...
    def __iter__(self): ...
    def __call__(self, /, *args, **kwargs): ...
    def overloads(self): ...

class _OpNamespace(types.ModuleType):
    name: Incomplete
    def __init__(self, name) -> None: ...
    def __iter__(self): ...
    def __getattr__(self, op_name): ...

class _PyOpNamespace(_OpNamespace):
    def __init__(self, name, ops) -> None: ...
    def __getattr__(self, name): ...

class _Ops(types.ModuleType):
    loaded_libraries: Incomplete
    def __init__(self) -> None: ...
    def __getattr__(self, name): ...
    def __iter__(self): ...
    def import_module(self, module) -> None: ...
    def load_library(self, path) -> None: ...

ops: _Ops
