import abc
import torch
from _typeshed import Incomplete
from torch._ops import OpOverload
from torch.utils._python_dispatch import TorchDispatchMode
from torch.utils.hooks import RemovableHandle
from typing import Any, Callable, Generator, Iterable, Literal, MutableMapping, NamedTuple, Sequence

__all__ = ['saved_tensors_hooks', 'save_on_cpu', 'disable_saved_tensors_hooks', 'register_multi_grad_hook', 'allow_mutation_on_saved_tensors', 'Node', 'GradientEdge', 'get_gradient_edge', 'increment_version']

class Node(abc.ABC, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def name(self) -> str: ...
    @property
    @abc.abstractmethod
    def next_functions(self) -> tuple[tuple[Node | None, int], ...]: ...
    @abc.abstractmethod
    def metadata(self) -> dict: ...
    @abc.abstractmethod
    def register_hook(self, fn: Callable[..., Any]) -> RemovableHandle: ...
    @abc.abstractmethod
    def register_prehook(self, fn: Callable[..., Any]) -> RemovableHandle: ...
    @classmethod
    def __subclasshook__(cls, subclass: type) -> bool: ...

class GradientEdge(NamedTuple):
    node: Node
    output_nr: int

def get_gradient_edge(tensor: torch.Tensor) -> GradientEdge: ...
def increment_version(tensor: torch.Tensor | Iterable[torch.Tensor]) -> None: ...

class saved_tensors_hooks:
    pack_hook: Incomplete
    unpack_hook: Incomplete
    def __init__(self, pack_hook: Callable[[torch.Tensor], Any], unpack_hook: Callable[[Any], torch.Tensor]) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, *args: object) -> None: ...

class save_on_cpu(saved_tensors_hooks):
    def __init__(self, pin_memory: bool = False, device_type: str = 'cuda') -> None: ...

def disable_saved_tensors_hooks(error_message: str) -> Generator[None, None, None]: ...

class _MultiHandle(RemovableHandle):
    handles: tuple[RemovableHandle, ...]
    def __init__(self, handles: tuple[RemovableHandle, ...]) -> None: ...
    def remove(self) -> None: ...

def register_multi_grad_hook(tensors: Sequence[torch.Tensor], fn: Callable[[Sequence[torch.Tensor | None]], None] | Callable[[torch.Tensor], None], *, mode: Literal['all', 'any'] = 'all') -> RemovableHandle: ...

class _Handle: ...

class _swap_with_cloned(saved_tensors_hooks):
    def __init__(self, ctx: _AllowMutationOnSavedContext) -> None: ...

class _CloneArgBeforeMutateMode(TorchDispatchMode):
    ctx: Incomplete
    def __init__(self, ctx: _AllowMutationOnSavedContext) -> None: ...
    def __torch_dispatch__(self, func: OpOverload, types: Iterable[type], args: tuple[Any, ...] = (), kwargs: dict[Any, Any] | None = None) -> Any: ...

class _AllowMutationOnSavedContext:
    cloned: MutableMapping[_Handle, torch.Tensor]
    original: MutableMapping[_Handle, torch.Tensor]
    tid_to_weakhandle: MutableMapping[_TID, _Handle]
    sid_to_tid: dict[_SID, set[_TID]]
    def __init__(self) -> None: ...
    def clear(self) -> None: ...

def allow_mutation_on_saved_tensors() -> Generator[_AllowMutationOnSavedContext, None, None]: ...
