from _typeshed import Incomplete
from contextlib import ContextDecorator as _ContextDecorator
from dataclasses import dataclass
from torch.autograd import ProfilerActivity, _ProfilerResult
from torch.autograd.profiler_util import EventList as EventList, FunctionEvent as FunctionEvent, MemRecordsAcc as MemRecordsAcc
from typing import Any, Iterable

__all__ = ['profile', 'record_function', 'emit_itt', 'emit_nvtx', 'load_nvprof', 'EnforceUnique', 'parse_nvprof_trace', 'KinetoStepTracker', 'EventList', 'FunctionEvent', 'MemRecordsAcc']

class _ContextDecorator:
    def __enter__(self) -> None: ...
    def __exit__(self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: types.TracebackType | None) -> None: ...
    def __call__(self, func): ...

@dataclass
class _ProfilerStats:
    profiling_window_duration_sec: float = ...
    number_of_events: int = ...
    profiler_prepare_call_duration_us: int = ...
    profiler_enable_call_duration_us: int = ...
    profiler_disable_call_duration_us: int = ...
    parse_kineto_call_duration_us: int = ...
    function_events_build_tree_call_duration_us: int = ...

class profile:
    enabled: bool
    use_cuda: Incomplete
    use_device: str | None
    entered: bool
    record_shapes: Incomplete
    with_flops: Incomplete
    profile_memory: Incomplete
    with_stack: Incomplete
    with_modules: Incomplete
    use_cpu: Incomplete
    acc_events: Incomplete
    experimental_config: Incomplete
    kineto_results: _ProfilerResult | None
    profiling_start_time_ns: int
    profiling_end_time_ns: int
    custom_trace_id_callback: Incomplete
    trace_id: str
    kineto_activities: Incomplete
    profiler_kind: Incomplete
    def __init__(self, enabled: bool = True, *, use_cuda: bool = False, use_device: Incomplete | None = None, record_shapes: bool = False, with_flops: bool = False, profile_memory: bool = False, with_stack: bool = False, with_modules: bool = False, use_kineto: bool = False, use_cpu: bool = True, experimental_config: Incomplete | None = None, acc_events: bool = False, custom_trace_id_callback: Incomplete | None = None) -> None: ...
    def default_trace_id(self): ...
    def create_trace_id(self): ...
    def config(self, create_trace_id: bool = False): ...
    def __enter__(self): ...
    def __exit__(self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: types.TracebackType | None): ...
    @property
    def function_events(self): ...
    def table(self, sort_by: Incomplete | None = None, row_limit: int = 100, max_src_column_width: int = 75, max_name_column_width: int = 55, max_shapes_column_width: int = 80, header: Incomplete | None = None, top_level_events_only: bool = False): ...
    def export_chrome_trace(self, path): ...
    def export_stacks(self, path: str, metric: str = 'self_cpu_time_total'): ...
    def toggle_collection_dynamic(self, enabled: bool, activities: Iterable[ProfilerActivity]): ...
    def key_averages(self, group_by_input_shape: bool = False, group_by_stack_n: int = 0): ...
    def total_average(self): ...
    @property
    def self_cpu_time_total(self): ...

class record_function(_ContextDecorator):
    name: str
    args: str | None
    run_callbacks_on_exit: bool
    record: Incomplete
    def __init__(self, name: str, args: str | None = None) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any): ...

class emit_itt:
    enabled: Incomplete
    entered: bool
    record_shapes: Incomplete
    def __init__(self, enabled: bool = True, record_shapes: bool = False) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: types.TracebackType | None): ...

class emit_nvtx:
    enabled: Incomplete
    entered: bool
    record_shapes: Incomplete
    def __init__(self, enabled: bool = True, record_shapes: bool = False) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: types.TracebackType | None): ...

def load_nvprof(path): ...

class EnforceUnique:
    seen: Incomplete
    def __init__(self) -> None: ...
    def see(self, *key) -> None: ...

def parse_nvprof_trace(path): ...

class KinetoStepTracker:
    @classmethod
    def init_step_count(cls, requester: str): ...
    @classmethod
    def erase_step_count(cls, requester: str) -> bool: ...
    @classmethod
    def increment_step(cls, requester: str) -> int: ...
    @classmethod
    def current_step(cls) -> int: ...
