from _typeshed import Incomplete
from collections import defaultdict
from collections.abc import Generator
from torch.autograd import DeviceType
from typing import Any, NamedTuple

__all__ = ['EventList', 'FormattedTimesMixin', 'Interval', 'Kernel', 'FunctionEvent', 'FunctionEventAvg', 'StringTable', 'MemRecordsAcc']

class EventList(list):
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def self_cpu_time_total(self): ...
    def table(self, sort_by: Incomplete | None = None, row_limit: int = 100, max_src_column_width: int = 75, max_name_column_width: int = 55, max_shapes_column_width: int = 80, header: Incomplete | None = None, top_level_events_only: bool = False): ...
    def export_chrome_trace(self, path) -> None: ...
    def supported_export_stacks_metrics(self): ...
    def export_stacks(self, path: str, metric: str): ...
    def key_averages(self, group_by_input_shapes: bool = False, group_by_stack_n: int = 0): ...
    def total_average(self): ...

class FormattedTimesMixin:
    cpu_time_str: Incomplete
    device_time_str: Incomplete
    cpu_time_total_str: Incomplete
    device_time_total_str: Incomplete
    self_cpu_time_total_str: Incomplete
    self_device_time_total_str: Incomplete
    @property
    def cpu_time(self): ...
    @property
    def device_time(self): ...
    @property
    def cuda_time(self): ...

class Interval:
    start: Incomplete
    end: Incomplete
    def __init__(self, start, end) -> None: ...
    def elapsed_us(self): ...

class Kernel(NamedTuple):
    name: Incomplete
    device: Incomplete
    duration: Incomplete

class FunctionEvent(FormattedTimesMixin):
    id: int
    node_id: int
    name: str
    trace_name: str
    time_range: Interval
    thread: int
    fwd_thread: int | None
    kernels: list[Kernel]
    count: int
    cpu_children: list[FunctionEvent]
    cpu_parent: FunctionEvent | None
    input_shapes: tuple[int, ...]
    concrete_inputs: list[Any]
    kwinputs: dict[str, Any]
    stack: list
    scope: int
    use_device: str | None
    cpu_memory_usage: int
    device_memory_usage: int
    is_async: bool
    is_remote: bool
    sequence_nr: int
    device_type: DeviceType
    device_index: int
    device_resource_id: int
    is_legacy: bool
    flops: int | None
    is_user_annotation: bool | None
    self_cpu_percent: int
    total_cpu_percent: int
    total_device_percent: int
    def __init__(self, id, name, thread, start_us, end_us, fwd_thread: Incomplete | None = None, input_shapes: Incomplete | None = None, stack: Incomplete | None = None, scope: int = 0, use_device: Incomplete | None = None, cpu_memory_usage: int = 0, device_memory_usage: int = 0, is_async: bool = False, is_remote: bool = False, sequence_nr: int = -1, node_id: int = -1, device_type=..., device_index: int = 0, device_resource_id: Incomplete | None = None, is_legacy: bool = False, flops: Incomplete | None = None, trace_name: Incomplete | None = None, concrete_inputs: Incomplete | None = None, kwinputs: Incomplete | None = None, is_user_annotation: bool = False) -> None: ...
    def append_kernel(self, name, device, duration) -> None: ...
    def append_cpu_child(self, child) -> None: ...
    def set_cpu_parent(self, parent) -> None: ...
    @property
    def self_cpu_memory_usage(self): ...
    @property
    def self_device_memory_usage(self): ...
    @property
    def self_cuda_memory_usage(self): ...
    @property
    def cpu_time_total(self): ...
    @property
    def self_cpu_time_total(self): ...
    @property
    def device_time_total(self): ...
    @property
    def cuda_time_total(self): ...
    @property
    def self_device_time_total(self): ...
    @property
    def self_cuda_time_total(self): ...
    @property
    def key(self): ...

class FunctionEventAvg(FormattedTimesMixin):
    key: str | None
    count: int
    node_id: int
    is_async: bool
    is_remote: bool
    use_device: str | None
    cpu_time_total: int
    device_time_total: int
    self_cpu_time_total: int
    self_device_time_total: int
    input_shapes: list[list[int]] | None
    stack: list | None
    scope: int | None
    cpu_memory_usage: int
    device_memory_usage: int
    self_cpu_memory_usage: int
    self_device_memory_usage: int
    cpu_children: list[FunctionEvent] | None
    cpu_parent: FunctionEvent | None
    device_type: DeviceType
    is_legacy: bool
    flops: int
    def __init__(self) -> None: ...
    is_user_annotation: Incomplete
    def add(self, other): ...
    def __iadd__(self, other): ...

class StringTable(defaultdict):
    def __missing__(self, key): ...

class MemRecordsAcc:
    def __init__(self, mem_records) -> None: ...
    def in_interval(self, start_us, end_us) -> Generator[Incomplete]: ...

# Names in __all__ with no definition:
#   Kernel
