from .memory import *
from .random import *
import torch
import torch._C
from . import amp as amp, jiterator as jiterator, nvtx as nvtx, profiler as profiler, sparse as sparse, tunable as tunable
from .graphs import CUDAGraph as CUDAGraph, graph as graph, graph_pool_handle as graph_pool_handle, is_current_stream_capturing as is_current_stream_capturing, make_graphed_callables as make_graphed_callables
from .streams import Event as Event, ExternalStream as ExternalStream, Stream as Stream
from _typeshed import Incomplete
from torch.storage import _LegacyStorage
from torch.types import Device
from typing import Any

__all__ = ['BFloat16Storage', 'BFloat16Tensor', 'BoolStorage', 'BoolTensor', 'ByteStorage', 'ByteTensor', 'CharStorage', 'CharTensor', 'ComplexDoubleStorage', 'ComplexFloatStorage', 'DoubleStorage', 'DoubleTensor', 'FloatStorage', 'FloatTensor', 'HalfStorage', 'HalfTensor', 'IntStorage', 'IntTensor', 'LongStorage', 'LongTensor', 'ShortStorage', 'ShortTensor', 'CUDAGraph', 'CudaError', 'DeferredCudaCallError', 'Event', 'ExternalStream', 'Stream', 'StreamContext', 'amp', 'caching_allocator_alloc', 'caching_allocator_delete', 'caching_allocator_enable', 'can_device_access_peer', 'check_error', 'cudaStatus', 'cudart', 'current_blas_handle', 'current_device', 'current_stream', 'default_generators', 'default_stream', 'device', 'device_count', 'device_memory_used', 'device_of', 'empty_cache', 'get_allocator_backend', 'CUDAPluggableAllocator', 'change_current_allocator', 'get_arch_list', 'get_device_capability', 'get_device_name', 'get_device_properties', 'get_gencode_flags', 'get_per_process_memory_fraction', 'get_rng_state', 'get_rng_state_all', 'get_sync_debug_mode', 'graph', 'graph_pool_handle', 'graphs', 'has_half', 'has_magma', 'init', 'initial_seed', 'ipc_collect', 'is_available', 'is_bf16_supported', 'is_current_stream_capturing', 'is_initialized', 'jiterator', 'list_gpu_processes', 'make_graphed_callables', 'manual_seed', 'manual_seed_all', 'max_memory_allocated', 'max_memory_cached', 'max_memory_reserved', 'mem_get_info', 'memory', 'memory_allocated', 'memory_cached', 'memory_reserved', 'memory_snapshot', 'memory_stats', 'memory_stats_as_nested_dict', 'memory_summary', 'memory_usage', 'MemPool', 'MemPoolContext', 'use_mem_pool', 'temperature', 'power_draw', 'clock_rate', 'nccl', 'nvtx', 'profiler', 'random', 'reset_accumulated_memory_stats', 'reset_max_memory_allocated', 'reset_max_memory_cached', 'reset_peak_memory_stats', 'seed', 'seed_all', 'set_device', 'set_per_process_memory_fraction', 'set_rng_state', 'set_rng_state_all', 'set_stream', 'set_sync_debug_mode', 'sparse', 'stream', 'streams', 'synchronize', 'tunable', 'utilization']

has_half: bool
has_magma: bool
default_generators: tuple[torch._C.Generator]

def is_available() -> bool: ...
def is_bf16_supported(including_emulation: bool = True): ...
def is_initialized(): ...

class DeferredCudaCallError(Exception): ...

def init() -> None: ...
def cudart(): ...

class cudaStatus:
    SUCCESS: int
    ERROR_NOT_READY: int

class CudaError(RuntimeError):
    def __init__(self, code: int) -> None: ...

def check_error(res: int) -> None: ...

class _DeviceGuard:
    idx: Incomplete
    prev_idx: int
    def __init__(self, index: int) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, type: Any, value: Any, traceback: Any): ...

class device:
    idx: Incomplete
    prev_idx: int
    def __init__(self, device: Any) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, type: Any, value: Any, traceback: Any): ...

class device_of(device):
    def __init__(self, obj) -> None: ...

def set_device(device: _device_t) -> None: ...
def get_device_name(device: _device_t | None = None) -> str: ...
def get_device_capability(device: _device_t | None = None) -> tuple[int, int]: ...
def get_device_properties(device: _device_t | None = None) -> _CudaDeviceProperties: ...
def can_device_access_peer(device: _device_t, peer_device: _device_t) -> bool: ...

class StreamContext:
    cur_stream: torch.cuda.Stream | None
    stream: Incomplete
    idx: Incomplete
    src_prev_stream: Incomplete
    dst_prev_stream: Incomplete
    def __init__(self, stream: torch.cuda.Stream | None) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, type: Any, value: Any, traceback: Any): ...

def stream(stream: torch.cuda.Stream | None) -> StreamContext: ...
def set_stream(stream: Stream): ...
def device_count() -> int: ...
def get_arch_list() -> list[str]: ...
def get_gencode_flags() -> str: ...
def current_device() -> int: ...
def synchronize(device: _device_t = None) -> None: ...
def ipc_collect(): ...
def current_stream(device: _device_t | None = None) -> Stream: ...
def default_stream(device: _device_t | None = None) -> Stream: ...
def current_blas_handle(): ...
def set_sync_debug_mode(debug_mode: int | str) -> None: ...
def get_sync_debug_mode() -> int: ...
def device_memory_used(device: Device | int | None = None) -> int: ...
def memory_usage(device: Device | int | None = None) -> int: ...
def utilization(device: Device | int | None = None) -> int: ...
def temperature(device: Device | int | None = None) -> int: ...
def power_draw(device: Device | int | None = None) -> int: ...
def clock_rate(device: Device | int | None = None) -> int: ...

class _CudaBase:
    is_cuda: bool
    is_sparse: bool
    def type(self, *args, **kwargs): ...
    __new__: Incomplete

class _CudaLegacyStorage(_LegacyStorage):
    @classmethod
    def from_buffer(cls, *args, **kwargs) -> None: ...

class ByteStorage(_CudaLegacyStorage):
    def dtype(self): ...

class DoubleStorage(_CudaLegacyStorage):
    def dtype(self): ...

class FloatStorage(_CudaLegacyStorage):
    def dtype(self): ...

class HalfStorage(_CudaLegacyStorage):
    def dtype(self): ...

class LongStorage(_CudaLegacyStorage):
    def dtype(self): ...

class IntStorage(_CudaLegacyStorage):
    def dtype(self): ...

class ShortStorage(_CudaLegacyStorage):
    def dtype(self): ...

class CharStorage(_CudaLegacyStorage):
    def dtype(self): ...

class BoolStorage(_CudaLegacyStorage):
    def dtype(self): ...

class BFloat16Storage(_CudaLegacyStorage):
    def dtype(self): ...

class ComplexDoubleStorage(_CudaLegacyStorage):
    def dtype(self): ...

class ComplexFloatStorage(_CudaLegacyStorage):
    def dtype(self): ...

class _WrappedTritonKernel:
    kernel: Incomplete
    kernel_invoked: bool
    def __init__(self, kernel) -> None: ...
    def __call__(self, *args, **kwargs): ...

# Names in __all__ with no definition:
#   BFloat16Tensor
#   BoolTensor
#   ByteTensor
#   CUDAPluggableAllocator
#   CharTensor
#   DoubleTensor
#   FloatTensor
#   HalfTensor
#   IntTensor
#   LongTensor
#   MemPool
#   MemPoolContext
#   ShortTensor
#   caching_allocator_alloc
#   caching_allocator_delete
#   caching_allocator_enable
#   change_current_allocator
#   empty_cache
#   get_allocator_backend
#   get_per_process_memory_fraction
#   get_rng_state
#   get_rng_state_all
#   graphs
#   initial_seed
#   list_gpu_processes
#   manual_seed
#   manual_seed_all
#   max_memory_allocated
#   max_memory_cached
#   max_memory_reserved
#   mem_get_info
#   memory
#   memory_allocated
#   memory_cached
#   memory_reserved
#   memory_snapshot
#   memory_stats
#   memory_stats_as_nested_dict
#   memory_summary
#   nccl
#   random
#   reset_accumulated_memory_stats
#   reset_max_memory_allocated
#   reset_max_memory_cached
#   reset_peak_memory_stats
#   seed
#   seed_all
#   set_per_process_memory_fraction
#   set_rng_state
#   set_rng_state_all
#   streams
#   use_mem_pool
