import enum
import torch
import traceback
from _typeshed import Incomplete
from dataclasses import dataclass, field
from torch.utils._python_dispatch import TorchDispatchMode as TorchDispatchMode
from typing import Any, Iterator, TypeVar

DEFAULT_STREAM_ID: int
TK = TypeVar('TK')
TVa = TypeVar('TVa')
TVb = TypeVar('TVb')
DataPtr = int
StreamId = int
EventId = int
SeqNum = int
logger: Incomplete
FACTORY_FUNCTION_REGEX: Incomplete

class AccessType(enum.Enum):
    READ = ...
    WRITE = ...

@dataclass
class Access:
    type: AccessType
    seq_num: SeqNum
    stream: StreamId
    operator: str
    aliases: list[str]
    is_output: bool
    stack_trace: traceback.StackSummary

class SynchronizationError(Exception): ...

class UnsynchronizedAccessError(SynchronizationError):
    data_ptr: Incomplete
    allocation_stack_trace: Incomplete
    current_access: Incomplete
    previous_access: Incomplete
    def __init__(self, data_ptr: DataPtr, allocation_stack_trace: traceback.StackSummary | None, current_access: Access, previous_access: Access) -> None: ...

class CUDASanitizerErrors(Exception):
    errors: Incomplete
    def __init__(self, errors: list[SynchronizationError]) -> None: ...

@dataclass
class TensorInfo:
    allocation_stack_trace: traceback.StackSummary | None
    reads: list[Access] = field(default_factory=list)
    write: Access | None = ...

class _TensorsAccessed:
    accesses: dict[DataPtr, TensorInfo]
    def __init__(self) -> None: ...
    def ensure_tensor_exists(self, data_ptr: DataPtr) -> None: ...
    def ensure_tensor_does_not_exist(self, data_ptr: DataPtr) -> None: ...
    def create_tensor(self, data_ptr: DataPtr, stack_trace: traceback.StackSummary | None) -> None: ...
    def delete_tensor(self, data_ptr: DataPtr) -> None: ...
    def were_there_reads_since_last_write(self, data_ptr: DataPtr) -> bool: ...
    def get_allocation_stack_trace(self, data_ptr: DataPtr) -> traceback.StackSummary | None: ...
    def get_write(self, data_ptr: DataPtr) -> Access | None: ...
    def get_reads(self, data_ptr: DataPtr) -> list[Access]: ...
    def add_read(self, data_ptr: DataPtr, access: Access) -> None: ...
    def set_write(self, data_ptr: DataPtr, access: Access) -> None: ...

class StreamSynchronizations:
    current_sync_states: dict[StreamId, dict[StreamId, SeqNum]]
    recorded_sync_states: dict[EventId, dict[StreamId, SeqNum]]
    host_sync_state: dict[StreamId, SeqNum]
    def __init__(self) -> None: ...
    def create_stream(self, stream: StreamId) -> None: ...
    def create_event(self, event: EventId) -> None: ...
    def delete_event(self, event: EventId) -> None: ...
    def update_seq_num(self, stream: StreamId, seq_num: SeqNum) -> None: ...
    def record_state(self, event: EventId, stream: StreamId) -> None: ...
    def stream_wait_for_event(self, stream: StreamId, event: EventId) -> None: ...
    def all_streams_wait_for_event(self, event: EventId) -> None: ...
    def all_streams_wait_for_stream(self, stream: StreamId) -> None: ...
    def sync_all_streams(self) -> None: ...
    def is_ordered_after(self, current_stream: StreamId, seq_num: SeqNum, other_stream: StreamId) -> bool: ...

class EventHandler:
    tensors_accessed: Incomplete
    syncs: Incomplete
    seq_num: SeqNum
    def __init__(self) -> None: ...

def zip_by_key(a: dict[TK, TVa], b: dict[TK, TVb]) -> Iterator[tuple[TK, TVa, TVb]]: ...
def zip_arguments(schema: torch.FunctionSchema, args: tuple[Any, ...], kwargs: dict[str, Any]) -> Iterator[tuple[torch.Argument, Any]]: ...

class ArgumentHandler:
    dataptrs_read: set[DataPtr]
    dataptrs_written: set[DataPtr]
    tensor_aliases: dict[DataPtr, list[str]]
    outputs: set[DataPtr]
    def __init__(self) -> None: ...
    def parse_inputs(self, schema: torch.FunctionSchema, args: tuple[Any, ...], kwargs: dict[str, Any], *, is_factory: bool) -> None: ...
    def parse_outputs(self, schema: torch.FunctionSchema, outputs: Any, *, is_factory: bool) -> None: ...

class CUDASanitizerDispatchMode(TorchDispatchMode):
    event_handler: Incomplete
    def __init__(self) -> None: ...
    def __torch_dispatch__(self, func, types, args=(), kwargs: Incomplete | None = None): ...

class CUDASanitizer:
    dispatch: Incomplete
    enabled: bool
    def __init__(self) -> None: ...
    def enable(self) -> None: ...
    def disable(self) -> None: ...
    def __del__(self) -> None: ...

def enable_cuda_sanitizer() -> None: ...

cuda_sanitizer: Incomplete
