import torch
from _typeshed import Incomplete
from torch._C import _MemPool, _MemPoolContext, _cuda_CUDAAllocator
from torch.types import Device
from typing import Any

__all__ = ['caching_allocator_alloc', 'caching_allocator_delete', 'caching_allocator_enable', 'get_per_process_memory_fraction', 'set_per_process_memory_fraction', 'empty_cache', 'memory_stats', 'memory_stats_as_nested_dict', 'reset_accumulated_memory_stats', 'reset_peak_memory_stats', 'reset_max_memory_allocated', 'reset_max_memory_cached', 'memory_allocated', 'max_memory_allocated', 'memory_reserved', 'max_memory_reserved', 'memory_cached', 'max_memory_cached', 'memory_snapshot', 'memory_summary', 'list_gpu_processes', 'mem_get_info', 'get_allocator_backend', 'CUDAPluggableAllocator', 'change_current_allocator', 'MemPool', 'MemPoolContext', 'use_mem_pool']

def caching_allocator_alloc(size, device: Device | int = None, stream: Incomplete | None = None): ...
def caching_allocator_delete(mem_ptr) -> None: ...
def caching_allocator_enable(value: bool = True) -> None: ...
def set_per_process_memory_fraction(fraction, device: Device | int = None) -> None: ...
def get_per_process_memory_fraction(device: Device | int = None) -> float: ...
def empty_cache() -> None: ...
def memory_stats(device: Device | int = None) -> dict[str, Any]: ...
def memory_stats_as_nested_dict(device: Device | int = None) -> dict[str, Any]: ...
def reset_accumulated_memory_stats(device: Device | int = None) -> None: ...
def reset_peak_memory_stats(device: Device | int = None) -> None: ...
def reset_max_memory_allocated(device: Device | int = None) -> None: ...
def reset_max_memory_cached(device: Device | int = None) -> None: ...
def memory_allocated(device: Device | int = None) -> int: ...
def max_memory_allocated(device: Device | int = None) -> int: ...
def memory_reserved(device: Device | int = None) -> int: ...
def max_memory_reserved(device: Device | int = None) -> int: ...
def memory_cached(device: Device | int = None) -> int: ...
def max_memory_cached(device: Device | int = None) -> int: ...
def memory_snapshot(): ...
def memory_summary(device: Device | int = None, abbreviated: bool = False) -> str: ...
def list_gpu_processes(device: Device | int = None) -> str: ...
def mem_get_info(device: Device | int = None) -> tuple[int, int]: ...
def get_allocator_backend() -> str: ...

class _CUDAAllocator:
    def __init__(self, allocator: torch._C._cuda_CUDAAllocator) -> None: ...
    def allocator(self): ...

class CUDAPluggableAllocator(_CUDAAllocator):
    def __init__(self, path_to_so_file: str, alloc_fn_name: str, free_fn_name: str) -> None: ...

def change_current_allocator(allocator: _CUDAAllocator) -> None: ...

class MemPoolContext(_MemPoolContext):
    def __init__(self, pool: _MemPool) -> None: ...
    @staticmethod
    def active_pool() -> _MemPool | None: ...

class MemPool(_MemPool):
    def __init__(self, allocator: _cuda_CUDAAllocator | None = None) -> None: ...
    @property
    def id(self) -> tuple[int, int]: ...
    @property
    def allocator(self) -> _cuda_CUDAAllocator | None: ...
    def use_count(self) -> int: ...
    def snapshot(self): ...

def use_mem_pool(pool: MemPool, device: Device | int = None): ...
