import threading
import torch
from numpy.typing import ArrayLike
from torch._C._distributed_c10d import Backend as C10dBackend
from torch.distributed.distributed_c10d import ProcessGroup

__all__ = ['init_device_mesh', 'DeviceMesh']

class _DeviceMeshStub: ...

class _MeshEnv(threading.local):
    mesh_stack: list[DeviceMesh]
    child_to_root_mapping: dict[DeviceMesh, DeviceMesh]
    mesh_dim_group_options: dict[int, tuple[str, C10dBackend.Options | None]]
    root_to_flatten_mapping: dict[DeviceMesh, dict[str, DeviceMesh]]
    flatten_name_to_root_dims: dict[DeviceMesh, dict[str, tuple[int, ...]]]
    def __init__(self) -> None: ...
    def get_current_mesh(self) -> DeviceMesh: ...
    def create_sub_mesh(self, device_mesh: DeviceMesh, submesh_dim_names: tuple[str, ...], submesh_dims: list[tuple[int, ...]]) -> DeviceMesh: ...
    def create_flatten_mesh(self, device_mesh: DeviceMesh, mesh_dim_name: str | None = None) -> DeviceMesh: ...
    def get_root_mesh(self, device_mesh: DeviceMesh) -> DeviceMesh: ...
    def get_root_mesh_dim(self, device_mesh: DeviceMesh) -> int | None: ...
    @staticmethod
    def num_devices_per_host(device_type: str) -> int: ...
    @staticmethod
    def num_hosts(device_type: str) -> int: ...
    def get_mesh_dim_by_name(self, device_mesh: DeviceMesh, mesh_dim_name: str) -> int: ...

class DeviceMesh:
    device_type: str
    mesh: torch.Tensor
    mesh_dim_names: tuple[str, ...] | None
    def __init__(self, device_type: str, mesh: torch.Tensor | ArrayLike, *, mesh_dim_names: tuple[str, ...] | None = None, _init_backend: bool = True) -> None: ...
    def __enter__(self) -> DeviceMesh: ...
    def __exit__(self, exc_type: type[BaseException] | None, exc_value: BaseException | None, exc_traceback: types.TracebackType | None) -> None: ...
    def __hash__(self): ...
    def __eq__(self, other: object) -> bool: ...
    def __getitem__(self, mesh_dim_names: str | tuple[str, ...]) -> DeviceMesh: ...
    def get_group(self, mesh_dim: int | str | None = None) -> ProcessGroup: ...
    def get_all_groups(self) -> list[ProcessGroup]: ...
    @staticmethod
    def from_group(group: ProcessGroup | list[ProcessGroup], device_type: str, mesh: torch.Tensor | ArrayLike | None = None, *, mesh_dim_names: tuple[str, ...] | None = None) -> DeviceMesh: ...
    def size(self, mesh_dim: int | None = None) -> int: ...
    @property
    def ndim(self) -> int: ...
    @property
    def shape(self) -> tuple[int, ...]: ...
    def get_rank(self) -> int: ...
    def get_local_rank(self, mesh_dim: int | str | None = None) -> int: ...
    def get_coordinate(self) -> list[int] | None: ...

def init_device_mesh(device_type: str, mesh_shape: tuple[int, ...], *, mesh_dim_names: tuple[str, ...] | None = None) -> DeviceMesh: ...
