import torch
from .constants import default_pg_timeout as default_pg_timeout
from _typeshed import Incomplete
from datetime import timedelta
from torch._C._distributed_c10d import AllToAllOptions as AllToAllOptions, AllreduceCoalescedOptions as AllreduceCoalescedOptions, AllreduceOptions as AllreduceOptions, BarrierOptions as BarrierOptions, BroadcastOptions as BroadcastOptions, DebugLevel as DebugLevel, GatherOptions as GatherOptions, PrefixStore as PrefixStore, ProcessGroup as ProcessGroup, ProcessGroupGloo as ProcessGroupGloo, ReduceOp as ReduceOp, ReduceOptions as ReduceOptions, ReduceScatterOptions as ReduceScatterOptions, ScatterOptions as ScatterOptions, Store as Store, Work as Work, get_debug_level as get_debug_level
from typing import Any, Callable, NamedTuple

__all__ = ['Backend', 'BackendConfig', 'GroupMember', 'P2POp', 'all_gather', 'all_gather_coalesced', 'all_gather_object', 'all_reduce', 'all_reduce_coalesced', 'all_to_all', 'all_to_all_single', 'barrier', 'batch_isend_irecv', 'broadcast', 'send_object_list', 'recv_object_list', 'broadcast_object_list', 'destroy_process_group', 'gather', 'gather_object', 'get_backend_config', 'get_backend', 'get_default_backend_for_device', 'get_rank', 'get_world_size', 'get_pg_count', 'group', 'init_process_group', 'irecv', 'is_gloo_available', 'is_initialized', 'is_mpi_available', 'is_backend_available', 'is_nccl_available', 'is_torchelastic_launched', 'is_ucc_available', 'is_xccl_available', 'isend', 'monitored_barrier', 'new_group', 'new_subgroups', 'new_subgroups_by_enumeration', 'recv', 'reduce', 'reduce_scatter', 'scatter', 'scatter_object_list', 'send', 'supports_complex', 'AllreduceCoalescedOptions', 'AllreduceOptions', 'AllToAllOptions', 'BarrierOptions', 'BroadcastOptions', 'GatherOptions', 'PrefixStore', 'ProcessGroup', 'ReduceOp', 'ReduceOptions', 'ReduceScatterOptions', 'ScatterOptions', 'Store', 'DebugLevel', 'get_debug_level', 'Work', 'default_pg_timeout', 'get_group_rank', 'get_global_rank', 'get_process_group_ranks', 'reduce_op', 'all_gather_into_tensor', 'reduce_scatter_tensor', 'get_node_local_rank', 'split_group', 'ProcessGroupGloo']

def supports_complex(reduceOp: ReduceOp) -> bool: ...

class Backend(str):
    UNDEFINED: str
    GLOO: str
    NCCL: str
    UCC: str
    MPI: str
    XCCL: str

    class _BackendPlugin(NamedTuple):
        creator_fn: Incomplete
        extended_api: Incomplete
    backend_list: Incomplete
    default_device_backend_map: dict[str, str]
    backend_capability: dict[str, list[str]]
    backend_type_map: dict[str, ProcessGroup.BackendType]
    def __new__(cls, name: str): ...
    @classmethod
    def register_backend(cls, name, func, extended_api: bool = False, devices: str | list[str] | None = None) -> None: ...

class BackendConfig:
    device_backend_map: dict[str, Backend]
    def __init__(self, backend: Backend) -> None: ...
    def get_device_backend_map(self) -> dict[str, Backend]: ...

class _reduce_op:
    __members__: Incomplete
    def __init__(self) -> None: ...
    def __getattribute__(self, key): ...

reduce_op: Incomplete

class P2POp:
    op: Incomplete
    tensor: Incomplete
    group: Incomplete
    peer: Incomplete
    tag: Incomplete
    group_peer: Incomplete
    def __init__(self, op: Callable, tensor: torch.Tensor, peer: int | None = None, group: ProcessGroup | None = None, tag: int = 0, group_peer: int | None = None) -> None: ...
    def __new__(cls, op: Callable, tensor: torch.Tensor, peer: int | None = None, group: ProcessGroup | None = None, tag: int = 0, group_peer: int | None = None): ...

class _CollOp:
    op: Incomplete
    tensor: Incomplete
    dst_tensor: Incomplete
    redop: Incomplete
    root: Incomplete
    def __init__(self, op: Callable, tensor: torch.Tensor, dst_tensor: torch.Tensor | None = None, redop: ReduceOp | None = None, root: int | None = None) -> None: ...

class _World:
    def __init__(self) -> None: ...
    @property
    def default_pg(self) -> ProcessGroup | None: ...
    @default_pg.setter
    def default_pg(self, value) -> None: ...
    @property
    def pg_map(self) -> dict[ProcessGroup, tuple[str, Store]]: ...
    @property
    def pg_names(self) -> dict[ProcessGroup, str]: ...
    @property
    def pg_group_ranks(self) -> dict[ProcessGroup, dict[int, int]]: ...
    @property
    def pg_backend_config(self) -> dict[ProcessGroup, str]: ...
    @property
    def group_count(self) -> int: ...
    @group_count.setter
    def group_count(self, value: int) -> None: ...
    @property
    def tags_to_pg(self) -> dict[str, list[ProcessGroup]]: ...
    @property
    def pg_to_tag(self) -> dict[ProcessGroup, str]: ...
    @property
    def pg_coalesce_state(self) -> dict[ProcessGroup, list[_CollOp]]: ...
    @property
    def pg_config_info(self) -> list[dict[str, Any]]: ...

class _WorldMeta(type):
    @property
    def WORLD(cls) -> ProcessGroup | None: ...
    @WORLD.setter
    def WORLD(cls, pg: ProcessGroup | None): ...

class group(metaclass=_WorldMeta): ...

class GroupMember(metaclass=_WorldMeta):
    NON_GROUP_MEMBER: int

def get_group_rank(group: ProcessGroup, global_rank: int) -> int: ...
def get_global_rank(group: ProcessGroup, group_rank: int) -> int: ...
def get_process_group_ranks(group: ProcessGroup) -> list[int]: ...
def is_mpi_available() -> bool: ...
def is_nccl_available() -> bool: ...
def is_gloo_available() -> bool: ...
def is_ucc_available() -> bool: ...
def is_xccl_available() -> bool: ...
def is_backend_available(backend: str) -> bool: ...
def is_initialized() -> bool: ...
def is_torchelastic_launched() -> bool: ...
def get_backend_config(group: ProcessGroup | None = None) -> str: ...
def get_backend(group: ProcessGroup | None = None) -> Backend: ...
def get_default_backend_for_device(device: str | torch.device) -> str: ...
def get_pg_count() -> int: ...
def get_node_local_rank(fallback_rank: int | None = None) -> int: ...
def init_process_group(backend: str | None = None, init_method: str | None = None, timeout: timedelta | None = None, world_size: int = -1, rank: int = -1, store: Store | None = None, group_name: str = '', pg_options: Any | None = None, device_id: torch.device | None = None) -> None: ...
def destroy_process_group(group: ProcessGroup | None = None): ...
def get_rank(group: ProcessGroup | None = None) -> int: ...
def get_world_size(group: ProcessGroup | None = None) -> int: ...
def isend(tensor: torch.Tensor, dst: int | None = None, group: ProcessGroup | None = None, tag: int = 0, group_dst: int | None = None) -> Work | None: ...
def irecv(tensor: torch.Tensor, src: int | None = None, group: ProcessGroup | None = None, tag: int = 0, group_src: int | None = None) -> Work | None: ...
def send(tensor: torch.Tensor, dst: int | None = None, group: ProcessGroup | None = None, tag: int = 0, group_dst: int | None = None) -> None: ...
def recv(tensor: torch.Tensor, src: int | None = None, group: ProcessGroup | None = None, tag: int = 0, group_src: int | None = None) -> int: ...

class _IllegalWork(Work):
    def __getattribute__(self, name) -> None: ...

class _CoalescingManager:
    works: list[Work]
    def __init__(self) -> None: ...
    def append(self, work: Work): ...
    def wait(self) -> None: ...

def batch_isend_irecv(p2p_op_list: list[P2POp]) -> list[Work]: ...
def broadcast(tensor: torch.Tensor, src: int | None = None, group: ProcessGroup | None = None, async_op: bool = False, group_src: int | None = None): ...
def all_reduce(tensor, op=..., group: Incomplete | None = None, async_op: bool = False): ...
def all_reduce_coalesced(tensors, op=..., group: Incomplete | None = None, async_op: bool = False): ...
def reduce(tensor: torch.Tensor, dst: int | None = None, op=..., group: ProcessGroup | None = None, async_op: bool = False, group_dst: int | None = None): ...
def all_gather_object(object_list, obj, group: Incomplete | None = None) -> None: ...
def gather_object(obj: Any, object_gather_list: list[Any] | None = None, dst: int | None = None, group: ProcessGroup | None = None, group_dst: int | None = None): ...
def send_object_list(object_list: list[Any], dst: int | None = None, group: ProcessGroup | None = None, device: torch.device | None = None, group_dst: int | None = None): ...
def recv_object_list(object_list: list[Any], src: int | None = None, group: ProcessGroup | None = None, device: torch.device | None = None, group_src: int | None = None): ...
def broadcast_object_list(object_list: list[Any], src: int | None = None, group: ProcessGroup | None = None, device: torch.device | None = None, group_src: int | None = None): ...
def scatter_object_list(scatter_object_output_list: list[Any], scatter_object_input_list: list[Any] | None = None, src: int | None = None, group: ProcessGroup | None = None, group_src: int | None = None): ...
def all_gather(tensor_list, tensor, group: Incomplete | None = None, async_op: bool = False): ...
def all_gather_into_tensor(output_tensor, input_tensor, group: Incomplete | None = None, async_op: bool = False): ...
def all_gather_coalesced(output_tensor_lists, input_tensor_list, group: Incomplete | None = None, async_op: bool = False): ...
def gather(tensor: torch.Tensor, gather_list: list[torch.Tensor] | None = None, dst: int | None = None, group: ProcessGroup | None = None, async_op: bool = False, group_dst: int | None = None): ...
def scatter(tensor: torch.Tensor, scatter_list: list[torch.Tensor] | None = None, src: int | None = None, group: ProcessGroup | None = None, async_op: bool = False, group_src: int | None = None): ...
def reduce_scatter(output, input_list, op=..., group: Incomplete | None = None, async_op: bool = False): ...
def reduce_scatter_tensor(output, input, op=..., group: Incomplete | None = None, async_op: bool = False): ...
def all_to_all_single(output, input, output_split_sizes: Incomplete | None = None, input_split_sizes: Incomplete | None = None, group: Incomplete | None = None, async_op: bool = False): ...
def all_to_all(output_tensor_list, input_tensor_list, group: Incomplete | None = None, async_op: bool = False): ...
def barrier(group: ProcessGroup | None = ..., async_op: bool = False, device_ids: Incomplete | None = None): ...
def monitored_barrier(group: ProcessGroup | None = ..., timeout: Incomplete | None = None, wait_all_ranks: bool = False): ...
def split_group(parent_pg: ProcessGroup | None = None, split_ranks: list | None = None, timeout: timedelta | None = None, pg_options: Any | None = None, group_desc: str | None = None) -> ProcessGroup | None: ...
def new_group(ranks: Incomplete | None = None, timeout: Incomplete | None = None, backend: Incomplete | None = None, pg_options: Incomplete | None = None, use_local_synchronization: bool = False, group_desc: Incomplete | None = None, device_id: torch.device | None = None): ...
def new_subgroups(group_size: Incomplete | None = None, group: Incomplete | None = None, timeout: Incomplete | None = None, backend: Incomplete | None = None, pg_options: Incomplete | None = None, group_desc: Incomplete | None = None): ...
def new_subgroups_by_enumeration(ranks_per_subgroup_list, timeout: Incomplete | None = None, backend: Incomplete | None = None, pg_options: Incomplete | None = None, group_desc: Incomplete | None = None): ...
