import io
import os
import torch
from .decomp_utils import CustomDecompTable as CustomDecompTable
from .dynamic_shapes import Constraint as Constraint, Dim as Dim, dims as dims
from .exported_program import ExportedProgram as ExportedProgram, ModuleCallEntry as ModuleCallEntry, ModuleCallSignature as ModuleCallSignature, default_decompositions as default_decompositions
from .graph_signature import ExportBackwardSignature as ExportBackwardSignature, ExportGraphSignature as ExportGraphSignature
from .unflatten import FlatArgsAdapter as FlatArgsAdapter, UnflattenedModule as UnflattenedModule, unflatten as unflatten
from torch._ops import OpOverload
from torch.fx.passes.infra.pass_base import PassResult
from typing import Any, Callable

__all__ = ['Constraint', 'Dim', 'ExportBackwardSignature', 'ExportGraphSignature', 'ExportedProgram', 'CustomDecompTable', 'ModuleCallEntry', 'ModuleCallSignature', 'default_decompositions', 'dims', 'export', 'export_for_training', 'export_for_inference', 'load', 'register_dataclass', 'save', 'unflatten', 'FlatArgsAdapter', 'UnflattenedModule']

PassType = Callable[[torch.fx.GraphModule], PassResult | None]

def export_for_training(mod: torch.nn.Module, args: tuple[Any, ...], kwargs: dict[str, Any] | None = None, *, dynamic_shapes: dict[str, Any] | tuple[Any] | list[Any] | None = None, strict: bool = True, preserve_module_call_signature: tuple[str, ...] = ()) -> ExportedProgram: ...
def export_for_inference(mod: torch.nn.Module, args: tuple[Any, ...], kwargs: dict[str, Any] | None = None, *, dynamic_shapes: dict[str, Any] | tuple[Any] | list[Any] | None = None, strict: bool = True, preserve_module_call_signature: tuple[str, ...] = (), decomp_table: dict['OpOverload', Callable | None] | None = None) -> ExportedProgram: ...
def export(mod: torch.nn.Module, args: tuple[Any, ...], kwargs: dict[str, Any] | None = None, *, dynamic_shapes: dict[str, Any] | tuple[Any] | list[Any] | None = None, strict: bool = True, preserve_module_call_signature: tuple[str, ...] = ()) -> ExportedProgram: ...
def save(ep: ExportedProgram, f: str | os.PathLike | io.BytesIO, *, extra_files: dict[str, Any] | None = None, opset_version: dict[str, int] | None = None) -> None: ...
def load(f: str | os.PathLike | io.BytesIO, *, extra_files: dict[str, Any] | None = None, expected_opset_version: dict[str, int] | None = None) -> ExportedProgram: ...
def register_dataclass(cls, *, serialized_type_name: str | None = None) -> None: ...
