import dataclasses
import sympy
import torch
import torch.utils._pytree as pytree
from .graph_signature import ArgumentSpec, ExportGraphSignature
from torch._export.verifier import Verifier
from torch._library.fake_class_registry import FakeScriptObject
from torch.export.decomp_utils import CustomDecompTable
from torch.fx.passes.infra.pass_base import PassResult
from typing import Any, Callable, Iterator

__all__ = ['ExportedProgram', 'ModuleCallEntry', 'ModuleCallSignature', 'default_decompositions']

PassType = Callable[[torch.fx.GraphModule], PassResult | None]

@dataclasses.dataclass
class ModuleCallSignature:
    inputs: list[ArgumentSpec]
    outputs: list[ArgumentSpec]
    in_spec: pytree.TreeSpec
    out_spec: pytree.TreeSpec
    forward_arg_names: list[str] | None = ...
    def replace_all_uses_with(self, original_node, new_node) -> None: ...

@dataclasses.dataclass
class ModuleCallEntry:
    fqn: str
    signature: ModuleCallSignature | None = ...

def default_decompositions() -> CustomDecompTable: ...

class ExportedProgram:
    def __init__(self, root: torch.nn.Module | dict[str, Any], graph: torch.fx.Graph, graph_signature: ExportGraphSignature, state_dict: dict[str, torch.Tensor | torch.nn.Parameter], range_constraints: dict[sympy.Symbol, Any], module_call_graph: list[ModuleCallEntry], example_inputs: tuple[tuple[Any, ...], dict[str, Any]] | None = None, constants: dict[str, torch.Tensor | FakeScriptObject | torch._C.ScriptObject] | None = None, *, verifiers: list[type[Verifier]] | None = None) -> None: ...
    @property
    def graph_module(self): ...
    @graph_module.setter
    def graph_module(self, value) -> None: ...
    @property
    def graph(self): ...
    @graph.setter
    def graph(self, value) -> None: ...
    @property
    def graph_signature(self): ...
    @graph_signature.setter
    def graph_signature(self, value) -> None: ...
    @property
    def state_dict(self): ...
    @state_dict.setter
    def state_dict(self, value) -> None: ...
    def parameters(self) -> Iterator[torch.nn.Parameter]: ...
    def named_parameters(self) -> Iterator[tuple[str, torch.nn.Parameter]]: ...
    def buffers(self) -> Iterator[torch.Tensor]: ...
    def named_buffers(self) -> Iterator[tuple[str, torch.Tensor]]: ...
    @property
    def range_constraints(self): ...
    @range_constraints.setter
    def range_constraints(self, value) -> None: ...
    @property
    def module_call_graph(self): ...
    @module_call_graph.setter
    def module_call_graph(self, value) -> None: ...
    @property
    def example_inputs(self): ...
    @example_inputs.setter
    def example_inputs(self, value) -> None: ...
    @property
    def call_spec(self): ...
    @call_spec.setter
    def call_spec(self, value) -> None: ...
    @property
    def verifier(self) -> Any: ...
    @verifier.setter
    def verifier(self, value) -> None: ...
    @property
    def dialect(self) -> str: ...
    @dialect.setter
    def dialect(self, value) -> None: ...
    @property
    def verifiers(self): ...
    @verifiers.setter
    def verifiers(self, value) -> None: ...
    @property
    def tensor_constants(self): ...
    @tensor_constants.setter
    def tensor_constants(self, value) -> None: ...
    @property
    def constants(self): ...
    @constants.setter
    def constants(self, value) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    def module(self) -> torch.nn.Module: ...
    def run_decompositions(self, decomp_table: dict[torch._ops.OperatorBase, Callable] | None = None) -> ExportedProgram: ...
    def validate(self) -> None: ...
