import abc
import torch
import torch.utils._pytree as pytree
from _typeshed import Incomplete
from dataclasses import dataclass
from enum import Enum
from torch.export.exported_program import ExportedProgram, ModuleCallSignature
from typing import Any

__all__ = ['FlatArgsAdapter', 'InterpreterModule', 'InterpreterModuleDispatcher', 'UnflattenedModule', 'unflatten']

class _AttrKind(Enum):
    PARAMETER = 'parameter'
    BUFFER = 'buffer'
    CONSTANT = 'constant'
    MODULE = 'module'

class InterpreterModule(torch.nn.Module):
    graph_module: torch.fx.GraphModule | None
    graph: Incomplete
    def __init__(self, graph: torch.fx.Graph) -> None: ...
    def forward(self, *args, **kwargs): ...
    arg_names: Incomplete
    def finalize(self) -> None: ...
    def print_readable(self, print_output: bool = True, include_stride: bool = False, include_device: bool = False, colored: bool = False): ...

class InterpreterModuleDispatcher(torch.nn.Module):
    def __init__(self, attrs: set[str], call_modules: list[InterpreterModule]) -> None: ...
    def forward(self, *args, **kwargs): ...
    def call_modules(self): ...
    def print_readable(self, print_output: bool = True, include_stride: bool = False, include_device: bool = False, colored: bool = False): ...

class FlatArgsAdapter(abc.ABC, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def adapt(self, target_spec: pytree.TreeSpec, input_spec: pytree.TreeSpec, input_args: list[Any]) -> list[Any]: ...

class UnflattenedModule(torch.nn.Module):
    graph_signature: Incomplete
    graph: Incomplete
    module_call_graph: Incomplete
    flat_args_adapter: Incomplete
    adapted: bool
    ivals: Incomplete
    range_constraints: Incomplete
    equality_constraints: list
    input_placeholders: Incomplete
    check_input_constraints: bool
    def __init__(self, export_module: ExportedProgram, flat_args_adapter: FlatArgsAdapter | None = None) -> None: ...
    def process_forward_inputs(self, *args, **kwargs): ...
    def forward(self, *args, **kwargs): ...
    def print_readable(self, print_output: bool = True, include_stride: bool = False, include_device: bool = False, colored: bool = False): ...

def unflatten(module: ExportedProgram, flat_args_adapter: FlatArgsAdapter | None = None) -> UnflattenedModule: ...

class _ModuleFrame:
    flat_graph: Incomplete
    nodes: Incomplete
    seen_nodes: Incomplete
    seen_modules: Incomplete
    seen_attrs: Incomplete
    created_modules: Incomplete
    parent: Incomplete
    module_stack: Incomplete
    module_id: Incomplete
    module_call_graph: Incomplete
    verbose: bool
    child_fqn: Incomplete
    module: torch.fx.GraphModule | UnflattenedModule | InterpreterModule
    ivals: Incomplete
    graph: Incomplete
    node_map: dict[torch.fx.Node, torch.fx.Node]
    node_to_placeholder: Incomplete
    parent_call_module: torch.fx.Node | None
    def __init__(self, flat_graph: torch.fx.Graph, nodes: tuple[torch.fx.Node, ...], seen_nodes, seen_modules, seen_attrs, created_modules, parent, module_stack: list[tuple[str, int]], module_id, module_call_graph: dict[str, ModuleCallSignature], module: torch.fx.GraphModule | UnflattenedModule | None = None) -> None: ...
    def add_placeholder(self, x) -> None: ...
    def copy_sym_call_function(self, x): ...
    def remap_input(self, x): ...
    def finalize_outputs(self): ...
    def copy_node(self, node) -> None: ...
    def run_outer(self) -> None: ...
    def print(self, *args, **kwargs) -> None: ...
    def run_from(self, node_idx): ...

@dataclass
class _SubmoduleEntry:
    parent_fqn: str
    parent_module: torch.nn.Module
    parent_call_module: torch.fx.Node
    fqn: str
    call_idx: int
    module: torch.nn.Module

class _IVals:
    fqns: Incomplete
    storage: Incomplete
    def __init__(self) -> None: ...
    def read(self, fqn, graph, node): ...
    def update(self, fqn, graph, node) -> None: ...
    def create(self, partitions, root_module) -> None: ...
