import torch.utils._pytree as pytree
from ._symbolic_trace import Tracer
from .graph_module import GraphModule
from .node import Argument, Node, Target
from _typeshed import Incomplete
from dataclasses import dataclass
from typing import Any, Callable, NamedTuple

__all__ = ['PythonCode', 'CodeGen', 'Graph']

TransformCodeFunc = Callable[[list[str]], list[str]]

class _CustomBuiltin(NamedTuple):
    import_str: str
    obj: Any

class _Namespace:
    def __init__(self) -> None: ...
    def create_name(self, candidate: str, obj: Any | None) -> str: ...
    def associate_name_with_obj(self, name: str, obj: Any): ...

@dataclass
class PythonCode:
    src: str
    globals: dict[str, Any]

class _InsertPoint:
    graph: Incomplete
    def __init__(self, graph, new_insert) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, type: type[BaseException] | None, value: BaseException | None, tb: types.TracebackType | None) -> None: ...

class _node_list:
    graph: Incomplete
    direction: Incomplete
    def __init__(self, graph: Graph, direction: str = '_next') -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self): ...
    def __reversed__(self): ...

class _PyTreeInfo(NamedTuple):
    orig_args: list[str]
    in_spec: pytree.TreeSpec
    out_spec: pytree.TreeSpec | None

@dataclass(frozen=True)
class _ParsedStackTrace:
    file: str
    lineno: str
    name: str
    code: str
    def get_summary_str(self): ...

class CodeGen:
    def __init__(self) -> None: ...
    def gen_fn_def(self, free_vars: list[str], maybe_return_annotation: str) -> str: ...
    def generate_output(self, output_args: Argument) -> str: ...
    def process_inputs(self, *args: Any) -> Any: ...
    def process_outputs(self, outputs: Any) -> Any: ...
    def additional_globals(self) -> list[tuple[str, Any]]: ...

class _PyTreeCodeGen(CodeGen):
    pytree_info: _PyTreeInfo
    def __init__(self, pytree_info: _PyTreeInfo) -> None: ...
    def process_inputs(self, *inputs: Any) -> Any: ...
    def process_outputs(self, out: Any) -> Any: ...
    def gen_fn_def(self, free_vars, maybe_return_annotation): ...
    def generate_output(self, output_args): ...

class _FindNodesLookupTable:
    table: dict[tuple[str, Target | None], dict[Node, None]]
    def __init__(self) -> None: ...
    def __contains__(self, node) -> bool: ...
    def insert(self, node: Node) -> None: ...
    def remove(self, node: Node) -> None: ...
    def find_nodes(self, *, op: str, target: Target | None = None): ...

class Graph:
    def __init__(self, owning_module: GraphModule | None = None, tracer_cls: type['Tracer'] | None = None, tracer_extras: dict[str, Any] | None = None) -> None: ...
    @property
    def owning_module(self): ...
    @owning_module.setter
    def owning_module(self, mod: GraphModule | None): ...
    @property
    def nodes(self) -> _node_list: ...
    def output_node(self) -> Node: ...
    def find_nodes(self, *, op: str, target: Target | None = None, sort: bool = True): ...
    def graph_copy(self, g: Graph, val_map: dict[Node, Node], return_output_node: bool = False) -> Argument | None: ...
    def __deepcopy__(self, memo: Incomplete | None = None) -> Graph: ...
    def create_node(self, op: str, target: Target, args: tuple['Argument', ...] | None = None, kwargs: dict[str, 'Argument'] | None = None, name: str | None = None, type_expr: Any | None = None) -> Node: ...
    def process_inputs(self, *args): ...
    def process_outputs(self, out): ...
    def erase_node(self, to_erase: Node) -> None: ...
    def inserting_before(self, n: Node | None = None): ...
    def inserting_after(self, n: Node | None = None): ...
    def placeholder(self, name: str, type_expr: Any | None = None, default_value: Any = ...) -> Node: ...
    def get_attr(self, qualified_name: str, type_expr: Any | None = None) -> Node: ...
    def call_module(self, module_name: str, args: tuple['Argument', ...] | None = None, kwargs: dict[str, 'Argument'] | None = None, type_expr: Any | None = None) -> Node: ...
    def call_method(self, method_name: str, args: tuple['Argument', ...] | None = None, kwargs: dict[str, 'Argument'] | None = None, type_expr: Any | None = None) -> Node: ...
    def call_function(self, the_function: Callable[..., Any], args: tuple['Argument', ...] | None = None, kwargs: dict[str, 'Argument'] | None = None, type_expr: Any | None = None) -> Node: ...
    def node_copy(self, node: Node, arg_transform: Callable[[Node], 'Argument'] = ...) -> Node: ...
    def output(self, result: Argument, type_expr: Any | None = None): ...
    def python_code(self, root_module: str, *, verbose: bool = False, include_stride: bool = False, include_device: bool = False, colored: bool = False) -> PythonCode: ...
    def print_tabular(self) -> None: ...
    def lint(self): ...
    def eliminate_dead_code(self, is_impure_node: Callable[[Node], bool] | None = None) -> bool: ...
    def set_codegen(self, codegen: CodeGen): ...
    def on_generate_code(self, make_transformer: Callable[[TransformCodeFunc | None], TransformCodeFunc]): ...
