import torch
from .graph import Graph
from torch._C import _NodeBase
from torch.fx.operator_schemas import ArgsKwargsPair
from typing import Any, Callable

__all__ = ['Node', 'map_arg', 'map_aggregate', 'has_side_effect']

Target = Callable[..., Any] | str

def has_side_effect(fn: Callable) -> Callable: ...

class Node(_NodeBase):
    graph: Graph
    name: str
    op: str
    target: Target
    users: dict['Node', None]
    type: Any | None
    meta: dict[str, Any]
    def __init__(self, graph: Graph, name: str, op: str, target: Target, args: tuple['Argument', ...], kwargs: dict[str, 'Argument'], return_type: Any | None = None) -> None: ...
    @property
    def next(self) -> Node: ...
    @property
    def prev(self) -> Node: ...
    def prepend(self, x: Node) -> None: ...
    def __gt__(self, other: Node) -> bool: ...
    def __lt__(self, other: Node) -> bool: ...
    def __ge__(self, other: Node) -> bool: ...
    def __le__(self, other: Node) -> bool: ...
    def append(self, x: Node) -> None: ...
    @property
    def args(self) -> tuple[Argument, ...]: ...
    @args.setter
    def args(self, a: tuple[Argument, ...]) -> None: ...
    @property
    def kwargs(self) -> dict[str, Argument]: ...
    @kwargs.setter
    def kwargs(self, k: dict[str, Argument]) -> None: ...
    @property
    def all_input_nodes(self) -> list['Node']: ...
    def update_arg(self, idx: int, arg: Argument) -> None: ...
    def insert_arg(self, idx: int, arg: Argument) -> None: ...
    def update_kwarg(self, key: str, arg: Argument) -> None: ...
    @property
    def stack_trace(self) -> str | None: ...
    @stack_trace.setter
    def stack_trace(self, trace: str | None) -> None: ...
    def format_node(self, placeholder_names: list[str] | None = None, maybe_return_typename: list[str] | None = None) -> str | None: ...
    def replace_all_uses_with(self, replace_with: Node, delete_user_cb: Callable[[Node], bool] = ..., *, propagate_meta: bool = False) -> list['Node']: ...
    def is_impure(self) -> bool: ...
    def normalized_arguments(self, root: torch.nn.Module, arg_types: tuple[Any] | None = None, kwarg_types: dict[str, Any] | None = None, normalize_to_only_use_kwargs: bool = False) -> ArgsKwargsPair | None: ...
    def replace_input_with(self, old_input: Node, new_input: Node) -> None: ...
    def __setattr__(self, name: str, value: Any) -> None: ...

def map_arg(a: Argument, fn: Callable[[Node], Argument]) -> Argument: ...
def map_aggregate(a: Argument, fn: Callable[[Argument], Argument]) -> Argument: ...
