from .graph import Graph
from .node import Argument, Node, Target
from _typeshed import Incomplete
from typing import Any, Callable, Iterator, OrderedDict

__all__ = ['TracerBase', 'GraphAppendingTracer', 'TraceError', 'Proxy', 'MetaProxy', 'Attribute', 'ParameterProxy', 'Scope', 'ScopeContextManager']

class Scope:
    module_path: Incomplete
    module_type: Incomplete
    def __init__(self, module_path: str, module_type: Any) -> None: ...

class ScopeContextManager:
    def __init__(self, scope: Scope, current_scope: Scope) -> None: ...
    def __enter__(self): ...
    def __exit__(self, *args) -> None: ...

class TracerBase:
    graph: Graph
    record_stack_traces: bool
    check_mutable_operations: bool
    trace_asserts: bool
    proxy_buffer_attributes: bool
    traced_func_name: str
    scope: Scope
    module_stack: OrderedDict[str, tuple[str, Any]]
    node_name_to_scope: dict[str, tuple[str, type]]
    def create_node(self, kind: str, target: Target, args: tuple[Argument, ...], kwargs: dict[str, Argument], name: str | None = None, type_expr: Any | None = None) -> Node: ...
    def proxy(self, node: Node) -> Proxy: ...
    def create_proxy(self, kind: str, target: Target, args: tuple[Any, ...], kwargs: dict[str, Any], name: str | None = None, type_expr: Any | None = None, proxy_factory_fn: Callable[[Node], 'Proxy'] = None): ...
    def create_arg(self, a: Any) -> Argument: ...
    def to_bool(self, obj: Proxy) -> bool: ...
    def iter(self, obj: Proxy) -> Iterator: ...
    def keys(self, obj: Proxy) -> Any: ...

class GraphAppendingTracer(TracerBase):
    graph: Incomplete
    scope: Incomplete
    module_stack: Incomplete
    node_name_to_scope: Incomplete
    def __init__(self, graph: Graph) -> None: ...

class TraceError(ValueError): ...

class Proxy:
    tracer: Incomplete
    node: Incomplete
    def __init__(self, node: Node, tracer: TracerBase | None = None) -> None: ...
    def __getattr__(self, k) -> Attribute: ...
    def __deepcopy__(self, memo) -> dict: ...
    def __call__(self, *args, **kwargs) -> Proxy: ...
    def __iter__(self) -> Iterator['Proxy']: ...
    def __abs__(self): ...
    def __bool__(self) -> bool: ...
    def keys(self): ...
    def __len__(self) -> int: ...
    @classmethod
    def __torch_function__(cls, orig_method, types, args: Incomplete | None = None, kwargs: Incomplete | None = None): ...

class MetaProxy(Proxy):
    fake_mode: Incomplete
    def __init__(self, node: Node, tracer: TracerBase | None = None, fake_mode: Incomplete | None = None) -> None: ...
    @classmethod
    def __torch_function__(cls, orig_method, types, args: Incomplete | None = None, kwargs: Incomplete | None = None): ...

class Attribute(Proxy):
    root: Incomplete
    attr: Incomplete
    tracer: Incomplete
    def __init__(self, root: Proxy, attr: str) -> None: ...
    @property
    def node(self): ...
    def __call__(self, *args, **kwargs): ...

class ParameterProxy(Proxy):
    param: Incomplete
    name: Incomplete
    def __init__(self, tracer: TracerBase, node: Node, name, param) -> None: ...
    @property
    def shape(self): ...
    def size(self): ...
    def dim(self): ...
    @property
    def ndim(self): ...
    def numel(self): ...
    def nelement(self): ...
